/*
* --------------------------------------------------------------------
*
*
* --------------------------------------------------------------------
*/

#define RTL8651B
#include <rtl8651b_cryptoEngine.h>
#include <linux/autoconf.h>
#include <asicRegs.h>
#include <rtl_glue.h>
#include <assert.h>


#define WRITE_MEM32(reg,val) REG32(reg)=val
#define READ_MEM32(reg) REG32(reg)

/*
Assumption:
1. UNCACHED_MALLOC can get 4-byte aligned memory block
*/
static volatile uint32 * desDescPtrArray;		//For DESDCR
static uint32 * desDescriptorArray;	//Each descriptor have 16-byte
static uint32 desDescNum, curDesDescPos;
static int8 * desKey, *desIv;
static int8 ** desBatchKey, **desBatchIv;
//Counters for interrupt
static uint32 cryptoDoneIntCounter, cryptoAllDoneIntCounter;


int32 rtl8651b_cryptoEngine_init(uint32 descNum, int8 mode32Bytes) {
	uint32 i;
	
	desDescNum = descNum;
	desDescPtrArray = (uint32 *)UNCACHED_MALLOC(descNum * sizeof(uint32));
	desDescriptorArray = (uint32 *)UNCACHED_MALLOC(descNum * 4 * sizeof(uint32));
	desKey = (int8 *)UNCACHED_MALLOC(24 * sizeof(int8));
	desIv = (int8 *)UNCACHED_MALLOC(8 * sizeof(int8));
	desBatchKey = (int8 **)UNCACHED_MALLOC(descNum * sizeof(int8 **));
	desBatchIv = (int8 **)UNCACHED_MALLOC(descNum * sizeof(int8 **));
	for(i=0; i<descNum; i++) {
		desDescPtrArray[i] = ((uint32)&desDescriptorArray[(i<<2)] ) & 0xfffffffc;//Clear last 2-bit
		desBatchKey[i] = (int8 *)UNCACHED_MALLOC(24 * sizeof(int8));
		desBatchIv[i] = (int8 *)UNCACHED_MALLOC(8 * sizeof(int8));
	}
	desDescPtrArray[descNum-1] |= DESC_WRAP;

	//Initial crypto-engine
	if(mode32Bytes == TRUE) {
		REG32(DESSCR) = 0x10400000;//Reset all descriptors, soft reset and Lexra bus burst size is 32 bytes
		REG32(DESSCR) = 0xd0000000;//Enable tx/rx and Lexra bus burst size is 32 bytes
	}
	else {
		REG32(DESSCR) = 0x00400000;//Reset all descriptors, soft reset and Lexra bus burst size is 16 bytes
		REG32(DESSCR) = 0xc0000000;//Enable tx/rx and Lexra bus burst size is 16 bytes
	}
	REG32(DESDCR) = (uint32)desDescPtrArray;

	REG32(DESEIMR) = 0;


	curDesDescPos = 0;

	return SUCCESS;
}


int32 rtl8651b_cryptoMemcpy(void *dest){
	uint32 destination= (uint32)dest & 0x0fffffff;
	WRITE_MEM32(TTLCR,(READ_MEM32(TTLCR)&0xf0000000)|((uint32)destination));
	//printk("DMA dest=%08x, ttlcr=%08x\n",(uint32) dest, (uint32)READ_MEM32(TTLCR)); 
	return 0;
}

//data, key and iv does not have 4-byte alignment limitatiuon
//mode bits:
// 0x01: 3DES
// 0x02: ECB
// 0x04: 
// 0x08: 
// 0x10: Generic DMA

 int32 rtl8651b_cryptoEngine_des(uint32 mode, int8 *data, uint32 len, int8 *key, int8 *iv ) {
	//printk("Mode: %x, Data: %08x, len:%d, Key:%08x, IV:%08x\n", mode, (uint32) data, len, (uint32) key, (uint32) iv);	
	if(len & 0x7)
		return FAILED;//Unable to process not 8-byte aligned data
	assert((desDescPtrArray[curDesDescPos] & DESC_OWNED_BIT) == DESC_RISC_OWNED);//Co-processor mode should guarantee function returned with RISC own descriptor
	desDescriptorArray[(curDesDescPos<<2)+0] = DES_RUNCIPHER | DES_DESCIV | (len & DES_DMA_LEN_MASK); //CTRL_LEN
	if(mode & DECRYPT_CBC_3DES) {
		desDescriptorArray[(curDesDescPos<<2)+0] |= DES_ALGO_3DES;
		if((mode &RTL8651_CRYPTO_GENERIC_DMA)==0)
			memcpy(desKey, key, 24);
	}
	else
		memcpy(desKey, key, 8);
	if(mode & DECRYPT_ECB_DES)
		desDescriptorArray[(curDesDescPos<<2)+0] |= DES_ECB;
	if((mode &RTL8651_CRYPTO_GENERIC_DMA)==0)
		memcpy(desIv, iv, 8);
	if(mode & ENCRYPT_CBC_DES)
		desDescriptorArray[(curDesDescPos<<2)+0] |= DES_ENCRYPTION;
	
	if(mode&RTL8651_CRYPTO_GENERIC_DMA){ //use crypto engine to do memcpy
		//printk("direct DMA\n");
		WRITE_MEM32(TTLCR,READ_MEM32(TTLCR)|(uint32)EN_MEM2MEM_DMA);
	}else{
		uint32 value;
		rtl8651b_cryptoMemcpy(data); //specify destination=source address
		value = READ_MEM32(TTLCR);
		value &=~EN_MEM2MEM_DMA;
		WRITE_MEM32(TTLCR,value);
	}
	desDescriptorArray[(curDesDescPos<<2)+1] = (uint32)data; //DMA_addr
	//assert(((uint32)desKey&0x3) == 0);
	desDescriptorArray[(curDesDescPos<<2)+2] = (uint32)desKey; //key_addr
	//assert(((uint32)desIv&0x3) == 0);
	desDescriptorArray[(curDesDescPos<<2)+3] = (uint32)desIv; //IV_addr is not used
	desDescPtrArray[curDesDescPos] |= DESC_ENG_OWNED;//Change owner to crypto-engine
#ifdef ISRMODEL
		{
			uint32 cryptoAllDoneIntPreCounter;
			cryptoAllDoneIntPreCounter = cryptoAllDoneIntCounter;
			REG32(DESEISR) = 0;//Clear interrupt status
			REG32(DESSCR) |= DES_FETCH; //Notify crypto-engine to get data to operate
			while((cryptoAllDoneIntCounter -cryptoAllDoneIntPreCounter)==0);
		}
#else
	REG32(DESEISR) = 0;//Clear interrupt status
	//printk("%02x trigger Crypto engine %08x=[%08x]\n", mode, (unsigned int)DESSCR, REG32(DESSCR) );
	REG32(DESSCR) |= DES_FETCH; //Notify crypto-engine to get data to operate
	//printk("ooo\n");	
	if(mode &RTL8651_CRYPTO_NON_BLOCKING) //non-blocking
		return SUCCESS;
	//printk("waiting...\n");
	while((REG32(DESEISR) & DES_OPALLDONE) != DES_OPALLDONE);//Wait until crypto-done
#endif
	while((desDescPtrArray[curDesDescPos] & DESC_OWNED_BIT) != DESC_RISC_OWNED);//Co-processor mode should guarantee function return with RISC own descriptor
	curDesDescPos = (curDesDescPos+1)%desDescNum;

	return SUCCESS;
}

 int32 rtl8651b_cryptoEngine_des_poll(int32 freq){
	uint32 poll=0, total=0;
	//if((REG32(DESEISR) & DES_OPALLDONE)== DES_OPALLDONE){
	//	if((desDescPtrArray[curDesDescPos] & DESC_OWNED_BIT) == DESC_RISC_OWNED){
	//		curDesDescPos = (curDesDescPos+1)%desDescNum;
	//		return SUCCESS;
	//	}
	//}
	while(1){
		poll++;
		if(poll>freq){
		  total+=poll;
		  if(((REG32(DESEISR) & DES_OPALLDONE) == DES_OPALLDONE)&&
		     ((desDescPtrArray[curDesDescPos] & DESC_OWNED_BIT) == DESC_RISC_OWNED))
		   	break;
		  poll=0;
		}
	}
	curDesDescPos = (curDesDescPos+1)%desDescNum;
	return -poll;
}
void rtl8651b_cryptoEngineGetIntCounter(uint32 * doneCounter, uint32 * allDoneCounter) {
	*doneCounter = cryptoDoneIntCounter;
	*allDoneCounter = cryptoAllDoneIntCounter;
}


