/*
* --------------------------------------------------------------------
*
*
* --------------------------------------------------------------------
*/

#ifndef RTL8651B_CRYPTOENGINE_H
#define RTL8651B_CRYPTOENGINE_H

#include <rtl_types.h>
#include <crypto.h>

int32 rtl8651b_cryptoEngine_init(uint32 descNum, int8 mode32Bytes);

/*
@func void	| rtl8651b_cryptoEngine_des	| envoke crypto engine
@parm uint32 	| mode		| Crypto mode
@parm int8 * 	| data	| Pointer to input data
@parm uint32 	| dataLen		| Length of input data
@parm int8 *		| key	| Pointer to key
@parm int8 *		| iv	| Pointer to iv
@rdesc None
@rvalue SUCCESS		| 	Encryption/decryption done as requested
@rvalue FAILED	| 	Invalid parameter
@comm User does not need to reverse the sequence of keys when applying for 3DES.
That is, when data is encrypted with keys A/B/C, decrypt it with A/B/C, instead of C/B/A.
 */
//Bit 0: 0:DES 1:3DES
//Bit 1: 0:CBC 1:ECB
//Bit 2: 0:Decrypt 1:Encrypt
#define DECRYPT_CBC_DES		0x00
#define DECRYPT_CBC_3DES		0x01
#define DECRYPT_ECB_DES		0x02
#define DECRYPT_ECB_3DES		0x03
#define ENCRYPT_CBC_DES		0x04
#define ENCRYPT_CBC_3DES		0x05
#define ENCRYPT_ECB_DES		0x06
#define ENCRYPT_ECB_3DES		0x07
#define RTL8651_CRYPTO_NON_BLOCKING	0x08
#define RTL8651_CRYPTO_GENERIC_DMA	0x10


//data, key and iv does not have 4-byte alignment limitatiuon
int32 rtl8651b_cryptoEngine_des(uint32 mode, int8 *data, uint32 len, int8 *key, int8 *iv );

#define rtl8651b_cryptoEngine_ecb_encrypt(input,len,key,encrypt) rtl8651b_cryptoEngine_des(encrypt?ENCRYPT_ECB_DES:DECRYPT_ECB_DES,input,len,key,NULL)
#define rtl8651b_cryptoEngine_3des_ecb_encrypt(input,len,key,encrypt) rtl8651b_cryptoEngine_des(encrypt?ENCRYPT_ECB_3DES:DECRYPT_ECB_3DES,input,len,key,NULL)
#define rtl8651b_cryptoEngine_cbc_encrypt(input,len,key,iv,encrypt) rtl8651b_cryptoEngine_des(encrypt?ENCRYPT_CBC_DES:DECRYPT_CBC_DES,input,len,key,iv)
#define rtl8651b_cryptoEngine_3des_cbc_encrypt(input,len,key,iv,encrypt) rtl8651b_cryptoEngine_des(encrypt?ENCRYPT_CBC_3DES:DECRYPT_CBC_3DES,input,len,key,iv)


void rtl8651b_cryptoEngineGetIntCounter(uint32 * doneCounter, uint32 * allDoneCounter);

#endif

