/*
* --------------------------------------------------------------------
*
*
* --------------------------------------------------------------------
*/

#include <desSim.h>
#include <des.h>

int32 desSim_ecb_encrypt(int8 *input, int8 * output, uint32 len, int8 * key, int8 encrypt) {
	uint32 idx;
	des_cblock in,out;
	des_key_schedule ks;

	if(encrypt != TRUE && encrypt != FALSE) //Only support either encrypt or decrypt operation
		return FAILED;
	
	for(idx = 0; idx<len; idx+=8) {
		des_set_key(key, ks);
		memcpy(in, &input[idx], 8);
		memset(out, 0,8);
		des_ecb_encrypt(&in,&out,ks,encrypt==TRUE? DES_ENCRYPT : DES_DECRYPT);
		memcpy(&output[idx], out, 8);
	}
	return SUCCESS;
}

int32 desSim_ede_ecb_encrypt(int8 *input, int8 * output, uint32 len, int8 * key, int8 encrypt) {
	uint32 idx;
	des_cblock in,out;
	des_key_schedule ks, ks2, ks3;

	if(encrypt != TRUE && encrypt != FALSE) //Only support either encrypt or decrypt operation
		return FAILED;

	for(idx = 0; idx<len; idx+=8) {
		if (des_set_key(&key[0], ks) != 0 || des_set_key(&key[8], ks2) != 0 || des_set_key(&key[16], ks3) != 0)
			return FAILED;
	
		memcpy(in, &input[idx], 8);
		memset(out, 0, 8);
		des_ecb3_encrypt((des_cblock *)&in,&out,&ks,&ks2, &ks3, encrypt==TRUE? DES_ENCRYPT : DES_DECRYPT);
		memcpy(&output[idx], out, 8);
	}
	return SUCCESS;
}

int32 desSim_cbc_encrypt(int8 *input, int8 * output, uint32 len, int8 * key, int8 * iv, int8 encrypt) {
	des_cblock iv3;
	des_key_schedule ks;

	if(encrypt != TRUE && encrypt != FALSE) //Only support either encrypt or decrypt operation
		return FAILED;

	if (des_set_key(&key[0], ks) != 0)
		return FAILED;
	
	memcpy(iv3, iv, 8);
	des_ncbc_encrypt(input, output, len, &ks, &iv3, encrypt==TRUE? DES_ENCRYPT : DES_DECRYPT);
	
	return SUCCESS;
}

int32 desSim_ede_cbc_encrypt(int8 *input, int8 * output, uint32 len, int8 * key, int8 * iv, int8 encrypt) {
	des_cblock iv3;
	des_key_schedule ks, ks2, ks3;

	if(encrypt != TRUE && encrypt != FALSE) //Only support either encrypt or decrypt operation
		return FAILED;

	if (des_set_key(&key[0], ks) != 0 || des_set_key(&key[8], ks2) != 0 || des_set_key(&key[16], ks3) != 0)
		return FAILED;
	
	memcpy(iv3, iv, 8);
	des_ede3_cbc_encrypt(input, output, len, &ks, &ks2, &ks3, &iv3, encrypt==TRUE? DES_ENCRYPT : DES_DECRYPT);
	
	return SUCCESS;
}

//IV is the first 8-byte of input and the real data length is len-8
int32 desSim_cbc_encryptEmbIV(int8 *input, int8 * output, uint32 len, int8 * key, int8 encrypt) {
	des_cblock iv3;
	des_key_schedule ks;

	if(encrypt != TRUE && encrypt != FALSE) //Only support either encrypt or decrypt operation
		return FAILED;

	if (des_set_key(&key[0], ks) != 0)
		return FAILED;
	
	memcpy(iv3, &input[0], 8);
	memcpy(&output[0], &input[0], 8);
	des_ncbc_encrypt(input+8, output+8, len-8, &ks, &iv3, encrypt==TRUE? DES_ENCRYPT : DES_DECRYPT);
	
	return SUCCESS;
}

//IV is the first 8-byte of input and the real data length is len-8
int32 desSim_ede_cbc_encryptEmbIV(int8 *input, int8 * output, uint32 len, int8 * key, int8 encrypt) {
	des_cblock iv3;
	des_key_schedule ks, ks2, ks3;

	if(encrypt != TRUE && encrypt != FALSE) //Only support either encrypt or decrypt operation
		return FAILED;

	if (des_set_key(&key[0], ks) != 0 || des_set_key(&key[8], ks2) != 0 || des_set_key(&key[16], ks3) != 0)
		return FAILED;
	
	memcpy(iv3, &input[0], 8);
	memcpy(&output[0], &input[0], 8);
	des_ede3_cbc_encrypt(input+8, output+8, len-8, &ks, &ks2, &ks3, &iv3, encrypt==TRUE? DES_ENCRYPT : DES_DECRYPT);
	
	return SUCCESS;
}


//data, key and iv does not have 4-byte alignment limitatiuon
int32 desSim_des(uint32 mode, int8 *input, int8 *output, uint32 len, int8 *key, int8 *iv ) {
	switch(mode) {
		case  SWDECRYPT_CBC_DES:
			return desSim_cbc_encrypt(input, output, len, key, iv, FALSE);
		case  SWDECRYPT_CBC_3DES:
			return desSim_ede_cbc_encrypt(input, output, len, key, iv, FALSE);
		case  SWDECRYPT_ECB_DES:
			return desSim_ecb_encrypt(input, output, len, key, FALSE);
		case  SWDECRYPT_ECB_3DES:
			return desSim_ede_ecb_encrypt(input, output, len, key, FALSE);
		case  SWENCRYPT_CBC_DES:
			return desSim_cbc_encrypt(input, output, len, key, iv, TRUE);
		case  SWENCRYPT_CBC_3DES:
			return desSim_ede_cbc_encrypt(input, output, len, key, iv, TRUE);
		case  SWENCRYPT_ECB_DES:
			return desSim_ecb_encrypt(input, output, len, key, TRUE);
		case  SWENCRYPT_ECB_3DES:
			return desSim_ede_ecb_encrypt(input, output, len, key, TRUE);
	}
	return FAILED;
}

int32 desSim_desBatch(uint32 mode, int8 ** input, int8 **output, uint32 *len, int8 **key, int8 **iv, uint32 dataNum){
	uint32 i;
	for(i=0; i<dataNum; i++) {
		if(desSim_des(mode, input[i], output[i], len[i], key[i], iv[i]) == FAILED)
			return FAILED;
	}
	return SUCCESS;
}

