/*
* --------------------------------------------------------------------
*
* --------------------------------------------------------------------
*/

#ifndef RTL8651B_AUTHENGINE_H
#define RTL8651B_AUTHENGINE_H

#include <rtl_types.h>

#define RTL8651B_MD5_DIGEST_LENGTH 		16
#define RTL8651B_SHA1_DIGEST_LENGTH		20

int32 rtl8651b_authEngine_init(uint32 descNum, int8 mode32Bytes);

/*
@func void	| rtl8651b_authEngine	| envoke authentication engine
@parm uint32 	| mode		| Authentication mode
@parm uint8 * 	| data	| Pointer to input data
@parm uint32 	| dataLen		| Length of input data
@parm uint8 *		| key	| Pointer to key
@parm uint32		| keyLen	| Length of key
@parm uint8 *		| digest	| Pointer to output digest
@rdesc None
@rvalue SUCCESS		| 	Authentication done as requested
@rvalue FAILED	| 	Invalid parameter
@comm This function will pad the data if necessary. User should input length of data without padding.
However, user must make sure that there is enough room for padding at the end of data.
 */
//Bit 0: 0:MD5 1:SHA1
//Bit 1: 0:Hash 1:HMAC
#define HASH_MD5		0x00
#define HASH_SHA1		0x01
#define HMAC_MD5		0x02
#define HMAC_SHA1		0x03
#define RTL8651_AUTH_NON_BLOCKING	0x08
#define RTL8651_AUTH_IOPAD_READY	0x00000100
int32 rtl8651b_authEngine(uint32 mode, uint8 * data, uint32 dataLen, uint8 * key, uint32 keyLen, uint8 * digest);

#define rtl8651b_authEngine_md5(data,dataLen,digest) rtl8651b_authEngine(HASH_MD5,data,dataLen,NULL,0,digest)
#define rtl8651b_authEngine_sha1(data,dataLen,digest) rtl8651b_authEngine(HASH_SHA1,data,dataLen,NULL,0,digest)
#define rtl8651b_authEngine_hmacMd5(data,dataLen,key,keyLen,digest) rtl8651b_authEngine(HMAC_MD5,data,dataLen,key,keyLen,digest)
#define rtl8651b_authEngine_hmacSha1(data,dataLen,key,keyLen,digest) rtl8651b_authEngine(HMAC_SHA1,data,dataLen,key,keyLen,digest)

#define RTL8651B_MAX_MD_CBLOCK 		64

//Management information counter
void rtl8651b_authEngineGetIntCounter(uint32 * doneCounter, uint32 * allDoneCounter);

#endif

