/*
*/
#ifndef RTL8651_FWDENGINE_H
#define RTL8651_FWDENGINE_H

/*********************************************************************************************************
	Forwarding Engine switches
**********************************************************************************************************/
#define _RTL_IP_FRAG			/* enable ip fragmenet */
#define _RTL_NAPT_FRAG			/* enable napt ip fragment process */
#define _RTL_NAPT_SHORT_CUT	/*enable napt short cut forward */
#define _RTL_DMZ 				/* enable Realtek define DMZ */
#define _RTL_BRIDGE 	 			/* enable Realtek's software bridge */
#define _RTL_UPNP 				/* enable Realtek's software UPNP IDG support */
#define _RTL_QUEUE_ARP_PACKET	/* enable Realtek's arp queue packet support */
#define _RTL_TRIGGER_PORT		/* enable Realtek's software Trigger Port support */
#define _RTL_DOS_PLUGIN		/* enable Realtek's software DOS plugin function */
#define _RTL_PORT_BOUNCING		/* enable Realtek's software DOS plugin function */
#define _RTL_PS_ACTION			/* enable Realtek's Protocol Stack Action function */
#define _RTL_NAPT_ADVANCED	/* enable Realtek's Advanced NAPT options */
#define _RTL_WEAK_NAPT_CTRL	/* enable Realtek's weak NAPT control function */

#ifdef _RTL_IP_FRAG
#define _RTL_QUEUE_FRAG_PKT	/* enable Realtek's fragment packet queue support */
#endif /* _RTL_IP_FRAG */
#define _RTL_NAPT_REDIRECT		/* enable Realtek's specific napt-outbound-flow trapping support */
#define _RTL_PS_FLOWCACHE		/* enable Realtek's procotocol stack flow cache  */
//#define _TONY_DEBUG // just for testing
#define FWDENG_TICKS_PER_SECOND 100
#if defined(CONFIG_RTL865X_PPTPL2TP)||defined(CONFIG_RTL865XB_PPTPL2TP)
#define _RTL_PPTPL2TP_HDR_REFILL	/* refill pptp/l2tp header for packet trapped to protocol stack */
#endif /* PPTPL2TP */
#define _RTL_TRAPPING_DISPATCH	/* trapping packet dispatch system */

#define DRAM_FWD					/* use DRAM */

/* forwarding engine return value statistic system */
#define _RTL8651_FWDENG_RETURN_STATISTIC
/*********************************************************************************************************
	Forwarding Engine error codes
**********************************************************************************************************/
#define FWDENG_DROP		0x1000

/* ACL related */
#define FWDENG_ACL_DROP_NOTIFY		1 	/* pkt dropped and a ICMP unreachable mesg sent to sender */
#define FWDENG_ACL_IN_USERTRAP		2	/* ingress pkt trapped to protocol stack */
#define FWDENG_ACL_IN_DROP			3	/* Pkt dropped by ingress ACL */
#define FWDENG_ACL_OUT_USERTRAP		4	/* egress pkt trapped to protocol stack */
#define FWDENG_ACL_OUT_DROP			5	/* Pkt dropped by egress ACL */

/* L2 related */
#define FWDENG_BROADCAST				10	/* L2 broadcast, send to protocol stack */
#define FWDENG_VLANINVALID			11	/* Invalid VLAN. Maybe not yet configured. */
#define FWDENG_VLANCFI					12	/* Vlan tagged pkt with CFI=1 */
#define FWDENG_PPPOE8863				13	/* PPPoE discovery phase pkt */
#define FWDENG_PPPOE8864				14	/* PPPoE session phase pkt */
#define FWDENG_ARP_NOTREQ				15	/* Not a ARP request */
#define FWDENG_ARP_OTHERS				16	/* ARP request to other host */
#define FWDENG_ARP_REQUEST			17	/* ARP request, Let protocol stack send reply. */
#define FWDENG_NOTIPPACKET			18	/* destined to gateway, but not IP packet. */
#define FWDENG_MACBLOCK				19
#define FWDENG_RTKPROTO				20	/* When EtherType field is 0x8899, this return value will appear*/
#define FWDENG_L2_CTRLMSG				21	/* Layer2 control message, IEEE 802.11D or IEEE 802.11Q reserved */

/* L3 - Non-IP */
#define FWDENG_NETIFACENOTFOUND		30	/* No layer3 network interface attached on receiving VLAN */
#define FWDENG_L34DISABLED			31	/* Layer3/4 operation is disabled. All such pkts goes to protocol stack. */
#define FWDENG_IGMPPKT				32	/* IGMP pkt not processed. Send to protocol stack */
#define FWDENG_GRE_IGNORE				33
#define FWDENG_ESP_IGNORE				34
#define FWDENG_RSVP_IGNORE			35
#define FWDENG_L3UNKNOWNPROTO		36	/* unknown L3 protocol */

/* IP */
#define FWDENG_IP_CANTFRAG					40	/* Can't handle IP fragment */
#define FWDENG_IP_WAN2WAN_ROUTING_LEGACY	41	/* pkt rcvd from WAN and should be routed to WAN but function disabled by user (legacy now) */
#define FWDENG_IP_CANTQUEUEFRAGMENT			42	/* Out of fragment queue space */
#define FWDENG_IP_BADCKSUM					43	/* IP cksum error */
#define FWDENG_IP_TTLEXCEED					44	/* TTL<2 and TTL-1 is required for routing */
#define FWDENG_IP_INVALID						45	/* invalid  IP in IP header */
#define FWDENG_IP_MULTICAST					46	/* IP multicast pkt.(not IGMP) */

/* L4 - misc */
#define FWDENG_L4_UNKNOWNPROTO			50	/* unsupported L4 protocol */
#define FWDENG_L4_NAPT_UNSUPPORTPROTO	51	/* must do NAPT but protocol is not TCP/UDP/ICMP */
#define FWDENG_L4_INVALIDPORT				52 	/* Src or dst Port number is 0 */
#define FWDENG_L4_NATNOENTRY				53	/* No matched entry in NAT table */
#define FWDENG_L4_PORTBOUNCING_LEGACY	54 	/* pkt rcvd from LAN destined to local server's public IP but function disabled. (legacy now) */

/* ICMP */
#define FWDENG_ICMP_OUT_ADDFAIL				60	/* out of ICMP entry space */
#define FWDENG_ICMP_OUT_INVALIDIPID			61	/* invalid ICMP ip or id */
#define FWDENG_ICMP_OUT_ERRFLOWNOTFOUND	62
#define FWDENG_ICMP_IN_ADDFAIL				63	/* can't add inbound ICMP echo req to DMZ host */
#define FWDENG_ICMP_IN_NOTFOUND  			64	/* inbound icmp flow not found */
#define FWDENG_ICMP_IN_ERRFLOWNOTFOUND		65	/* flow not found in inbound ICMP error mesg */
#define FWDENG_ICMP_BADCKSUM					66	/* bad icmp checksum */
#define FWDENG_ICMP_UNSUPPORTED				67

/* UDP */
#define FWDENG_UDP_BADCKSUM						70	/* bad udp cksum */
#define FWDENG_UDP_ECHOCHARGENBOMB				71	/* suspicous UDP echo/chargen bomb pkt. */
#define FWDENG_UDP_NAPT_OUT_NOTFOUND			72	/* Failed to find specified UDP flow in NAPT table */
#define FWDENG_UDP_NAPT_OUT_NOPORT				73	/* All usable port numbers exhausted. */
#define FWDENG_UDP_NAPT_OUT_ADDFAIL				74	/* Failed to add new UDP napt entry. Table full? */
#define FWDENG_UDP_NAPT_OUT_SVRALGADDFAIL		75	/* Failed to add new UDP server alg napt entry. Table full? */
#define FWDENG_UDP_NAPT_OUT_CLIALGADDFAIL		76	/* Failed to add new UDP client alg napt entry. Table full? */
#define FWDENG_UDP_NAPT_OUT_IGNORE				77
#define FWDENG_UDP_NAPT_IN_NOTFOUND			78
#define FWDENG_UDP_NAPT_IN_ALGADDFAIL			79
#define FWDENG_UDP_NAPT_IN_ADDFAIL				80
#define FWDENG_UDP_NAPT_IN_SERVERPORTNOTFOUND	81
#define FWDENG_UDP_NAPT_IN_IGNORE				82
#define FWDENG_UDP_NAT_IN_CLIALGADDFAIL 		83
#define FWDENG_UDP_NAT_IN_SVRALGADDFAIL 		84
#define FWDENG_UDP_NAT_IN_ALGNOTFOUND			85
#define FWDENG_UDP_NAT_IN_ALGIGNORE				86
#define FWDENG_UDP_NAT_OUT_ALGIGNORE			87
#define FWDENG_UDP_NAT_OUT_ALGNOTFOUND		88
#define FWDENG_UDP_NAT_OUT_CLIALGADDFAIL		89
#define FWDENG_UDP_NAT_OUT_SVRALGADDFAIL		90

/* TCP */
#define FWDENG_TCP_BADCKSUM						100
#define FWDENG_TCP_URLFILTERED					101
#define FWDENG_TCP_NAPT_OUT_CLIALGADDFAIL 		102	/* adding a existing ALG control flow or ALG list full */
#define FWDENG_TCP_NAPT_OUT_SVRALGADDFAIL 		103	/* adding a existing ALG control flow or ALG list full */
#define FWDENG_TCP_NAPT_OUT_ALGNOTFOUND		104
#define FWDENG_TCP_NAPT_OUT_NOIPPORT			105
#define FWDENG_TCP_NAPT_OUT_NOTFOUND			106
#define FWDENG_TCP_NAPT_OUT_ADDFAIL				107
#define FWDENG_TCP_NAPT_OUT_IGNORE				108
#define FWDENG_TCP_NAPT_IN_CLIALGADDFAIL		109
#define FWDENG_TCP_NAPT_IN_IGNORE				110
#define FWDENG_TCP_NAPT_IN_SVRALGADDFAIL		111
#define FWDENG_TCP_NAPT_IN_NOTFOUND				112
#define FWDENG_TCP_NAPT_IN_ADDFAIL				113
#define FWDENG_TCP_NAT_IN_ALGNOTFOUND			114
#define FWDENG_TCP_NAT_IN_ALGADDFAIL			115
#define FWDENG_TCP_NAT_IN_ALGIGNORE				116
#define FWDENG_TCP_NAT_OUT_ALGNOTFOUND			117
#define FWDENG_TCP_NAT_OUT_ALGIGNORE			118
#define FWDENG_TCP_NAT_OUT_ADDFAIL				119
#define FWDENG_TCP_NAT_OUT_ENTRYUPDATE			120

/* misc */
#define FWDENG_USER_IDSCHECKDROP			130
#define FWDENG_USER_DROP					131
#define FWDENG_TO_GW						132	/* packet must be trapped to gateway */
#define FWDENG_QUEUED						133
#define FWDENG_QUEUED_TO_GW				134
#define FWDENG_NOTXPORT					135
#define FWDENG_DECISION_DROP				136	/* decision table indicate this packet is illegal and need to be dropped */
#define FWDENG_DECISION_BLOCK				137	/* the forwarding decision of this packet is being blocked */
#define FWDENG_DEMANDRT_FOUND			138	/* demand route found, and the hooked callBack function is forked and forwarding is stopped */
#define FWDENG_RT_NOTFOUND				139	/* can not match any routing table entry */

#define FWDENG_TOTALERRS					140


/* Error Number for Protocol Stack Action Table */
#define FWDENG_EPSA_IPPORTRANGE_CONFLICT	1000	/* PS action table entry addition failed due to there's conflict with existing entries */
#define FWDENG_EPSA_TABLEFULL					1001	/* When adding PSA, PS action table is full */
#define FWDENG_EPSA_IPRANGENOTCORRECT		1002	/* Invalid PSA, lower IP is larger than upper IP in IP range arguments */
#define FWDENG_EPSA_ICMPARGISNOTZERO		1003	/* Invalid PSA, ICMP protocol with incorrect arguments */
#define FWDENG_EPSA_PORTRANGENOTCORRECT	1004	/* Invalid PSA, lower port number is larger than upper port number */
#define FWDENG_EPSA_DSTIPNOTEXTGWIP			1005	/* Invalid PSA, assigned dstIp is not gateway ip. */
#define FWDENG_EPSA_PORTRANGETOOLARGE		1006	/* Specified port range is larger than (legal port upper bound - fwdPort) */
#define FWDENG_EPSA_PSAENTRYNOTFOUND		1007	/* Specified PSA entry not found in PSA table */
#define FWDENG_EPSA_INVALIDPSACTION		1008	/* Invalid PS action */
#define FWDENG_EPSA_INVALIDPSROLE		1009	/* Invalid PS role */

/* forwarding engine internal control number : start from 30000 */

/* used by fwdEngineSend process */
#define FWDENG_SEND_PASSED				30000
#define FWDENG_SEND_GOTO_SEND			30001
#define FWDENG_SEND_GOTO_TOSWCORE		30002


/* forwarding engine: Protocol Stack Offloading Engine control number : start from 31000 */

/* used by _rtl8651_offloadEngineL3Send L3 offloading process */
#define OFFENG_EINVALIDINPUT			31000			/* input parameters fail */
#define OFFENG_EHEADROOM2SMALL		31001			/* offload packet without enough headroom */


/*********************************************************************************************************
	Forwarding Engine initial parameter's data structure
**********************************************************************************************************/
#define RTL8651_FWDENGINE_PARA_UNDEFINED_VALUE		0xffffffff	/* undefined value for forwarding engine parameters */
typedef struct rtl8651_fwdEngineInitPara_s {
	/* ip Fragment */
	uint32 MaxFragPktCnt;
	uint32 MaxFragSubPktCnt;
	uint32 MaxNegativeListEntryCnt;
	uint32 MaxPositiveListEntryCnt;
	uint32 MaxFragPoolCnt;
	uint32 MaxFragTimeOut;
	uint32 MaxNegativeListTimeOut;
	/* ip multicast */
	uint32 MCastDefaultVersion;				/* default igmp version supported by multicast system */
	uint32 MCastProcessOption;				/* set options for multicast system process */
	uint32 MaxMCastCnt;
	uint32 MaxMCastMemberCnt;
	uint32 MaxMCastTotalMemberCnt;
	uint32 MaxMCastTotalSourceCnt;
	uint32 MCastMember_ExpireTime;
	uint32 MCastUpstream_Timeout;
	uint32 MCastFilterEntryCnt;				/* Entry Count of Multicast Filter entry(for multicast group which need processed by protocol stack) */
	uint32 igmp_qi;
	uint32 igmp_qri;
	uint32 igmp_oqpi;
	uint32 igmp_sqc;
	uint32 igmp_sqi;
	uint32 igmp_lmqc;
	uint32 igmp_lmqi;
	uint32 igmp_gmi;							/* Group Member Interval */
	uint32 igmp_group_query_interval;
	uint32 igmp_response_tolerance_delay;
	/* dos */
	/* per-sip flood */
	uint32 dos_SynFlood_trackCnt;
	uint32 dos_StealthFin_trackCnt;
	uint32 dos_UdpFlood_trackCnt;
	uint32 dos_IcmpFlood_trackCnt;
	uint32 dos_total_trackCnt;
	/* port scan tracking */
	uint32 dos_total_portScan_trackCnt;
	/* per-sip connection control */
	uint32 dos_TcpConn_trackCnt;
	uint32 dos_UdpConn_trackCnt;
	uint32 dos_TcpUdpConn_trackCnt;
	uint32 dos_totalConn_trackCnt;
	/* dos proc : source ip blocking */
	uint32 dosProc_sipblock_count;
	/* pppoe NOT existed session termination */
	uint32 pppoeActiveSessionTrackingNumber;
	uint32 mbufHeadroom;//mbuf reserve headroom
	/* napt redirect */
#ifdef _RTL_NAPT_REDIRECT
	uint32 redirectTblSize;
	uint32 redirectFlowCacheSize;
#endif /* _RTL_NAPT_REDIRECT */
#ifdef _RTL_PPTPL2TP_HDR_REFILL
	uint32 pptpHdrCacheSize;
#endif /* _RTL_PPTPL2TP_HDR_REFILL */
#ifdef _RTL_PS_ACTION
	uint32 protoStackMaxActions;
#endif /* _RTL_PS_ACTION */
#ifdef _RTL_TRAPPING_DISPATCH
	uint32 trappingPktDispatchTblSize;
#endif /* _RTL_TRAPPING_DISPATCH */
	/* DNS process module */
	uint32 dns_dnsDB_processCnt;
	uint32 dns_dnsDB_tblSize;
	uint32 dns_dnsDB_cnameMapTblSize;
	uint32 dns_dnsDB_cnameMapTimeout;
	/* domain blocking */
	uint32 domainBlock_tblSize;
	uint32 domainBlock_maxBlockEntryCnt;
	uint32 domainBlock_blockEntryTimeout;

	/* trigger port */
#ifdef _RTL_TRIGGER_PORT
	uint32 triggerPortEntryCnt;	/* total entry count of trigger port entry */
	uint32 triggerPortCacheSize;	/* total cache count of trigger port flow */
#endif	/* _RTL_TRIGGER_PORT */

} rtl8651_fwdEngineInitPara_t;

/*********************************************************************************************************
	Forwarding Engine Initiation
**********************************************************************************************************/
int32 rtl8651_fwdEngineInit(struct rtl8651_fwdEngineInitPara_s *);
int32 rtl8651_fwdEngineReinit(void);

/*********************************************************************************************************
	Forwarding Engine/protocol stack Hooking API
**********************************************************************************************************/
int32 rtl8651_fwdEngineInput(void * pkthdr);
int32 rtl8651_fwdEngineExtPortRecv(void *id, uint8 *data,  uint32 len, uint16 myvid, uint32 myportmask, uint32 linkId);

/*********************************************************************************************************
	Forwarding Engine function switch
**********************************************************************************************************/
int32 rtl8651_fwdEngineDropPktCase(uint32 fwdengDropCase, int32 *flag);
int32 rtl8651_fwdEngineSend(void * pkthdr, int16 dvid, int32 iphdrOffset);
int32 rtl8651_fwdEngineQosSend(void *pkthdr, int16 dvid, int32 iphdrOffset, int32 qualityId);
int32 rtl8651_fwdEngineArp(int8 enable);
int32 rtl8651_fwdEngineIcmp(int8 enable);
int32 rtl8651_fwdEngineIcmpRoutingMsg(int8 enable);
int32 rtl8651_fwdEngineAlwaysReplyICMPErrForFragment(int8 enable);
int32 rtl8651_fwdEngineDMZHostIcmpPassThrough(int8 enable);
int32 rtl8651_fwdEngineArpProxy(int8 enable);
int32 rtl8651_fwdEngineProcessL34(int8 routingEnable);
int32 rtl8651_fwdEngineProcessIPFragment(int8 enable);
int32 rtl8651_fwdEngineProcessIPMulticast(int8 enable);
int32 rtl8651_fwdEnginePppoeNotExistedSessionTermination(int8 enable);
int32 rtl8651_fwdEngineProcessPortBouncing(int8 enable);
int32 rtl8651_fwdEngineProcessUpnpPortBouncing(int8 enable);
int32 rtl8651_fwdEngineEnableWDS(int32 flag);
int32 rtl8651_fwdEngineRemoveHostsOnExtLinkID(uint32 linkID);
int32 rtl8651_fwdEngineEnableWanRouting(int8 enable);
int32 rtl8651_fwdEngineFwdGeneralL4ToDMZ(int8 enable);
int32 rtl8651_fwdEngineFwdUnicastIGMPPkt(int8 enable);
int32 rtl8651_fwdEnginePortBouncingDoProtocolStackAction(int8 enable);
int32 rtl8651_fwdEngineTrapPktDemandRouteTrigger(int8 enable);

#ifdef _RTL_NAPT_ADVANCED
int32 rtl8651_fwdEngineInexactUdpFlow(int8 enable);
/* int32 rtl8651_fwdEngineInexactTcpFlow(int8 enable); */
int32 rtl8651_fwdEngineUdpStaticNaptPortTranslation(int8 enable);
int32 rtl8651_fwdEngineTcpStaticNaptPortTranslation(int8 enable);
#endif /* _RTL_NAPT_ADVANCED */

#ifdef _RTL_WEAK_NAPT_CTRL
int32 rtl8651_fwdEngineWeakTcpNaptProcess(int8 enable);
#endif /* _RTL_WEAK_NAPT_CTRL */

int32 rtl8651_fwdEnginePsDontTimeout(int enable);
int32 rtl8651_fwdEnginePsToWanErrMsgDontTimeout(int enable);

int32 rtl8651_fwdEngineDynamicRemoteHostPolicyRT(int enable);

/*********************************************************************************************************
	ALG
**********************************************************************************************************/
typedef int32 (*alg_funcptr)(struct rtl_pktHdr *pkthdr , void *ipheader, void *);
int32 rtl8651_fwdEngineRegisterAlgModule(uint16 ,  uint16 , 
	int32 	(*alg_init)(void *),
	alg_funcptr alg_outboundClientcb,
	alg_funcptr alg_inboundClientcb,
	alg_funcptr alg_outboundServercb,
	alg_funcptr alg_inboundServercb
); 
int32 rtl8651_fwdEngineRemoveAlgModule(uint16 ,  uint16 );
/* rtl8651_layer2fwd.c function declare for EXPORT*/

int32 rtl8651_fwdEngineAddWlanSTA(uint8 *smac,uint16 myvid, uint32 myportmask, uint32 linkId);
int32 rtl8651_fwdEngineDelWlanSTA(uint8 *mac,uint16 myvid);
int32 rtl8651_fwdEngineExtPortUcastFastRecv(struct rtl_pktHdr *pkt,uint16 myvid, uint32 myportmask);
/*********************************************************************************************************
	Counter
**********************************************************************************************************/
typedef struct rtl8651_tblDrvFwdEngineCounts_s {
	uint32 arpReceive, arpAddSuccess, arpAddFail, arpDelete, arpReply, arpGenerate;
	uint32 ipReceive, ipErrorReceive;
	uint32 icmpReceive, icmpErrorReceive, icmpReply, icmpReplyFailed, icmpDrop, icmpSend;
	uint32 igmpReceive, igmpErrorReceive;
	uint32 udpReceive, udpErrorReceive,udpSend;
	uint32 tcpReceive, tcpErrorReceive,tcpSend;
	uint32 l34actionLookupFailed, l34actionFailed;
	uint32 pppoeDiscoveryReceive, pppoeDiscoveryProcessed;
	uint32 pppoeSessionReceive, pppoeSessionProcessed;
	uint32 inAclDrop, inAclPermit,inAclCpu;
	uint32 outAclDrop, outAclPermit, outAclCpu;
	uint32 dmzDrop;
	uint32 natdrop, wlan2lanRelay, lan2wlanRelay;
	uint32 dosDrop;
	uint32 fastFragUdpMeetCount, fastFragUdpMissCount;
} rtl8651_tblDrvFwdEngineCounts_t;
int32 rtl8651_getFwdEngineCounter(rtl8651_tblDrvFwdEngineCounts_t * counter);

/*********************************************************************************************************
	UPnP
**********************************************************************************************************/
#ifdef _RTL_UPNP

#define UPNP_VALID				0x01
#define UPNP_TCP				0x02
#define UPNP_ONESHOT			0x04
#define UPNP_PERSIST			0x08
/* we reserve 3 bits for internal control */
#define UPNP_INTERNAL_FLAG		0xe0

void rtl8651_addUpnpMap(uint32 ,ipaddr_t , uint16 , ipaddr_t , uint16 , ipaddr_t , uint16 );
void rtl8651_addUpnpMapLeaseTime(uint32, ipaddr_t, uint16, ipaddr_t, uint16, ipaddr_t, uint16,uint32);
void rtl8651_delUpnpMap(uint32 ,ipaddr_t , uint16 , ipaddr_t , uint16 ,	ipaddr_t , uint16 );
uint32 rtl8651_queryUpnpMapTimeAge(uint32, ipaddr_t, uint16 , ipaddr_t, uint16);
#endif /* _RTL_UPNP */

/*********************************************************************************************************
	Trigger port
**********************************************************************************************************/
#ifdef _RTL_TRIGGER_PORT

typedef struct _rtl8651_triggerPort_param_s
{
	uint8 isStaticMapping;
	uint32 age;

	union {
		struct {
			uint8		__outType;
			ipaddr_t		__triggeringInsideLocalIpAddrStart;
			ipaddr_t		__triggeringInsideLocalIpAddrEnd;
			uint16		__outPortStart;
			uint16		__outPortEnd;
		} DYNAMIC_MAPPING;
		struct {
			ipaddr_t		__mappedInsideLocalIpAddr;
			uint8		__isStatic;
		} STATIC_MAPPING;
	} UN_MAPPING;

	#define	tp_outType							UN_MAPPING.DYNAMIC_MAPPING.__outType
	#define	tp_triggeringInsideLocalIpAddrStart	UN_MAPPING.DYNAMIC_MAPPING.__triggeringInsideLocalIpAddrStart
	#define	tp_triggeringInsideLocalIpAddrEnd	UN_MAPPING.DYNAMIC_MAPPING.__triggeringInsideLocalIpAddrEnd
	#define	tp_outPortStart						UN_MAPPING.DYNAMIC_MAPPING.__outPortStart
	#define	tp_outPortEnd						UN_MAPPING.DYNAMIC_MAPPING.__outPortEnd

	#define	tp_mappedInsideLocalIpAddr			UN_MAPPING.STATIC_MAPPING.__mappedInsideLocalIpAddr
	#define	tp_isStatic							UN_MAPPING.STATIC_MAPPING.__isStatic

}_rtl8651_triggerPort_param_t;



void rtl8651a_flushTriggerPortRules(uint32 netIntfId);
void rtl8651_flushTriggerPortRules(void);
int32 rtl8651a_addTriggerPortRule(uint32 netIntfId, uint8 inType, uint16 inPortStart, uint16 inPortEnd, uint8 outType,uint16 outPortStart,uint16 outPortEnd, ipaddr_t localIpAddr);
int32 rtl8651_addTriggerPortRule(uint8 inType, uint16 inPortStart, uint16 inPortEnd, uint8 outType,uint16 outPortStart,uint16 outPortEnd, ipaddr_t localIpAddr);

#endif /* _RTL_TRIGGER_PORT */

void rtl8651a_setUdpSizeThreshValue(uint32 dsid, uint32 value);

/*********************************************************************************************************
	Protocol stack action
**********************************************************************************************************/
#if 0
void rtl8651_flushProtoStackUsedUdpPorts(void);
void rtl8651_flushProtoStackServerUsedTcpPorts(void);
int32 rtl8651_addProtoStackUsedUdpPort(uint16  s_port);
int32 rtl8651_addProtoStackServerUsedTcpPort(uint16  s_port);
#endif
void rtl8651_flushProtoStackActions(void);
int32 rtl8651_addProtoStackActions(ipaddr_t ip,ipaddr_t mask,uint8 action,uint8 role,uint8 protocol,uint16 orgPort,uint16 fwdPort);

/*********************************************************************************************************
	plug-in
**********************************************************************************************************/
#if 0
typedef int32 (*user_ids_chkFunc)(struct rtl_pktHdr *);
int32 rtl8651_fwdEngineRegUserIDScheckFunc(user_ids_chkFunc func);
#endif

/*********************************************************************************************************
	udp blocking
**********************************************************************************************************/
void rtl8651a_setUdpSizeThreshValue(uint32 SessionID, uint32 value);

/*********************************************************************************************************
	trapping dispatch system
**********************************************************************************************************/
#ifdef _RTL_TRAPPING_DISPATCH

#define RTL8651_DEFAULT_TRAPPING_DISPATCH_CATEGORY		0	/* default category or trapping dispatch system */

int32 rtl8651_registerTrappingDispatchEntry(uint8 l4Proto, ipaddr_t srcIp, ipaddr_t dstIp, int32 srcPort, int32 dstPort, uint16 category);
int32 rtl8651_unregisterTrappingDispatchEntry(uint8 l4Proto, ipaddr_t srcIp, ipaddr_t dstIp, int32 srcPort, int32 dstPort);
#endif /* _RTL_TRAPPING_DISPATCH */
/*********************************************************************************************************
	napt redirect
**********************************************************************************************************/
#ifdef _RTL_NAPT_REDIRECT
int32 rtl8651_registerRedirectOutboundNaptFlow(
		uint8	isTcp,			/* TRUE: tcp, FALSE: udp */
		ipaddr_t	sip,				/* val 0 for wildcard */
		uint16	sport,			/* val 0 for wildcard */
		ipaddr_t	originalDip,		/* val 0 for wildcard */
		uint16	originalDport,	/* val 0 for wildcard */
		ipaddr_t	newDip,			/* val 0 for don't care, automatically assigned by gateway's LAN IP */
		int32	newDport		/* must assign a new Dport to protocol stack, if < 0, both 'newDip' and 'newDport' will not be set(original dip/dport are kept) */
);
int32 rtl8651_unregisterRedirectOutboundNaptFlow(
		uint8	isTcp,			/* TRUE: tcp, FALSE: udp */
		ipaddr_t	sip,				/* val 0 for wildcard */
		uint16	sport,			/* val 0 for wildcard */
		ipaddr_t	originalDip,		/* val 0 for wildcard */
		uint16	originalDport,	/* val 0 for wildcard */
		ipaddr_t	newDip,			/* val 0 for wildcard */
		int32	newDport		/* val 0 for wildcard */
);
int32 rtl8651_queryRedirectOutboundNaptFlow(
		uint8	isTcp,
		ipaddr_t	sip,
		uint16	sport,
		ipaddr_t	*pOriginalDip,
		uint16	*pOriginalDport,
		ipaddr_t	newDip,
		uint16	newDport
);
int32 rtl8651_registerRedirectOutboundNaptExceptionList(
		uint8	isTcp,			/* TRUE: tcp, FALSE: udp */
		ipaddr_t	sip,				/* val 0 for wildcard */
		uint16	sport,			/* val 0 for wildcard */
		ipaddr_t	dip,				/* val 0 for wildcard */
		uint16	dport			/* val 0 for wildcard */
);
int32 rtl8651_unregisterRedirectOutboundNaptExceptionList(
		uint8	isTcp,			/* TRUE: tcp, FALSE: udp */
		ipaddr_t	sip,				/* val 0 for wildcard */
		uint16	sport,			/* val 0 for wildcard */
		ipaddr_t	dip,				/* val 0 for wildcard */
		uint16	dport			/* val 0 for wildcard */
);
#endif /* _RTL_NAPT_REDIRECT */

/*********************************************************************************************************
	Counter
**********************************************************************************************************/
int32 rtl8651_updateCounter1( void );

/*********************************************************************************************************
	Offload
**********************************************************************************************************/
#define OFFLOAD_IP_CKSUM		0x0001
#define OFFLOAD_L4_CKSUM		0x0002
int32 rtl8651_offloadEngineL3Send(void * pPkt, uint32 L3PktLength, uint32 l34CkumOffload, int32 highPriority);


#endif /* RTL8651_FWDENGINE_H */
