/*
*
*/
#ifndef _TYPES_H
#define _TYPES_H

#include "rtl_types.h"


#ifndef NIPQUAD
#define NIPQUAD(addr) \
	((unsigned char *)&addr)[0], \
	((unsigned char *)&addr)[1], \
	((unsigned char *)&addr)[2], \
	((unsigned char *)&addr)[3]
#endif	

#ifndef NIPQUAD2STR
/* Example: printf( "intIp="NIPQUAD2STR"\n", NIPQUAD2(ntohl(intIp)) ); */
#define NIPQUAD2STR "%d.%d.%d.%d"
#define NIPQUAD2STR2 "%3d.%3d.%3d.%3d" /* aligned format */
#endif
#ifndef NIPQUAD2
#define NIPQUAD2(addr) \
	((addr&0xff000000)>>24), \
	((addr&0x00ff0000)>>16), \
	((addr&0x0000ff00)>> 8), \
	((addr&0x000000ff)>> 0)
#endif	

#define swapl32(x)\
        ((((x) & 0xff000000U) >> 24) | \
         (((x) & 0x00ff0000U) >>  8) | \
         (((x) & 0x0000ff00U) <<  8) | \
         (((x) & 0x000000ffU) << 24))
#define swaps16(x)        \
        ((((x) & 0xff00) >> 8) | \
         (((x) & 0x00ff) << 8))


//Get/Set host-order value from/to little endian buffer. 
//first byte ptr points to Little endian system's LSB.
#define GET_UINT16_LITTLE_ENDIAN_UNALIGNED( ptr )  ((uint16)  (*(((uint8*)(ptr))+1)<<8) | *((uint8*)(ptr)))
#define SET_UINT16_LITTLE_ENDIAN_UNALIGNED(  u16value, ptr )	do{\
		*((uint8*)(ptr)) =  (uint8)((u16value));/* get and set LSB */\
		*(((uint8*)(ptr))+1) =  (uint8)((u16value)>>8);/*get and set MSB*/\
}while(0)
#define GET_UINT32_LITTLE_ENDIAN_UNALIGNED( ptr )  ((uint32)  (*(((uint8*)(ptr))+3)<<24)|(*(((uint8*)(ptr))+2)<<16)|(*(((uint8*)(ptr))+1)<<8) | *((uint8*)(ptr)))
#define SET_UINT32_LITTLE_ENDIAN_UNALIGNED(  u32value, ptr )	do{\
		*((uint8*)(ptr)) =  (uint8)((u32value));/* get and set LSB */\
		*(((uint8*)(ptr))+1) =  (uint8)((u32value)>>8);\
		*(((uint8*)(ptr))+2) =  (uint8)((u32value)>>16);\
		*(((uint8*)(ptr))+3) =  (uint8)((u32value)>>24);/*get and set MSB*/\
}while(0)


#if 1
//Get/Set host-order value from/to big endian buffer. 
//first byte ptr points to Big endian system's the MSB
#define GET_UINT16_BIG_ENDIAN_UNALIGNED( ptr )  ((uint16)  (*((uint8*)(ptr))<<8) | *(((uint8*)(ptr))+1))
#define SET_UINT16_BIG_ENDIAN_UNALIGNED(  u16value, ptr )	do{\
	*((uint8*)(ptr)) = (uint8)(((uint16)u16value)>>8);  /*get and set MSB*/\
	*(((uint8*)(ptr))+1) = (uint8)(u16value); /* get and set LSB */\
}while(0)
#define GET_UINT32_BIG_ENDIAN_UNALIGNED( ptr )  ((uint32)  (*((uint8*)(ptr))<<24)|(*(((uint8*)(ptr))+1)<<16)|(*(((uint8*)(ptr))+2)<<8) | *(((uint8*)(ptr))+3))
#define SET_UINT32_BIG_ENDIAN_UNALIGNED( u32value, ptr )	do{\
	*((uint8*)(ptr)) =  (uint8)((u32value)>>24);/* get and set MSB */\
	*(((uint8*)(ptr))+1) =  (uint8)((u32value)>>16);\
	*(((uint8*)(ptr))+2) =  (uint8)((u32value)>>8);\
	*(((uint8*)(ptr))+3) =  (uint8)((u32value));/*get and set LSB*/\
}while(0)
#else
#define GET_UINT16_BIG_ENDIAN_UNALIGNED( ptr ) 			(*((uint16 *)(ptr)))
#define SET_UINT16_BIG_ENDIAN_UNALIGNED(  u16value, ptr ) 	(*((uint16 *)(ptr))) = (uint16)(u16value)
#define GET_UINT32_BIG_ENDIAN_UNALIGNED( ptr )			(*((uint32 *)(ptr)))
#define SET_UINT32_BIG_ENDIAN_UNALIGNED( u32value, ptr )	(*((uint32 *)(ptr))) = (uint32)(u32value)
#endif


#ifdef _LITTLE_ENDIAN

#ifndef __linux__
	#define ntohs(x)   (swaps16(x))
	#define ntohl(x)   (swapl32(x))
	#define htons(x)   (swaps16(x))
	#define htonl(x)   (swapl32(x))
#endif /* __linux__ */	
	#define NTOHL(d) ((d) = ntohl((d)))
	#define NTOHS(d) ((d) = ntohs((uint16)(d)))
	#define HTONL(d) ((d) = htonl((d)))
	#define HTONS(d) ((d) = htons((uint16)(d)))

	#define GET_UINT16_UNALIGNED( ptr )  				GET_UINT16_LITTLE_ENDIAN_UNALIGNED(ptr)
	#define SET_UINT16_UNALIGNED(  u16value, ptr )		SET_UINT16_LITTLE_ENDIAN_UNALIGNED(  u16value, ptr )
	#define GET_UINT32_UNALIGNED( ptr )  				GET_UINT32_LITTLE_ENDIAN_UNALIGNED(ptr)
	#define SET_UINT32_UNALIGNED(  u32value, ptr )		SET_UINT32_LITTLE_ENDIAN_UNALIGNED(  u32value,ptr)
	#define PKTGET_UINT16_UNALIGNED( ptr )  				GET_UINT16_BIG_ENDIAN_UNALIGNED(ptr)
	#define PKTSET_UINT16_UNALIGNED(  u16value, ptr 	)	SET_UINT16_BIG_ENDIAN_UNALIGNED(  u16value, ptr )
	#define PKTGET_UINT32_UNALIGNED( ptr )  				GET_UINT32_BIG_ENDIAN_UNALIGNED(ptr)
	#define PKTSET_UINT32_UNALIGNED(  u32value, ptr )	SET_UINT32_BIG_ENDIAN_UNALIGNED(  u32value,ptr)

#else /*Big endian system */

/* chhuang */
#ifdef __linux__
	#undef ntohs
	#undef ntohl
	#undef htons
	#undef htonl
	#define ntohs(x)   (x)
	#define ntohl(x)   (x) 
	#define htons(x)   (x)
	#define htonl(x)   (x)
	#define _LINUX_BYTEORDER_GENERIC_H
#endif /* __linux__ */	
	#define NTOHL(d) ((d) = ntohl(d))
	#define NTOHS(d) ((d) = ntohs((uint16)(d)))
	#define HTONL(d) ((d) = htonl(d))
	#define HTONS(d) ((d) = htons((uint16)(d)))
	#define GET_UINT16_UNALIGNED( ptr )  			GET_UINT16_BIG_ENDIAN_UNALIGNED(ptr)
	#define SET_UINT16_UNALIGNED(  u16value, ptr )	SET_UINT16_BIG_ENDIAN_UNALIGNED(  u16value, ptr )
	#define GET_UINT32_UNALIGNED( ptr )  			GET_UINT32_BIG_ENDIAN_UNALIGNED(ptr)
	#define SET_UINT32_UNALIGNED(  u32value, ptr )	SET_UINT32_BIG_ENDIAN_UNALIGNED(  u32value,ptr)
	#define PKTGET_UINT16_UNALIGNED( ptr )  			GET_UINT16_BIG_ENDIAN_UNALIGNED(ptr)
	#define PKTSET_UINT16_UNALIGNED(  u16value, ptr)	SET_UINT16_BIG_ENDIAN_UNALIGNED(  u16value, ptr )
	#define PKTGET_UINT32_UNALIGNED( ptr )  			GET_UINT32_BIG_ENDIAN_UNALIGNED(ptr)
	#define PKTSET_UINT32_UNALIGNED(  u32value, ptr)	SET_UINT32_BIG_ENDIAN_UNALIGNED(  u32value,ptr)	


#endif




#endif

