/*
* ----------------------------------------------------------------
*
* ---------------------------------------------------------------
*/


#ifndef _SWNIC_H
#define _SWNIC_H

#ifdef RTL865X_MODEL_USER
#include "rtl_types.h" /* for spinlock_t only */
#endif

struct rtl_pktHdr;
typedef int    (*proc_input_pkt_funcptr_t) (struct rtl_pktHdr*);

/* --------------------------------------------------------------------
 * ROUTINE NAME - swNic_init
 * --------------------------------------------------------------------
 * FUNCTION: This service initializes the switch NIC.
 * INPUT   : 
        nRxPkthdrDesc: Number of Rx pkthdr descriptors.
        nRxMbufDesc: Number of Rx mbuf descriptors.
        nTxPkthdrDesc: Number of Tx pkthdr descriptors.
        clusterSize: Size of a mbuf cluster.
        processInputPacket: Received packet processing function.
        shortCut: Short cut callback function. User can install a 
            routine which could decide whether the received packet 
            should be forwarded and (if necessary) modify the packet 
            in a very short time. If this routine returns 0, the 
            packet is immediately sent out, otherwise it is handled 
            as usual. Note that this routine must be fast, never 
            allocate or release pkthdr or mbuf, and all shared data 
            must be protected if this routine touches it, because this 
            routine will be called inside the interrupt handler.
 * OUTPUT  : None.
 * RETURN  : Upon successful completion, the function returns ENOERR. 
        Otherwise, 
		EINVAL: Invalid argument.
		ENFILE: Packet header or mbuf available.
 * NOTE    : None.
 * -------------------------------------------------------------------*/
#ifdef CONFIG_RTL865XC 
int32 swNic_init(uint32 nRxPkthdrDesc0,uint32 nRxPkthdrDesc1,uint32 nRxPkthdrDesc2,
			uint32 nRxPkthdrDesc3,uint32 nRxPkthdrDesc4,uint32 nRxPkthdrDesc5,
			uint32 nRxMbufDesc0, 
			uint32 nTxPkthdrDesc0, uint32 nTxPkthdrDesc1, 
			uint32 clusterSize, 
                        proc_input_pkt_funcptr_t processInputPacket, 
                        int32 reserved(struct rtl_pktHdr *));
 #else
int32 swNic_init(uint32 nRxPkthdrDesc, uint32 nRxMbufDesc, uint32 nTxPkthdrDesc, 
                        uint32 clusterSize, 
                        proc_input_pkt_funcptr_t processInputPacket, 
                        int32 shortCut(struct rtl_pktHdr *));
#endif 
                        
proc_input_pkt_funcptr_t swNic_installedProcessInputPacket(proc_input_pkt_funcptr_t);


/* --------------------------------------------------------------------
 * ROUTINE NAME - swNic_intHandler
 * --------------------------------------------------------------------
 * FUNCTION: This function is the NIC interrupt handler.
 * INPUT   :
		intPending: Pending interrupts.
 * OUTPUT  : None.
 * RETURN  : None.
 * NOTE    : None.
 * -------------------------------------------------------------------*/

int32 swNic_intHandler(int32 *lastRxDescIdx);

#define UNCACHE_MASK		0x20000000
#define UNCACHE(x)		((u32)x|UNCACHE_MASK)
#define CACHED(x)		((u32)x & ~UNCACHE_MASK)
#define SWNIC_RX_ALIGNED_IPHDR

#ifdef CONFIG_RTL865XC 

typedef struct swNicCounter_s {
    int32 rxIntNum, txIntNum, rxPkthdrRunoutNum, rxMbufRunoutNum, rxPkthdrRunoutSolved, rxPktErrorNum, txPktErrorNum, rxPktCounter, txPktCounter;
    int32 currRxPkthdrDescIndex0,currRxPkthdrDescIndex1,currRxPkthdrDescIndex2,currRxPkthdrDescIndex3,currRxPkthdrDescIndex4,currRxPkthdrDescIndex5,
		currRxMbufDescIndex0, currTxPkthdrDescIndex0, freeTxPkthdrDescIndex0, currTxPkthdrDescIndex1, freeTxPkthdrDescIndex1;
	
	int32 linkChanged;
} swNicCounter_t;


#else

typedef struct swNicCounter_s {
    int32 rxIntNum, txIntNum, rxPkthdrRunoutNum, rxMbufRunoutNum, rxPkthdrRunoutSolved, rxPktErrorNum, txPktErrorNum, rxPktCounter, txPktCounter;
    int32 currRxPkthdrDescIndex, currRxMbufDescIndex, currTxPkthdrDescIndex, freeTxPkthdrDescIndex;
	int32 linkChanged;
} swNicCounter_t;

#endif


void swNic_rxThread(unsigned long param);
 int32 swNic_write(void * output);
extern int8 swNic_Id[];
extern int32 miiPhyAddress;
extern spinlock_t *rtl865xSpinlock;


#ifdef CONFIG_RTL865XC 

extern uint32	totalRxPkthdr0, totalRxMbuf0,totalTxPkthdr0, totalTxPkthdrShift0;  /*total desc count on each ring */
extern uint32	totalRxPkthdr1,totalTxPkthdr1, totalTxPkthdrShift1;  /*total desc count on each ring */
extern uint32	totalRxPkthdr2;  /*total desc count on each ring */
extern uint32	totalRxPkthdr3;  /*total desc count on each ring */
extern uint32	totalRxPkthdr4;  /*total desc count on each ring */
extern uint32	totalRxPkthdr5;  /*total desc count on each ring */

extern uint32 	rxMbufIndex0, rxPhdrIndex0, txDoneIndex0, txFreeIndex0,lastTxIndex0; /* current desc index */
extern uint32 	rxPhdrIndex1, txDoneIndex1, txFreeIndex1,lastTxIndex1; /* current desc index */
extern uint32 	rxPhdrIndex2; /* current desc index */
extern uint32 	rxPhdrIndex3; /* current desc index */
extern uint32 	rxPhdrIndex4; /* current desc index */
extern uint32 	rxPhdrIndex5; /* current desc index */

extern volatile uint32	*RxPkthdrRing0,*RxMbufRing0, *TxPkthdrRing0; /* descriptor ring, should be volatile */
extern volatile uint32 	*RxPkthdrRing1, *TxPkthdrRing1; /* descriptor ring, should be volatile */
extern volatile uint32 	*RxPkthdrRing2; /* descriptor ring, should be volatile */
extern volatile uint32 	*RxPkthdrRing3; /* descriptor ring, should be volatile */
extern volatile uint32 	*RxPkthdrRing4; /* descriptor ring, should be volatile */
extern volatile uint32 	*RxPkthdrRing5; /* descriptor ring, should be volatile */
#define TxPkthdrDescFull0	(((txFreeIndex0+1)&((1<<totalTxPkthdrShift0)-1))==txDoneIndex0)
#define TxPkthdrDescFull1	(((txFreeIndex1+1)&((1<<totalTxPkthdrShift1)-1))==txDoneIndex1)

/*----------------------------------------------------------------------------------------------------
 *                 !!! WARNING !!!
 *  All the variables in this section should be ONLY available in 865xC model code.
 *  Because we want ROME driver to compile without any error.
 *  However, in final, this section should be discard.
 *----------------------------------------------------------------------------------------------------*/
 #if 0
#define totalRxPkthdr totalRxPkthdr0
#define totalRxMbuf totalRxMbuf0
#define totalTxPkthdr totalTxPkthdr0
#define rxPhdrIndex rxPhdrIndex0
#define rxMbufIndex rxMbufIndex0
#define txDoneIndex txDoneIndex0
#define txFreeIndex txFreeIndex0
#define TxPkthdrRing TxPkthdrRing0
#define RxPkthdrRing RxPkthdrRing0
#define TxPkthdrRing TxPkthdrRing0
#define CPURPDCR CPURPDCR0
#define CPUTPDCR CPUTPDCR0
#define CPURMDCR CPURMDCR0
#define RxMbufRing RxMbufRing0
#define lastTxIndex lastTxIndex0
#define lastReclaim lastReclaim0
#endif

#else

extern uint32	totalRxPkthdr, totalRxMbuf,totalTxPkthdr, totalTxPkthdrShift;  /*total desc count on each ring */
extern uint32 	rxMbufIndex, rxPhdrIndex, txDoneIndex, txFreeIndex, lastTxIndex; /* current desc index */
extern volatile uint32 	*RxPkthdrRing,*RxMbufRing, *TxPkthdrRing; /* descriptor ring */
#define TxPkthdrDescFull	(((txFreeIndex+1)&((1<<totalTxPkthdrShift)-1))==txDoneIndex)

#endif

extern int32  rxEnable, txEnable;


#define SWNIC_DEBUG  
#ifdef SWNIC_DEBUG 
extern int32 nicDbgMesg;
#define NIC_RX_PKTDUMP			0x15
#define NIC_TX_PKTDUMP			0x2a
#define NIC_PHY_RX_PKTDUMP	0x1
#define NIC_PHY_TX_PKTDUMP		0x2
#define NIC_EXT_RX_PKTDUMP		0x4
#define NIC_EXT_TX_PKTDUMP		0x8
#define NIC_PS_RX_PKTDUMP		0x10
#define NIC_PS_TX_PKTDUMP		0x20
#endif


#ifdef CONFIG_RTL865X_CLE
#include "cle/rtl_cle.h" 
extern cle_exec_t swNic_cmds[];
#define CMD_RTL8651_SWNIC_CMD_NUM		6

#endif
#endif /* _SWNIC_H */
