/*
 */

#include <stdarg.h>
#include "../boa.h"
#include "httpd.h"
#include <signal.h>  

#include <sys/klog.h>
#include <sys/sysinfo.h>

#include <re865x.h>
#include <board.h>
#include "../rtl865x/flashdrv.h"
#include "../rtl865x/rtl_board.h"

#include <dni_shutils.h>
#include <pathnames.h>

extern romeCfgParam_t * pRomeCfgParam;

//#define DPRINTF printf
#define DPRINTF

/* redefined */
#define RTL8651_LOGMODULE_SYS 		0x00000001
#define RTL8651_LOGMODULE_ACL 		0x00000002
#define RTL8651_LOGMODULE_DOS 		0x00000004
#define RTL8651_LOGMODULE_URL 		0x00000008
#define RTL8651_LOGMODULE_NEWFLOW	0x00000010
/* end */

/* Copy each token in wordlist delimited by space into word */
#define foreach(word, wordlist, next) \
	for (next = &wordlist[strspn(wordlist, " ")], \
	     strncpy(word, next, sizeof(word)), \
	     word[strcspn(word, " ")] = '\0', \
	     word[sizeof(word) - 1] = '\0', \
	     next = strchr(next, ' '); \
	     strlen(word); \
	     next = next ? &next[strspn(next, " ")] : "", \
	     strncpy(word, next, sizeof(word)), \
	     word[strcspn(word, " ")] = '\0', \
	     word[sizeof(word) - 1] = '\0', \
	     next = strchr(next, ' '))
	     
extern char *nvram_get(char *name);
extern int nvram_set(char *name, char *value);
int so_printf(webs_t r, const char *fmt, ...);	     

#define MAX_STRING_LEN	1024

struct attached_devices _attached_devices[MAX_ATTACHED_DEVICES];//ATTACHED_DEVICES
int atchedDvisCnt;//ATTACHED_DEVICES

struct non_attached_devices _non_attached_devices[MAX_ATTACHED_DEVICES];//ATTACHED_DEVICES
int non_atchedDvisCnt;//ATTACHED_DEVICES

struct dhcpc_rsvd_addr _dhcpc_rsvd_addr[MAX_RESERVED_FIXED];//ATTACHED_DEVICES
int dhcpcRsvdAddrCnt;//ATTACHED_DEVICES

int so_printf(webs_t r, const char *fmt, ...)
{
	va_list argptr;
	int nbytes;
	int buf_end = r->buffer_end;
	
	char buffer[MAX_STRING_LEN+50];	

	//buffer = malloc(MAX_STRING_LEN+50);
	//if (!buffer)
	//	return (buf_end);
	
	va_start(argptr, fmt);
	nbytes = vsprintf(buffer, fmt, argptr);
	va_end(argptr);
	
	buf_end = req_write(r, buffer);
	//if (buffer) {
	//	free(buffer);
	//}	
	
	return buf_end;
}	


static int
ej_get_firmware_ver(int eid, webs_t req, int argc, char **argv)
{
	char szVer[15];
	
	return websWrite(req, "%s", "1.00.00");
}


static int
ej_get_udpserver_status(int eid, webs_t req, int argc, char **argv)
{
	FILE *f1;

	f1 = fopen("/var/run/UDPserver.pid","r");
	if (f1) {		
		return websWrite(req, "%d", 1);
	}	
	else {
		return websWrite(req, "%d", 0);		
	}	
}

/*
 * Example: 
 * wan_proto=dhcp
 * <% nvram_invmatch("wan_proto", "dhcp", "disabled"); %> does not produce
 * <% nvram_invmatch("wan_proto", "static", "disabled"); %> produces "disabled"
 */
static int
ej_nvram_invmatch(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *invmatch, *output;

	if (ejArgs(argc, argv, "%s %s %s", &name, &invmatch, &output) < 3) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	if (nvram_invmatch(name, invmatch)) {
		return websWrite(wp, output);
	}	

	return 0;
}	

/*
 * Example: 
 * filter_maclist=00:12:34:56:78:00 00:87:65:43:21:00
 * <% nvram_list("filter_maclist", 1); %> produces "00:87:65:43:21:00"
 * <% nvram_list("filter_maclist", 100); %> produces ""
 */
static int
ej_nvram_list(int eid, webs_t wp, int argc, char **argv)
{
	char *name;
	int which;
	char word[256], *next;

	if (ejArgs(argc, argv, "%s %d", &name, &which) < 2) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	foreach(word, nvram_safe_get(name), next) {
		if (which-- == 0) {
			return websWrite(wp, word);
		}	
	}

	return 0;
}

extern char *none_backup_nvram[];
static int
ej_nvram_show(int eid, webs_t wp, int argc, char **argv)
{
	char *value;
	int ret = 0, i = 0, cfg_len = 0, totalLen = 0;
	char /**cfg_chksum_str = "cfg_chksum",*/ len[30];
	//int val = 0;
	
	value = nvram_show_all(1, 0, none_backup_nvram);
	while (value) {
		if (*value) {
			//ret += websWrite(wp, "%s\n", value);
			//if (strncmp(value, cfg_chksum_str, strlen(cfg_chksum_str)) != NULL) {
			ret += websWrite(wp, "%s\n", value);
			cfg_len = strlen(value);
			for (i = 0; i < cfg_len; i++)
				totalLen += value[i];
			//} else {
				//sprintf(len, "%s=%d", cfg_chksum_str, totalLen);
				//printf("len = %s\n", len);
				//ret += websWrite(wp, "%s\n", len);
			//}
				
			//printf("value = %s; len = %d; total length = %d\n", value, j, totalLen);
		}	

		value = nvram_show_all(0, 0, none_backup_nvram);
	}	
	
	//sprintf(len, "%s=%d", cfg_chksum_str, totalLen);
	websWrite(wp, "end\n");
	sprintf(len, "%d", totalLen);

	//printf("len = %s\n", len);
	ret += websWrite(wp, "%s\n", len);
				
	return ret;
}

extern int noread;
int Check_Reload_Cfg(webs_t wp)
{
	if (wp->method != M_GET)
		return -1;
	if (noread > 0) {
		noread--;	/* noread counter */
		return -1;
	}	
	if (strstr(wp->pathname, "pack_frame.html")) {
		acl_init();
		return 0;
	}	
	else if (strstr(wp->pathname, "keywordform.html")) {	
		if (cfgmgr_read(CFGMGR_TABID_URLFILTER,(void*)(pRomeCfgParam->urlfilterCfgParam), sizeof(struct urlfilterCfgParam_s)*MAX_PPPOE_SESSION*MAX_URL_FILTER)!=0)
		{
			printf("urlfilter_init: call cfgmgr_read fail\n");
			/* take proper actions */
			return -1;
		}		
		return 0;
	}
	else if (strstr(wp->pathname, "port_trigger.html")) {	
		specialap_init();	
		return 0;
	}
	else if (strstr(wp->pathname, "lanform.html")) {	
		dhcps_init();	
		return 0;
	}			
	else if (!strcasecmp(wp->pathname, "/www/WL_acl.html")) {
		wlan_acl_init();	
		return 0;
	}
	else 
		return -1;
}			



#if !defined(NVRAM_SHARED_LIB)

static int match( const char* pattern, const char* string );
static int match_one( const char* pattern, int patternlen, const char* string );


int nvram_handler_match( const char* pattern, const char* string )
{
    return (match(pattern,string));
}

/* Simple shell-style filename matcher.  Only does ? * and **, and multiple
** patterns separated by |.  Returns 1 or 0.
*/
int
match( const char* pattern, const char* string )
    {
    const char* or;

    for (;;)
	{
	or = strchr( pattern, '|' );
	if ( or == (char*) 0 )
	    return match_one( pattern, strlen( pattern ), string );
	if ( match_one( pattern, or - pattern, string ) )
	    return 1;
	pattern = or + 1;
	}
    }


static int
match_one( const char* pattern, int patternlen, const char* string )
    {
    const char* p;

    for ( p = pattern; p - pattern < patternlen; ++p, ++string )
	{
	if ( *p == '?' && *string != '\0' )
	    continue;
	if ( *p == '*' )
	    {
	    int i, pl;
	    ++p;
	    if ( *p == '*' )
		{
		/* Double-wildcard matches anything. */
		++p;
		i = strlen( string );
		}
	    else
		/* Single-wildcard matches anything but slash. */
		i = strcspn( string, "/" );
	    pl = patternlen - ( p - pattern );
	    for ( ; i >= 0; --i )
		if ( match_one( p, pl, &(string[i]) ) )
		    return 1;
	    return 0;
	    }
	if ( *p != *string )
	    return 0;
	}
    //if ( *string == '\0' || *string == '?' )
    if ( *string == '\0' )
	return 1;
    return 0;
}


char* nvram_get(char *name)
{
	char *c=NULL;
	struct ej_nvram_handler *handler;	

        /* request semaphore */
    	//status = NU_Obtain_Semaphore(&nvram_get_semaphore, NU_SUSPEND);	
	
	/* Call handler */
	for (handler = &nvram_handlers[0]; handler->pattern; handler++) {
		//if (strncmp(handler->pattern, name, strlen(handler->pattern)) == 0) {
	    if (nvram_handler_match(handler->pattern, name)) {
			if (handler->get)
			  c = (char *)handler->get(name);
			break;
		}	

	}
    	//status = NU_Release_Semaphore(&nvram_get_semaphore);
	return c;
}

int nvram_set(char *name, char *value)
{
	char *c=NULL;
	struct ej_nvram_handler *handler;	

        /* request semaphore */
    	//status = NU_Obtain_Semaphore(&nvram_get_semaphore, NU_SUSPEND);	
	
	/* Call handler */
	for (handler = &nvram_handlers[0]; handler->pattern; handler++) {
		//if (strncmp(handler->pattern, name, strlen(handler->pattern)) == 0) {
	    if (nvram_handler_match(handler->pattern, name)) {
			if (handler->set)
			  c = (char *)handler->set(name, value);
			break;
		}	

	}
    	//status = NU_Release_Semaphore(&nvram_get_semaphore);
	return c;
}
#endif	/* #if !defined(NVRAM_SHARED_LIB) */


int nvram_match(char *name, char *match)
{
	char *c;
	int ret = 0;
	
	c = nvram_get(name);
	if (c != NULL) {
		if (strcmp(c, match) == 0)
		  ret = 1;
	}
	return ret;
}

int nvram_invmatch(char *name, char *invmatch)
{
	char *c;
	int ret = 1;
	
	c = nvram_get(name);
	if (c != NULL) {
		if (strcmp(c, invmatch) == 0)
		  ret = 0;
	}
	return ret;
}


extern struct ej_nvram_handler nvram_handlers[];
struct ej_nvram_handler *pNvramHandle=nvram_handlers;


/* argv[0] -> skip nvram item */
char*
nvram_show_all(int restart, int agrc, char **argv)
{
	char *pstrValue;
	static nvram_value[100];	
	int i = 0;
	
	if (restart)
		pNvramHandle=nvram_handlers;
	
	if ( pNvramHandle->pattern ) {
		
		i = 0;
		while (argv[i] != 0) {
			if (strstr(pNvramHandle->pattern, argv[i])) {
				strcpy(nvram_value, "");
				goto exit_show;
			}	
			i++;
		}		
		pstrValue = nvram_get( pNvramHandle->pattern );
		sprintf(nvram_value, "%s=%s", pNvramHandle->pattern, (pstrValue ? pstrValue : ""));		
exit_show:
		pNvramHandle++;
		return (nvram_value);
	}
	return NULL;	

}

int nvram_get_convert(webs_t wp, char *ptr)
{
	char *c;
	int ret = 0;
	int convert = 0, all_convert=0;	
	
	for (c = ptr; *c; c++) {
		//if (isprint((int) *c) &&
		//if ((all_convert == 0) && isprint((int) *c) &&
		if ((convert == 0) && (all_convert == 0) && isprint((int) *c) &&		
		    *c != '"' && *c != '&' && *c != '<' && *c != '>' && *c != '\'') {
		    	if ((convert == 1) && isdigit((int) *c)) {
				ret += websWrite(wp, "&#%d", *c);
				convert = 1;		    		
			}
			else {	
				ret += websWrite(wp, "%c", *c);
				convert = 0;
			}	
		}	
		else {
			ret += websWrite(wp, "&#%d", *c);
			convert = 1;
		}	
	}

	return ret;
}
		  
/* these nvram related function for temperate testing only */

/*
 * Example: 
 * lan_ipaddr=192.168.1.1
 * <% nvram_get("lan_ipaddr"); %> produces "192.168.1.1"
 * <% nvram_get("undefined"); %> produces ""
 */
static int
ej_nvram_get(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *c;
	int ret = 0;
	int convert = 0, all_convert=0;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}
	
	if (!strcmp(name, "ssid"))
		all_convert = 1;		/* for netscape */

	for (c = nvram_safe_get(name); *c; c++) {
		//if (isprint((int) *c) &&
		if ((all_convert == 0) && isprint((int) *c) &&
		    *c != '"' && *c != '&' && *c != '<' && *c != '>' && *c != '\'') {
		    	if ((convert == 1) && isdigit((int) *c)) {
				ret += websWrite(wp, "&#%d", *c);
				convert = 1;		    		
			}
			else {	
				ret += websWrite(wp, "%c", *c);
				convert = 0;
			}	
		}	
		else {
			ret += websWrite(wp, "&#%d", *c);
			convert = 1;
		}	
	}

	return ret;
}



static int
ej_POT_get(int eid, webs_t wp, int argc, char **argv)
{
	int pot;

	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	unsigned int port0;
	unsigned int port1;
//	char *port0, *port1;	
	unsigned int ports, porte;
	
    //printf("#### POT_ReadPowerOnTime = %d\r\n",pot);
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "wan_POT_counter") == 0) {
		pot = POT_ReadPowerOnTime();
		
		ret +=websWrite(wp, "%d", pot);
	}//Add Jenny 2006/01/05	
	else if (strcmp(name, "wan_POT_Resolution") == 0) {
		port0 = POT_GetResolution();
		
		ret +=websWrite(wp, "%d", port0);
	}
	else if (strcmp(name, "wan_POT_Threshold") == 0) {
		port1 = POT_GetRepackThreshold();

		ret +=websWrite(wp, "%d", port1);
	}
	//end Jenny			
	
	return ret;	
}


/*
 * Example: 
 * wan_proto=dhcp
 * <% nvram_match("wan_proto", "dhcp", "selected"); %> produces "selected"
 * <% nvram_match("wan_proto", "static", "selected"); %> does not produce
 */
static int
ej_nvram_match(int eid, webs_t wp, int argc, char **argv)
{
	char *name, *match, *output;

	if (ejArgs(argc, argv, "%s %s %s", &name, &match, &output) < 3) {
		//websError(wp, 400, "Insufficient args\n");
		return -1;
	}

	if (nvram_match(name, match)) {
		return websWrite(wp, output);
	}	

	return 0;
}	


int get_wanstatus(void)
{
	char *ptr, *proto;
	int up, speed, duplex, range;	
	FILE *fp;

	rtl8651_checkEthernetPortStatus(0/*WAN_PORT*/,&up, &speed, &duplex, &range);
	if (up == 0)
		return 0;

	proto = nvram_safe_get("wan_proto");
	
	if (strcmp(proto, "static") == 0 || strcmp(proto, "dhcp") == 0) {
		ptr = nvram_safe_get("wan0_ipaddr");
	
		if (!(*ptr) || (strcmp(ptr, "0.0.0.0") == 0))
			return 0;
		return 1;
	}
	else if (strcmp(proto, "telstra") == 0) {
		ptr = nvram_safe_get("wan0_ipaddr");
		if (!(*ptr) || (strcmp(ptr, "0.0.0.0") == 0))
			return 0;
		if((fp = fopen(PPP_LINK_PATH"bpa", "r"))) {
			fclose(fp);			
			return 1;
		}	
		return 0;		
	}
	else if (strcmp(proto, "pppoe") == 0) {
		if (pppoe_dial_status(0) == 3)
			return 1;
		return 0;
	}
	else if (strcmp(proto, "pptp") == 0) {
		if (pptp_dial_status(0) == 3)
			return 1;
		return 0;
	}
	else if (strcmp(proto, "l2tp") == 0) {
		if (l2tp_dial_status(0) == 3)
			return 1;
		return 0;
	}
	else if (strcmp(proto, "dhcpl2tp") == 0) {
		if (l2tp_dial_status(0) == 3)
			return 1;
		return 0;
	}		
				
	return 0;	
}
				

static int
ej_wanstatus_get(int eid, webs_t wp, int argc, char **argv)
{
	/* up -> connected
	   down -> disconnected */
	   
	int status;   

	status = get_wanstatus();
	if (status == 1) {	   
		return websWrite(wp, "1");		
	}
	else {
		return websWrite(wp, "0");
	}		
}

static int
ej_wan_dhcp_get(int eid, webs_t wp, int argc, char **argv)
{
   
	char *ptr;   

	ptr = nvram_safe_get("wan_proto");
	if (!strcmp(ptr, "dhcp")) {	   
		return websWrite(wp, "DHCPClient");		
	}
	else if (!strcmp(ptr, "telstra")) {	   
		return websWrite(wp, "BPA");		
	}
	else if (!strcmp(ptr, "pppoe")) {	   
		return websWrite(wp, "PPPoE");		
	}	
	else if (!strcmp(ptr, "pptp")) {	   
		return websWrite(wp, "PPTP");		
	}	
	else if (!strcmp(ptr, "l2tp")) {	   
		return websWrite(wp, "Static L2TP");		
	}
	else if (!strcmp(ptr, "dhcpl2tp")) {	   
		return websWrite(wp, "Dynamic L2TP");		
	}					
	else if (!strcmp(ptr, "static")) {	   
		return websWrite(wp, "FixedIP");		
	}
	else {
		return websWrite(wp, "None");
	}		
}

static int
ej_lan_dhcp_get(int eid, webs_t wp, int argc, char **argv)
{
   
	char *ptr;   

	ptr = nvram_safe_get("lan_proto");
	if (strcmp(ptr, "dhcp") == 0) {	   
		return websWrite(wp, "ON");		
	}
	else {
		return websWrite(wp, "OFF");
	}		
}

extern int g_bLogin;

static int
ej_logout(int eid, webs_t wp, int argc, char **argv)
{
	clr_http_client();
	
	return websWrite(wp, "L");		
}

extern char return_page[];
extern int refresh_time;

static int
ej_get_refresh_page(int eid, webs_t wp, int argc, char **argv)
{
	//return websWrite(wp, "http://%s%s", nvram_safe_get("lan_ipaddr"), return_page);		
	return websWrite(wp, "%s", return_page);		
	
}

static int
ej_get_refresh_time(int eid, webs_t wp, int argc, char **argv)
{
	return websWrite(wp, "%d", refresh_time);		
}


extern client_req_t client_req;

static int
ej_login_ipaddr(int eid, webs_t wp, int argc, char **argv)
{
	return websWrite(wp, "%s", client_req.client_ip);		
}

static int
ej_login_hostname(int eid, webs_t wp, int argc, char **argv)
{
	char PCip[20];
	unsigned char dhcpc_hostname[34]={0};
	unsigned char hostname[34]={0};

	memset(PCip, 0, 20);
	strncpy(PCip, client_req.client_ip, strlen(client_req.client_ip));
	strcpy(dhcpc_hostname, check_dhcpc_login_hostname(PCip));
	
	if ( dhcpc_hostname[0] == NULL )
		strcpy(hostname, check_http_login_hostname(PCip));
	else
		strcpy(hostname, dhcpc_hostname);	
	
	//if ( hostname[0] == NULL )
	//	return 0;	
	//else
	return websWrite(wp, "%s", hostname);	
}

#define DHCPD_LEASES_PATH "/var/"
char login_hostname[34] = "";
unsigned char *
check_dhcpc_login_hostname(char *ip) {

	FILE *fp;	
	struct in_addr addr;
	char sigusr1[] = "-XX";
	char line[100];
	unsigned char ipa[20]="", mac[20]="", hostname[34]="";
	int pid;
	char PCip[20], _PCip[20];

	strcpy(login_hostname, "");
	inet_aton(ip, PCip);
	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR2);    
		sleep(1);		/* waiting writed completed */
	}

	if ((fp = fopen(DHCPD_LEASES_PATH "dhcps_client", "r")) == NULL)
		printf("unable to open file\n");
	else {
		while( fgets(line, sizeof(line), fp)!=NULL) {
			memset(mac, 0, 20);		
			memset(ipa, 0, 20);
			memset(hostname, 0, sizeof(hostname));	

			if(sscanf(line, "%s %s %s\n", mac, ipa, hostname) != 3 )
				continue;

			inet_aton(ipa, _PCip);
			printf("%s; PCip = %d; ipa = %d\n", __FUNCTION__, PCip, _PCip);	
			if(memcmp(PCip, _PCip, 4))
				continue;
			else 
				break;			
	    	}	
	}
	if (fp)
		fclose(fp);
	
        strcpy(login_hostname, hostname);
	return login_hostname;		
}

unsigned char * 
check_http_login_hostname(char *ip)
{
	FILE *fp;
       unsigned char line[100];
	unsigned char ipa[50];           // ip address
	unsigned char hwa[50];           // HW address 
	unsigned char mask[50];          // ntemask   
	unsigned char dev[50];           // interface
	int type;               // HW type
	int flags;              // flags
	unsigned char mac[20]; 
	unsigned char hostname[34];
	char PCip[20], _PCip[20];

	inet_aton(ip, PCip);
       if ((fp = fopen("/proc/net/arp", "r")) == NULL) 
		return NULL;
        	
        
	      // Read the ARP cache entries.
	      // IP address       HW type     Flags       HW address            Mask     Device
	      // 192.168.2.1      0x1         0x2         xx:xx:xx:xx:xx:xx     *          br0
		while( fgets(line, sizeof(line), fp)!=NULL)
		{
			if(sscanf(line, "%s 0x%x 0x%x %s %s %s\n", ipa, &type, &flags, hwa, mask, dev) != 6)
				continue;
			
			inet_aton(ipa, _PCip);
			if(memcmp(PCip, _PCip, 4))
				continue;
			else {
				if ((attached_devices(ipa, hwa, hostname) == 1)  || (attached_devices(ipa, hwa, hostname) == 2)) 
					break;	/* break while */
			}
		}

	if (fp)
		fclose(fp);
	
       strcpy(login_hostname, hostname);
	return login_hostname;	
}


static char *
rfctime(const time_t *timep)
{
	static char s[201];
	struct tm tm;

	memcpy(&tm, localtime(timep), sizeof(struct tm));
	strftime(s, 200, "%A, %d %B %Y %H:%M:%S", &tm);	
//	strftime(s, 200, "%a, %d %b %Y %H:%M:%S %z", &tm);
//	strftime(s, 200, "%Y/%m/%d,%H:%M:%S", &tm);
	return s;
}
/* Report time in RFC-822 format */
static int
ej_localtime(int eid, webs_t wp, int argc, char **argv)
{
	time_t tm;
    
	time(&tm);
//	printf("current time is %s\n", ctime(&tm));
	
	return websWrite(wp, rfctime(&tm));
}

static int
ej_sys_up_time(int eid, webs_t wp, int argc, char **argv)
{
	int ret = 0;
	unsigned long hour, minute, sec;
	
	sec = GetSysUpTime();			
	hour = sec / (60 * 60);
	sec = sec % (60 * 60);
	minute = sec / 60;
	sec = sec % 60;
	ret += websWrite(wp, "%02d:%02d:%02d", hour, minute, sec);

	return ret;	

}

static int
ej_ddns_status(int eid, webs_t wp, int argc, char **argv)
{
	int status=0, ret = 0;
	FILE *fp=NULL;

   	fp = fopen("/tmp/ddns_status.txt", "r");	
   	
   	if (strcmp(nvram_get("ddns_enable"), "0")==0) {
   		ret += websWrite(wp, "Dynamic DNS service is not enabled!");	
   	}
   	
   	else if (!fp) {
   		ret += websWrite(wp, "Dynamic DNS NULL IP");	
   	}
   	else {
   		char buf[256];
   		
   		fgets(buf, 255, fp);
   		ret += websWrite(wp, "Dynamic DNS %s", buf);	
	}   
	if (fp)
		fclose(fp);		
	return ret;	

}


static int
ej_bpa_login_status(int eid, webs_t wp, int argc, char **argv)
{
	int status=0, ret = 0;
	FILE *fp=NULL;

	if((fp = fopen(PPP_LINK_PATH"bpa", "r"))) {
		fclose(fp);			
		ret += websWrite(wp, "Logged In");
	}	
	else {
		ret += websWrite(wp, "Logged Out");
	}
		
	return ret;	

}



long GetSysUpTime()
{
struct sysinfo info;

	sysinfo(&info);
	return info.uptime;
}

struct lease_t {
	unsigned char chaddr[16];
	unsigned long yiaddr;
	unsigned long expires;
	char hostname[32]; //ATTACHED_DEVICES, original is 64
};


/*
 * Reads file and returns contents
 * @param	fd	file descriptor
 * @return	contents of file or NULL if an error occurred
 */
char *
fd2str(int fd)
{
	char *buf = NULL;
	size_t count = 0, n;

	do {
		buf = realloc(buf, count + 512);
		n = read(fd, buf + count, 512);
		if (n < 0) {
			free(buf);
			buf = NULL;
		}
		count += n;
	} while (n == 512);

	close(fd);
	if (buf)
		buf[count] = '\0';
	return buf;
}

/*
 * Reads file and returns contents
 * @param	path	path to file
 * @return	contents of file or NULL if an error occurred
 */
char *
file2str(const char *path)
{
	int fd;

	if ((fd = open(path, O_RDONLY)) == -1) {
		perror(path);
		return NULL;
	}

	return fd2str(fd);
}


/**********************************
return value:
	0:	fail to open var/run/XXX.pid file
	pid: pid number
***********************************/
int get_task_pid(char *strTask)
{
	int pid; 
	char filename[80];
	char *strPid;

	
	snprintf(filename, sizeof(filename), "/var/run/%s.pid", strTask);
	if( (strPid = file2str( filename )) )
	{
		pid = atoi(strPid);
		free( strPid );
	}
	else
		pid = 0;;

	return pid;
}


/**********************************
return value:
	0:	task of this pid is dead
	1: 	task of this pid is alive
***********************************/
int pid_is_running(int pid)
{
	char filename[80];
	FILE *fp;
	
	snprintf(filename, sizeof(filename), "/proc/%d/status", pid);

	if((fp = fopen(filename, "r"))){
		//fgets(line, sizeof(line), fp);
        	/* Buffer should contain a string like "Name:   binary_name" */
		//sscanf(line, "%*s %s", name);
        	fclose(fp);
		return 1;
        }
	
	 //cprintf("cannot open %s\n", filename);
        return 0;
}

/*********************************************************************
return value:
	0:	task is dead
	1:	task is alive
	2:	cannot open XXX.pid file(this task is never brought up from system start)
*********************************************************************/
int task_is_running(char *strTask)
{
	int pid;
	
	if( (pid=get_task_pid(strTask)) )
	{
		return pid_is_running(pid);
	}
	else
	{
		//cprintf("fail to open var/run/%s.pid\n", strTask);
		return 2;
	}
}

#if 0
#define BUFSIZE		8196
int dump_firewall_log(webs_t wp) 
{
	char *buf=NULL;
	int cmd=3;	/* 4 -> read/clear */
	int lastc;
	int i, n;	
	int out_cnt = 0;	
	
	buf = malloc(BUFSIZE);	
	if ((n = klogctl(cmd, buf, BUFSIZE)) < 0)
		return 0;

	lastc = '\n';
	for (i = 0; i < n; i++) {
		if (lastc == '\n' && buf[i] == '<') {
			i++;
			while (buf[i] >= '0' && buf[i] <= '9')
				i++;
			if (buf[i] == '>')
				i++;
		}
		lastc = buf[i];
		websWrite(wp, "%c", lastc);
		out_cnt++;
	}
	if (lastc != '\n') {
		websWrite(wp, "%c", '\n');
		out_cnt++;
	}	
all_done:
	if (buf) {
		free(buf);
	}
	
	return out_cnt;

}	
#endif


struct circ_buf {
	int size;			
	int head;			
	int tail;			
	char data[1];		
};

/* 12/28/2005 */

/**** Add by Timo Wu*****/
uint32 log_clear_message(void)
{
		FILE *fp;
		
	   	timer_t timep;
	   	struct tm *tm;
	   	char *wday[]={"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
		char *wmonth[]={"Jan", "Feb","Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
		
	   	time(&timep);
	   	tm=gmtime(&timep);
	   	
		if(!(fp = fopen("/var/system.log", "w"))) {
			return -1;
		}
	
		//fprintf(fp, "[Log Cleared] %s, %02d %s %04d %02d:%02d:%02d\n",wday[tm->tm_wday], tm->tm_mday, wmonth[tm->tm_mon], (1900+tm->tm_year), tm->tm_hour, tm->tm_min, tm->tm_sec);

		fclose(fp); 
		
		return; 		
}

#if 0	/* move to shutils.c */
int log_message(const char *fmt, ...)
{
	FILE *fp;
	va_list argptr;
	int nbytes;
	char buffer[200];	

        timer_t timep;
	struct tm *tm;
	char *wday[]={"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	char *wmonth[]={"Jan", "Feb","Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	
	time(&timep);
	tm=gmtime(&timep);
	
	va_start(argptr, fmt);
	nbytes = vsprintf(buffer, fmt, argptr);
	va_end(argptr);
	

	if (!(fp = fopen("/var/system.log", "a+"))) {
			perror("write error /var/system.log");
			return -1;
	}
	
	//fputs(buffer, fp);
	fprintf(fp, "%s,%02d %s %04d %02d:%02d:%02d %s\n",wday[tm->tm_wday], tm->tm_mday, wmonth[tm->tm_mon], (1900+tm->tm_year), tm->tm_hour, tm->tm_min, tm->tm_sec, buffer);
	
	fclose(fp);
	return 0;
}	
#endif	

//Modified by Timo Wu
void clear_syslog(void)
{
	struct circ_buf *buf;		
	int j;
	
	unlink("/var/system.log");
     
}	


//Modified By Timo Wu
void email_log(void)
{
	FILE *fp;
	int return_value;
	char string[100] = "";
	struct stat stat_buf;	
	int fd;
	
	fd=open("/var/system.log", O_RDONLY);
	if(fd < 0)
	{
		printf("fd open error\n");
		return -1;
	}
				

	fstat(fd ,&stat_buf);
	close(fd);	
		
	if(stat_buf.st_size != 0)
		system("/bin/smtpclient subject_log /var/system.log no_trigger");
}
	 

#if 1 //Add by Timo Wu 2006/01/13
      //Modified by Timo Wu 2006/01/16
static int
ej_dumplog(int eid, webs_t wp, int argc, char_t **argv)
{
	int ret=0;
	FILE *fp=NULL;
	char content[200];
	struct circ_buf *buf;	
	unsigned int index=0;
	int i=-1;
	int head, size, tail;
	int show = 0;  
	int log_len = 0;
        
        //printf("ej_dumplog\n");
        
	if ((fp = fopen("/var/system.log","r"))!=NULL)
	{

		while (fgets(content, sizeof(content), fp) != NULL)
		{	
			ret += websWrite(wp, "%s",content);				
		}
		fclose(fp);
	}
	
	return ret;

}

#else

static int
ej_dumplog(int eid, webs_t wp, int argc, char_t **argv)
{
	int  ret = 0;
	FILE *fp=NULL;
	char content[200];
	int log_len = 0;

	//Brian+: dump system log start
	
    	//ret += websWrite(wp, "System Log...\n" );
    	
	struct circ_buf *buf;		
	unsigned int index=0;
	int i=-1;
	int head, size, tail;
	int show = 0;    	
	
	int j; //for loop show
	
	//if (wp == NULL) {
	//	if (!(fp = fopen("/var/system.log", "w"))) {
	//		perror("/var/system.log");
	//		return errno;
	//	}
	//}	
	
#if 0	
	if ((fp = fopen("/var/log/messages","r"))!=NULL)

	{
		while (fgets(content, 200, fp) != NULL)
		{	
			ret += websWrite(wp, "%s",content);	
			ret += websWrite(wp, "<br>");	
		}
		fclose(fp);
	}

#endif

	{

		buf=getCircBufPtrByModuleId(0x01/*moduleId*/);

		if(buf==-1) 
		{
			goto out;
		}
	
		head=buf->head;
		size=buf->size;
		tail=buf->tail;
		
		DPRINTF("head = %d\n", head);
		DPRINTF("size = %d\n", size);
		DPRINTF("tail = %d\n", tail);						
			
		if(head>tail) tail+=size;

		/*
			determine show message or not according to the "prefix" of each message:
				show message only when this message is not dsid related ('x') or the dsid is same as the dsid(it's "ms") we select now.
		*/
		if((buf->data[head]==('0')) || buf->data[head] == 'x')
			show = 1;
		else
			show = 0;

		for( i = head ; i < tail ; i++)
		{
			int idx = i;
			unsigned char c;
			
			while (idx >= size)
				idx -= size;

			c = buf->data[idx];
			content[log_len++] = c;
			if (log_len >= sizeof(content)) {
				log_len = 0;
				content[log_len] = 0;
			}	
						
			if (c == 13)
			{	/* CR: Carriage Return, we ignore it */ 
				continue;
			}

			if(c == 10) 
			{	/* LF: Line Feed, we add new line in html form */
				
				continue;
			}	
			if(c == 0) 
			{	/* end of string: check next message */
				if((buf->data[(idx+1)%size] == ('0')) || (buf->data[(idx+1)%size] == 'x'))
					show = 1;
				else
					show = 0;
				
				if (wp) {			
					ret += websWrite(wp, "%s", content);		
				}
				else {
					fprintf(fp, "%s", content);
				}		
				log_len = 0;
			}

		}						
		
		shmdt(buf);
		
	}
	
	//return ret;
    	//ret += websWrite(wp, "\nFirewall Log...\n" );
	//ret += dump_firewall_log(wp);
	{
	  for(j =0 ;j<4; j++) //Add by Timo Wu
	  {   	   
		char *circBuff=NULL;
	
		circBuff = malloc(sizeof(struct circ_buf)+16000);
		if ( circBuff )
		{
			memset(circBuff, 0, (sizeof(struct circ_buf)+16000));
		   if (j==0)
		   {	
		   	continue;
		   }
		   else if(j==1)
		   {	
		   	
			if (rtl8651_getLogFromKernel(circBuff, RTL8651_LOGMODULE_URL, (sizeof(struct circ_buf)+16000)) == SUCCESS)
			{
				buf = (struct circ_buf *)circBuff;
				
			} else
			{
				buf = NULL;
				
			}
		   }
		   else if(j==2)
		   {	
		   	
			if (rtl8651_getLogFromKernel(circBuff, RTL8651_LOGMODULE_ACL, (sizeof(struct circ_buf)+16000)) == SUCCESS)
			{
				buf = (struct circ_buf *)circBuff;
			} else
			{
				buf = NULL;
				
			}
		   } else if(j==3)
		   {	
		   	
			if (rtl8651_getLogFromKernel(circBuff, RTL8651_LOGMODULE_DOS, (sizeof(struct circ_buf)+16000)) == SUCCESS)
			{
				buf = (struct circ_buf *)circBuff;
			} else
			{
				buf = NULL;
				
			}
		   }			   		   
		} 
		else
		{
			buf = NULL;
		}
		
		if ((buf == -1) ||
			(buf == NULL) )
		{	
	  		if (circBuff)
				free(circBuff);			
			//goto out;
			continue;
		}
	
	
		head=buf->head;
		size=buf->size;
		tail=buf->tail;
		
		DPRINTF("head = %d\n", head);
		DPRINTF("size = %d\n", size);
		DPRINTF("tail = %d\n", tail);						
			
		if(head>tail) tail+=size;

		/*
			determine show message or not according to the "prefix" of each message:
				show message only when this message is not dsid related ('x') or the dsid is same as the dsid(it's "ms") we select now.
		*/
		if((buf->data[head]==('0')) || buf->data[head] == 'x')
			show = 1;
		else
			show = 0;

		for( i = head ; i < tail ; i++)
		{
			int idx = i;
			unsigned char c;
			
			while (idx >= size)
				idx -= size;

			c = buf->data[idx];
			content[log_len++] = c;
			if (log_len >= sizeof(content)) {
				log_len = 0;
				content[log_len] = 0;
			}	
						
			if (c == 13)
			{	/* CR: Carriage Return, we ignore it */ 
				continue;
			}

			if(c == 10) 
			{	/* LF: Line Feed, we add new line in html form */
				
				continue;
			}	
			if(c == 0) 
			{	/* end of string: check next message */
				if((buf->data[(idx+1)%size] == ('0')) || (buf->data[(idx+1)%size] == 'x'))
					show = 1;
				else
					show = 0;
				
				if (wp) {
					ret += websWrite(wp, "%s", content);		
				}
				else {
					fprintf(fp, "%s", content);
				}
				log_len = 0;
			}

		}						
	  	if (circBuff)
			free(circBuff);
	  }
	}
out:			
	if (fp)
		fclose(fp);	
	return ret;	 		
}
#endif

//#define DHCPD_LEASES_PATH "/var/lib/misc/"
#define DHCPD_LEASES_PATH "/var/"
static int
ej_dumpleases(int eid, webs_t wp, int argc, char_t **argv) 
{
	FILE *fp;
	struct lease_t lease;
	int i;
	struct in_addr addr;
	unsigned long expires;
	char sigusr1[] = "-XX";
	int ret = 0;
	int count = 0;
	char *ipaddr, mac[20]="", expires_time[50]="";
	int pid;


#if 0
        /* Write out leases file */
        sprintf(sigusr1, "-%d", SIGUSR1);
        eval("killall", sigusr1, "udhcpd");
#endif    

	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR1);    
		sleep(1);		/* waiting writed completed */
	}
	
	/* Parse leases file */
	if ((fp = fopen(DHCPD_LEASES_PATH "udhcpd.leases", "r"))) {
	    while (fread(&lease, sizeof(lease), 1, fp)) {
		strcpy(mac,"");

		for (i = 0; i < 6; i++) {
			sprintf(mac+strlen(mac),"%02X", lease.chaddr[i]);
			if (i != 5) sprintf(mac+strlen(mac),":");
		}
		mac[17] = '\0';
		if(!strcmp(mac,"00:00:00:00:00:00"))
			continue;
		
		addr.s_addr = lease.yiaddr;

		ipaddr = inet_ntoa(addr);

		expires = ntohl(lease.expires);

             
		strcpy(expires_time,"");
		if (!expires){
			continue;
			strcpy(expires_time,"expired");
		}
		else {
			if (expires > 60*60*24) {
				sprintf(expires_time+strlen(expires_time),"%ld days, ",expires / (60*60*24));
				expires %= 60*60*24;
			}
			if (expires > 60*60) {
				sprintf(expires_time+strlen(expires_time),"%02ld:",expires / (60*60));	// hours
				expires %= 60*60;
			}
			else{
				sprintf(expires_time+strlen(expires_time),"00:");	// no hours
			}
			if (expires > 60) {
				sprintf(expires_time+strlen(expires_time),"%02ld:",expires / 60);	// minutes
				expires %= 60;
			}
			else{
				sprintf(expires_time+strlen(expires_time),"00:");	// no minutes
			}

			sprintf(expires_time+strlen(expires_time),"%02ld:",expires);		// seconds

			expires_time[strlen(expires_time)-1]='\0';
		     }
		ret += websWrite(wp, "%c\"%s\",\"%s\",\"%s\"", //Brian+ for change show order
				count ? ',' :' ',
				mac,  
				/*!*lease.hostname ? "&nbsp;" : lease.hostname, */
				"&nbsp;",
				ipaddr
				);
		count++;
		}
	fclose(fp);
	}
	else
        
	return ret;
}
//added by pamela
static int
ej_upnp_port_list(int eid, webs_t wp, int argc, char_t **argv) 
{
	FILE *fp;
	int i;
	struct in_addr addr;
	char sigusr2[] = "-XX";
	int ret = 0;
      char line[100],act[4],proto[50],ipa[50];
	int Int=0, Ext=0;

        /* Write out upnp file */
      sprintf(sigusr2, "-%d", SIGUSR2);
      eval("killall", sigusr2, "upnp");
	sleep(1);

	/* Parse upnp file */
	if ((fp = fopen("/var/tmp/upnp_portmap", "r")) == NULL) 
		return NULL;
	else{
			// 1       TCP     3       2       192.168.1.22
			while( fgets(line, sizeof(line), fp)!=NULL)
			{
				if(sscanf(line, "%s %s %d %d %s\n", act, proto,&Int, &Ext, ipa)!=5)
					continue;

				ret += websWrite(wp, "<tr>");
				ret += websWrite(wp, "<TD  width=\"10%%\" align=\"center\"><span class=\"ttext\">%s</span></TD>",act);
				ret += websWrite(wp, "<TD  width=\"15%%\" align=\"center\"><span class=\"ttext\">%s</span></TD>",proto);
				ret += websWrite(wp, "<TD  width=\"15%%\" align=\"center\"><span class=\"ttext\">%d</span></TD>",Int);
				ret += websWrite(wp, "<TD  width=\"15%%\" align=\"center\"><span class=\"ttext\">%d</span></TD>",Ext);
				ret += websWrite(wp, "<TD  width=\"25%%\" align=\"center\"><span class=\"ttext\">%s</span></TD>",ipa);
				ret += websWrite(wp, "</tr>");
			}
	}
 	fclose(fp);
	return ret;
}


//#define MAX_URL_FILTER		8
/* #defined in board.h */
static int
ej_nvram_fw_domain_list(int eid, webs_t wp, int argc, char_t **argv)
{
	char name[50], *ptr;
	int ret = 0;
	int i;

	for (i = 0; i < MAX_URL_FILTER; i++) {
		int count = 1;
		
		sprintf(name, "fw_block_keyword_domain%d", i);		     
		ptr = nvram_safe_get(name);
		if (*ptr) {
			ret += websWrite(wp, "<OPTION value=%d>%s</OPTION>", count++, ptr);		
		}	
	}
	
	return ret;
}

static int
ej_checkbox_checked(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0;
	
	if (ejArgs(argc, argv, "%s %s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	c = nvram_safe_get(name);
	
	if (atoi(c) == 1) {
	  ret +=websWrite(wp, "checked");
	}  
	
	return ret;
}	

static int ej_checked_match(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	
	if (ejArgs(argc, argv, "%s %s", &name, &select) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	c = nvram_safe_get(name);
	
	if (strcmp(c, select) == 0) {
	  ret +=websWrite(wp, "checked");
	}  
	
	return ret;
}	

#define AIM		5190
#define AGE_OF_EMPIRE	47624
#define FTP 		21
#define HTTP		80
#define ICUII		23566
#define IP_PHONE	6670
#define NEETMEETING	1720
#define NEWS            119
#define PPTP            1723
#define QUAKEII_III     27960
#define REAL_AUDIO      6970
#define TELNET          23

static int ej_client_selected_match(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s %s", &name, &select) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "port") == 0) {
		port = atoi(select);
		c = nvram_get_current_fw_client(name);


		/* Check for LAN IP address specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;

		ports = atoi(port0);
		porte = atoi(port1);
		if (port == 0) {
			if ((ports == AIM) || (ports == AGE_OF_EMPIRE) || (ports == FTP) || (ports == HTTP)  
				|| (ports == ICUII) || (ports == IP_PHONE) || (ports == NEETMEETING)
				|| (ports == NEWS) || (ports == PPTP) || (ports == QUAKEII_III)
				|| (ports == REAL_AUDIO) || (ports == TELNET) ||
		    	    (porte == AIM) || (porte == AGE_OF_EMPIRE) || (porte == FTP) || (porte == HTTP)  
				|| (porte == ICUII) || (porte == IP_PHONE) || (porte == NEETMEETING)
				|| (porte == NEWS) || (porte == PPTP) || (porte == QUAKEII_III)
				|| (porte == REAL_AUDIO) || (porte == TELNET)) {
				;
			}
			else {
  		   		ret +=websWrite(wp, "selected");
  			}   
				
		}		  
		else if ((ports == port) || (porte == port)) {
	  		ret +=websWrite(wp, "selected");
		}  
		return ret;
	}
	else if (strcmp(name, "proto") == 0) {
		c = nvram_get_current_fw_client(name);

		if (strcmp(c, select) == 0) {
			ret +=websWrite(wp, "selected");
		}	  
		return ret;
	}
	//DNI:Kevin:Add.06May3+++
	else if (strcmp(name, "service") == 0) {
		c = nvram_get_current_fw_client(name);

		if (strcmp(c, select) == 0) {
			ret +=websWrite(wp, "selected");
		}	  
		return ret;
	}
	//---
	return ret;	
}

static int ej_forward_selected_match(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s %s", &name, &select) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "proto") == 0) {
		c = nvram_get_current_forward(name);

		if (strcmp(c, select) == 0) {
			ret +=websWrite(wp, "selected");
		}	  
		return ret;
	}
	return ret;	
}

static int ej_qos_selected_match(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s %s", &name, &select) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "proto") == 0) {
		c = nvram_get_current_qos(name);

		if (strcmp(c, select) == 0) {
			ret +=websWrite(wp, "selected");
		}	  
		return ret;
	}
	return ret;	
}


static int ej_client_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "startp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_fw_client("port");

		/* Check for LAN IP address specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port0);
	}	
	else if (strcmp(name, "endp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_fw_client("port");

		/* Check for LAN IP address specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port1);
	}
	else if (strcmp(name, "service") == 0) {
		c = nvram_get_current_fw_client("service");		
		//ret +=websWrite(wp, "%s", c);
		ret += nvram_get_convert(wp, c);		
	}			

	return ret;	
}


static int ej_forward_port_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "startp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_forward("port");

		/* Check for private LAN start port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port0);
	}	
	else if (strcmp(name, "endp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_forward("port");

		/* Check for private LAN end port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port1);
	}
	else if (strcmp(name, "service") == 0) {
		//ret +=websWrite(wp, "%s", "VS Service");
		c = nvram_get_current_forward("service");		
		//ret += websWrite(wp, "%s", c);
		ret += nvram_get_convert(wp, c);		
		
	}
	else if (strcmp(name, "checked_enable") == 0) {
		c = nvram_get_current_forward("enable");
		
		if (strcmp(c, "on") == 0) {
			ret +=websWrite(wp, "%s", "checked");
		}	
	}				

	return ret;	
}

static int ej_trigger_port_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "i_startp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_trigger("i_port");

		/* Check for wan income start port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port0);
	}	
	else if (strcmp(name, "i_endp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_trigger("i_port");

		/* Check for wan income end port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port1);
	}
	if (strcmp(name, "o_startp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_trigger("o_port");

		/* Check for wan outgoing start port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port0);
	}	
	else if (strcmp(name, "o_endp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_trigger("o_port");

		/* Check for wan outgoing end port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port1);
	}	
	else if (strcmp(name, "i_proto") == 0) {
		c = nvram_get_current_trigger("i_proto");
		ret +=websWrite(wp, "%s", c);
	}
	else if (strcmp(name, "o_proto") == 0) {
		c = nvram_get_current_trigger("o_proto");		
		ret +=websWrite(wp, "%s", c);
	}
	else if (strcmp(name, "service") == 0) {
		c = nvram_get_current_trigger("service");		
		//ret +=websWrite(wp, "%s", c);
		ret += nvram_get_convert(wp, c);		
	}
	
	else if (strcmp(name, "enable") == 0) {
		c = nvram_get_current_trigger("enable");
		ret +=websWrite(wp, "%s", c);
	}	
	
	else if (strcmp(name, "checked_enable") == 0) {
		c = nvram_get_current_trigger("enable");
		if (strcmp(c, "on") == 0) {
			ret +=websWrite(wp, "%s", "checked");
		}	
	}				

	return ret;	
}


static int ej_static_route_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c;	
	int ret = 0;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "name") == 0) {
		c = nvram_get_current_static_route("desc");
		//ret +=websWrite(wp, "%s", c);
		ret += nvram_get_convert(wp, c);
	}	
	else if (strcmp(name, "ipaddr") == 0) {
		c = nvram_get_current_static_route("ipaddr");
		ret +=websWrite(wp, "%s", c);
	}
	else if (strcmp(name, "netmask") == 0) {
		c = nvram_get_current_static_route("netmask");
		ret +=websWrite(wp, "%s", c);
	}
	else if (strcmp(name, "gateway") == 0) {
		c = nvram_get_current_static_route("gateway");
		ret +=websWrite(wp, "%s", c);
	}
	else if (strcmp(name, "metric") == 0) {
		c = nvram_get_current_static_route("metric");
		ret +=websWrite(wp, "%s", c);
	}		
	else if (strcmp(name, "checked_valid") == 0) {
		c = nvram_get_current_static_route("enable");
		
		if (strcmp(c, "on") == 0)
			ret +=websWrite(wp, "%s", "checked");
	}
	else if (strcmp(name, "checked_private") == 0) {
		//c = nvram_get_current_static_route("interface");
		c = nvram_get_current_static_route("private");		
		
		if (strcmp(c, "1") == 0)
			ret +=websWrite(wp, "%s", "checked");
	}					

	return ret;	
}

static int ej_qos_port_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *select, *c;	
	int ret = 0;
	unsigned int port;
	char *port0, *port1;	
	unsigned int ports, porte;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "startp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_qos("port");

		/* Check for private LAN start port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port0);
	}	
	else if (strcmp(name, "endp") == 0) {
		//port = atoi(select);
		c = nvram_get_current_qos("port");

		/* Check for private LAN end port specification */
		port1 = c;
		port0 = strsep(&port1, "-");
		if (!port1)
			port1 = port0;
			
		ret +=websWrite(wp, "%s", port1);
	}
	else if (strcmp(name, "service") == 0) {
		//ret +=websWrite(wp, "%s", "VS Service");
		c = nvram_get_current_qos("service");		
		//ret += websWrite(wp, "%s", c);
		ret += nvram_get_convert(wp, c);		
		
	}
	else if (strcmp(name, "checked_enable") == 0) {
		c = nvram_get_current_qos("enable");
		
		if (strcmp(c, "on") == 0) {
			ret +=websWrite(wp, "%s", "checked");
		}	
	}				

	return ret;	
}

extern int dhcpc_res_cur_index;
static int ej_dhcpc_res_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c, str[200];	
	int ret = 0;
	char tmp[20];
	int i;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	//printf("%s = dhcpc_res_cur_index = %d\n", __FUNCTION__, dhcpc_res_cur_index);
	//if ( dhcpcRsvdAddrCnt == 0)
		//return;
	if (strcmp(name, "name") == 0) {
		//c = nvram_get_current_dhcpc_res("desc");//ATTACHED_DEVICES
		sprintf(str, _dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_host); 
		c = str;
		//strncpy(c, _dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_host, 34);//ATTACHED_DEVICES
		ret +=websWrite(wp, "%s", c);
	}	
	else if (strcmp(name, "ipaddr") == 0) {
		c = nvram_get_current_dhcpc_res("ipaddr");//ATTACHED_DEVICES
		//strncpy(c, _dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_ip, 20);//ATTACHED_DEVICES
		ret +=websWrite(wp, "%s", c);
	}
	else if (strcmp(name, "macaddr") == 0) {
		//c = nvram_get_current_dhcpc_res("macaddr");//ATTACHED_DEVICES
		//c = ether_etoa(_dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_mac, str);
		sprintf(str, _dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_mac); 
		c = str;
		//strncpy(c, _dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_mac, 20);//ATTACHED_DEVICES
		name = tmp;
		for (i = 0; i < strlen(c); i++) {
			if (c[i] != ':') 
				*name++	= c[i];
		}
		*name = 0;		
		c = tmp;	
		ret +=websWrite(wp, "%s", c);
	}

	return ret;	
}
		

static int
ej_checked_getblockdays(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *day, *c, *ptr;	
	int day_select, ret = 0;
	char tmp[20];
	
	if (ejArgs(argc, argv, "%s %s", &name, &day) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	day_select=atoi(day);
	
	strcpy(tmp, (char *)nvram_safe_get(name));
	c = tmp;
	do {	
		ptr = strsep(&c, ":,");
		if (day_select <= 0) {
			day_select = 0;
			break;
		}	
		day_select--;
	} while (ptr && c);	
	
	if ((day_select == 0) && (ptr) && (*ptr == '1')) {
	  ret +=websWrite(wp, "checked");
	}  
	
	return ret;
}	

static int
ej_selected_match(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *select, *c;	
	int ret = 0;
	
	if (ejArgs(argc, argv, "%s %s", &name, &select) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	c = nvram_safe_get(name);
	
	if (!strcmp(name, "wan_proto")) {
		if (!strcmp(c, select))
	  		ret +=websWrite(wp, "selected");					
	}
	else if (atoi(c) == atoi(select)) {	
	  ret +=websWrite(wp, "selected");
	}  
	
	return ret;
}	


static int
ej_field_nvram_get(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *select, *c, *sep;	
	int ret = 0, field = 0;
	char *ptr;
	char tmp[200];
	
	if (ejArgs(argc, argv, "%s %s %s", &name, &sep, &select) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	field = atoi(select);
	
	if (strcmp(name, "cur_forward_port_ip") == 0)
		strcpy(tmp, nvram_get_current_forward("ip"));
	else if (strcmp(name, "client_filter_ip") == 0)
		strcpy(tmp, nvram_get_current_fw_client("ip"));
	else if (strcmp(name, "client_filter_s_ip") == 0)
		strcpy(tmp, nvram_get_current_fw_client("ip_s"));
	else if (strcmp(name, "client_filter_e_ip") == 0) //pamela
		strcpy(tmp, nvram_get_current_fw_client("ip_e"));
	else if (strcmp(name, "cur_trigger_port_ip") == 0) //pamela
		strcpy(tmp, nvram_get_current_trigger("ip"));		
	else if (strcmp(name, "dhcpc_res_ip") == 0)
	{
		strcpy(tmp, nvram_get_current_dhcpc_res("ipaddr"));
		//sprintf(tmp, _dhcpc_rsvd_addr[dhcpc_res_cur_index].rsvd_ip);//ATTACHED_DEVICES
	}
	else	
		strcpy(tmp, (char *)nvram_safe_get(name));
		
	c= tmp;
	do {	
		ptr = strsep(&c, sep);
		if (field <= 0) 
			field = 0;
		else 
			field--;
		if (ptr && (field == 0)) {
			ret +=websWrite(wp, "%s", ptr);	
			break;
		}	
	} while ((ptr != NULL) && (field != 0));	
	

	return ret;
}	

static int
ej_vs_lan_mask(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *lan_ip, *lan_mask, *c, *ptr;	
	int ret = 0;
	char tmp[20];
	unsigned char ip_addr[4]={0}, ip_netmask[4]={0};

	lan_ip=nvram_safe_get("lan_ipaddr");
	strcpy(tmp, lan_ip);
	lan_mask=nvram_safe_get("lan_netmask");
	
	c = tmp;
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_addr[0] = atoi(ptr);
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_addr[1] = atoi(ptr);		
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_addr[2] = atoi(ptr);		
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_addr[3] = atoi(ptr);	
		
	strcpy(tmp, lan_mask);
	c = tmp;
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_netmask[0] = atoi(ptr);
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_netmask[1] = atoi(ptr);		
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_netmask[2] = atoi(ptr);		
	ptr = strsep(&c, ".");
	if (ptr != NULL)
		ip_netmask[3] = atoi(ptr);	
		
	ip_addr[0] &= ip_netmask[0];
	ip_addr[1] &= ip_netmask[1];
	ip_addr[2] &= ip_netmask[2];
	ip_addr[3] &= ip_netmask[3];	
	sprintf(tmp, "%d.%d.%d.%d", ip_addr[0], ip_addr[1], ip_addr[2], ip_addr[3]);							
	
	ret = websWrite( wp, "%s", tmp );	
	return ret;		
}	


/* d-d>ip:d-d,proto,on/off,desc */
static int
ej_forward_port_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0, field = 0;
	char *ptr;
	char tmp[200];
	static int index = 0;
	char *wan_port0, *wan_port1, *lan_ipaddr, *lan_port0, *lan_port1, *proto, *enable, *desc;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	if (strcmp(name, "forward_port0") == 0)
		index = 0;
	ptr = nvram_safe_get(name);
	if (!*ptr)
		return 0;
		
	strcpy(tmp, ptr);
	/* Check for wan income port specification */
	lan_ipaddr = tmp;
	wan_port0 = strsep(&lan_ipaddr, ">");
	if (!lan_ipaddr)
		return 0;

	/* Check for LAN destination port specification */
	lan_port0 = lan_ipaddr;
	lan_ipaddr = strsep(&lan_port0, ":");
	if (!lan_port0)
		return 0;

	/* Check for protocol specification */
	proto = lan_port0;
	lan_port0 = strsep(&proto, ":,");
	if (!proto)
		return 0;

	/* Check for enable specification */
	enable = proto;
	proto = strsep(&enable, ":,");
	if (!enable)
		return 0;

	/* Check for description specification (optional) */
	desc = enable;
	enable = strsep(&desc, ":,");

	/* Check for WAN destination port range (optional) */
	wan_port1 = wan_port0;
	wan_port0 = strsep(&wan_port1, "-");
	if (!wan_port1)
		wan_port1 = wan_port0;

	/* Check for LAN destination port range (optional) */
	lan_port1 = lan_port0;
	lan_port0 = strsep(&lan_port1, "-");
	if (!lan_port1)
		lan_port1 = lan_port0;

#if 0		
	ret+=websWrite(wp, "<tr>");
	ret+=websWrite(wp, "<td>&nbsp;</td>");	
	ret+=websWrite(wp, "<td>#</td>");	
	ret+=websWrite(wp, "<td>%s</td>", enable);	
	ret+=websWrite(wp, "<td>%s</td>", desc);	
	ret+=websWrite(wp, "<td>%s</td>", lan_port0);
	ret+=websWrite(wp, "<td>%s</td>", lan_port1);
	ret+=websWrite(wp, "<td>%s</td>", lan_ipaddr);			
	ret+=websWrite(wp, "</tr>");	
	
	return ret;
#endif
	
	if (index==0) {	
		ret += websWrite(wp, "<tr><td width=\"3%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"R1\" onClick=\"check(this.value)\"></td><td width=\"5%%\" align=\"center\">%d</td>", index, "checked", index+1);
	}	
	else {
		ret += websWrite(wp, "<tr><td width=\"3%%\" align=\"center\"><input type=\"radio\" value=\"%d\" name=\"R1\" onClick=\"check(this.value)\"></td><td width=\"5%%\" align=\"center\">%d</td>", index, index+1);
	}	
#if !defined(NEW_WEB_PAGES)	
	ret += websWrite(wp, "<td width=\"5%%\" align=\"center\">");
	if (strcmp(enable, "on") == 0) {
		ret += websWrite(wp, "<input type=\"CHECKBOX\" name=\"VirSerActEnt\" %s><input type=\"hidden\" name=\"VirActIn\" value=%d></td>", "checked",index);
	}	
	else {
		ret += websWrite(wp, "<input type=\"CHECKBOX\" name=\"VirSerActEnt\" %s><input type=\"hidden\" name=\"VirActIn\" value=%d></td>", "",index);
	}	
#else	
	
#endif				
	ret += websWrite(wp, "<td width=\"25%%\" align=\"center\">");
	if (strlen(desc) == 0) {
		ret += websWrite(wp, "%s</td>", "&nbsp");
	}	
	else {
		ret += websWrite(wp, "%s</td>", desc);
	}	
	//ret += websWrite(wp, "<td width=\"17%%\" align=\"center\">%s</td><td width=\"15%%\" align=\"center\">%s</td><td width=\"30%%\" align=\"center\">%s</td>", lan_port0, lan_port1, lan_ipaddr);				
	ret += websWrite(wp, "<td width=\"17%%\" align=\"center\">%s</td><td width=\"15%%\" align=\"center\">%s</td><td width=\"30%%\" align=\"center\">%s</td>", wan_port0, wan_port1, lan_ipaddr);				
	ret += websWrite(wp, "</tr>");
	
	index++;
				
	return ret;
	
}

/* d-d,proto,on/off,priority(0/1),desc */
static int
ej_qos_port_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0, field = 0;
	char *ptr;
	char tmp[200];
	static int index = 0;
	char *port0, *port1, *proto, *enable, *priority, *desc;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	if (strcmp(name, "qos_port0") == 0)
		index = 0;
	ptr = nvram_safe_get(name);
	if (!*ptr)
		return 0;
		
	strcpy(tmp, ptr);

	proto = tmp;
	port0 = strsep(&proto, ":,");
	if (!proto)
		return 0;

	/* Check for enable */
	enable = proto;
	proto = strsep(&enable, ":,");
	if (!enable)
		return 0;
		
	/* Check for priority (optional) */
	priority = enable;
	enable = strsep(&priority, ":,");		
	if (!priority)
		return 0;

	/* Check for description specification (optional) */
	desc = priority;
	priority = strsep(&desc, ":,");

	/* Check for port range (optional) */
	port1 = port0;
	port0 = strsep(&port1, "-");
	if (!port1)
		port1 = port0;

	if (index==0) {	
		ret += websWrite(wp, "<tr><td width=\"3%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"R1\" onClick=\"check(this.value)\"></td><td width=\"5%%\" align=\"center\">%d</td>", index, "checked", index+1);
	}	
	else {
		ret += websWrite(wp, "<tr><td width=\"3%%\" align=\"center\"><input type=\"radio\" value=\"%d\" name=\"R1\" onClick=\"check(this.value)\"></td><td width=\"5%%\" align=\"center\">%d</td>", index, index+1);
	}	
			
	ret += websWrite(wp, "<td width=\"25%%\" align=\"center\">");
	if (strlen(desc) == 0) {
		ret += websWrite(wp, "%s</td>", "&nbsp");
	}	
	else {
		ret += websWrite(wp, "%s</td>", desc);
	}	
	//ret += websWrite(wp, "<td width=\"17%%\" align=\"center\">%s</td><td width=\"15%%\" align=\"center\">%s</td><td width=\"30%%\" align=\"center\">%s</td>", lan_port0, lan_port1, lan_ipaddr);				
	ret += websWrite(wp, "<td width=\"17%%\" align=\"center\">%s</td><td width=\"15%%\" align=\"center\">%s</td>", port0, port1);				
	ret += websWrite(wp, "</tr>");
	
	index++;
				
	return ret;
	
}


//#define MAX_SPECIAL_AP		20 //pamela20060106
/* #defined in board.h */

/* d-d,proto>ip:d-d,proto,on/off,used,desc */

/* outbound, inbound */
static int
ej_trigger_port_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0, field = 0;
	char *ptr;
	char tmp[200];
	static int index = 0;
	char *wan_port0, *wan_port1, *wan_proto, *lan_ipaddr, *lan_port0, *lan_port1, *proto, *enable, *desc, *used;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	if (strcmp(name, "trigger_port0") == 0)
		index = 0;
	ptr = nvram_safe_get(name);
	if (!*ptr)
		return 0;
		
	strcpy(tmp, ptr);
	/* Check for WAN PORT specification */
		
	wan_proto = tmp;
	wan_port0 = strsep(&wan_proto, ",");
	if (!wan_proto)
		return 0;
		
	lan_ipaddr = wan_proto;
	wan_proto = strsep(&lan_ipaddr, ">");
	if (!lan_ipaddr)
		return 0;		

	/* Check for LAN destination port specification */
	lan_port0 = lan_ipaddr;
	lan_ipaddr = strsep(&lan_port0, ":");
	if (!lan_port0)
		return 0;

	/* Check for protocol specification */
	proto = lan_port0;
	lan_port0 = strsep(&proto, ":,");
	if (!proto)
		return 0;

	/* Check for enable specification */
	enable = proto;
	proto = strsep(&enable, ":,");
	if (!enable)
		return 0;

	/* Check for used specification (optional) */
	used = enable;
	enable = strsep(&used, ":,");
	if (!used)
		return 0;
	
	/* Check for description (optional) */
	desc = used;
	used = strsep(&desc, ":,");	
	if (!desc)
		return 0;

	/* Check for WAN destination port range (optional) */
	wan_port1 = wan_port0;
	wan_port0 = strsep(&wan_port1, "-");
	if (!wan_port1)
		wan_port1 = wan_port0;

	/* Check for LAN destination port range (optional) */
	lan_port1 = lan_port0;
	lan_port0 = strsep(&lan_port1, "-");
	if (!lan_port1)
		lan_port1 = lan_port0;

#if 0		
	ret+=websWrite(wp, "<tr>");
	ret+=websWrite(wp, "<td>&nbsp;</td>");	
	ret+=websWrite(wp, "<td>#</td>");	
	ret+=websWrite(wp, "<td>%s</td>", enable);	
	ret+=websWrite(wp, "<td>%s</td>", desc);	
	ret+=websWrite(wp, "<td>%s</td>", lan_port0);
	ret+=websWrite(wp, "<td>%s</td>", lan_port1);
	
	if (strcmp(lan_ipaddr, "0.0.0.0") == 0)  
		ret+=websWrite(wp, "<td>%s</td>", "ANY");	
	else
		ret+=websWrite(wp, "<td>%s</td>", lan_ipaddr);					
	ret+=websWrite(wp, "</tr>");	
	
	return ret;
#endif

	if (index==0) {
		ret += websWrite(wp, "<tr><td width=\"8%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"R1\" onClick=\"check(this.value)\"></td><td width=\"5%%\" align=\"center\">%d</td>", index, "checked", index+1);
	}
	else {

		ret += websWrite(wp, "<tr><td width=\"8%%\" align=\"center\"><input type=\"radio\" value=\"%d\" name=\"R1\" onClick=\"check(this.value)\"></td><td width=\"5%%\" align=\"center\">%d</td>", index, index+1);
	}
	ret += websWrite(wp, "<td width=\"11%%\" align=\"center\">");

	if (strcmp(enable, "on") == 0) {
		ret += websWrite(wp, "<input type=\"CHECKBOX\" name=\"ProTrEntAct\" %s><input type=\"hidden\" name=\"TriEntrNum\" value=%d></td>", "checked",index);
	}	
	else {
		ret += websWrite(wp, "<input type=\"CHECKBOX\" name=\"ProTrEntAct\" %s><input type=\"hidden\" name=\"TriEntrNum\" value=%d></td>", "",index);
	}	
				
	ret += websWrite(wp, "<td width=\"11%%\" align=\"center\">");
	if (strlen(desc) == 0) {	
		ret += websWrite(wp, "%s</td>", "&nbsp");
	}	
	else {
		ret += websWrite(wp, "%s</td>", desc);
	}			
	
#if !defined(NEW_WEB_PAGES)	
	ret += websWrite(wp, "<td width=\"11%%\" align=\"center\">");
	if (strcmp(used, "on") == 0) {
		ret += websWrite(wp, "%s</td>", "NO");
	}	
	else {
		ret += websWrite(wp, "%s</td>", "YES");
	}	
	ret += websWrite(wp, "<td width=\"9%%\" align=\"center\">%s</td><td width=\"9%%\" align=\"center\">%s</td><td width=\"9%%\" align=\"center\">%s</td><td width=\"9%%\" align=\"center\">%s</td><td width=\"18%%\" align=\"center\">%s</td>", wan_port0, wan_port1, lan_port0, lan_port1, lan_ipaddr);				
	ret += websWrite(wp, "</tr>");	
#else
	ret += websWrite(wp, "<td width=\"23%%\" align=\"center\">");
	if (strcmp(wan_proto, "both") == 0) {
		ret += websWrite(wp, "%s:%s</td>", "TCP/UDP", wan_port0);
	}	
	else {
		ret += websWrite(wp, "%s:%s</td>", (wan_proto[0] == 't' ? "TCP" : "UDP"), wan_port0);
	}	
	ret += websWrite(wp, "<td width=\"23%%\" align=\"center\">");	
	if (strcmp(proto, "both") == 0) {
		if (!strcmp(lan_port0, lan_port1)) {
			ret += websWrite(wp, "%s:%s</td>", "TCP/UDP", lan_port0);
		}	
		else {		
			ret += websWrite(wp, "%s:%s..%s</td>", "TCP/UDP", lan_port0, lan_port1);
		}	
	}	
	else {
		if (!strcmp(lan_port0, lan_port1)) {
			ret += websWrite(wp, "%s:%s</td>", (proto[0] == 't' ? "TCP" : "UDP"), lan_port0);			
		}	
		else {
			ret += websWrite(wp, "%s:%s..%s</td>", (proto[0] == 't' ? "TCP" : "UDP"), lan_port0, lan_port1);
		}	
	}		
	
	ret += websWrite(wp, "<td width=\"24%%\" align=\"center\">%s</td>", (lan_ipaddr[0] == '0') ? "ANY" : lan_ipaddr);
				
	ret += websWrite(wp, "</tr>");
#endif	
	index++;	
			
	return ret;
}


//#define MAX_ACL				8
/* #defined in board.h */
static int
ej_filter_client_list(int eid, webs_t wp, int argc, char_t **argv)
{
	char *ptr, *name;
	int ret = 0;
	char *lan_ipaddr0, *lan_ipaddr1, *lan_port0, *lan_port1, *proto;
	char *day_start, *day_end, *sec_start, *sec_end;
	char *enable, *desc;
	char tmp[200];
	static int index = 0;	
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	if (strcmp(name, "filter_client0") == 0)
		index = 0;
	ptr = nvram_safe_get(name);
	if (!(*ptr))
		return 0;

	printf("%s = %s\n", name, ptr);		
	strcpy(tmp, ptr);
	/* Check for port specification */
	lan_port0 = tmp;
	lan_ipaddr0 = strsep(&lan_port0, ":");
	if (!lan_port0)
		return 0;

	/* Check for protocol specification */
	proto = lan_port0;
	lan_port0 = strsep(&proto, ",");
	if (!proto)
		return 0;

	/* Check for day specification */
	day_start = proto;
	proto = strsep(&day_start, ",");
	if (!day_start)
		return 0;

	/* Check for time specification */
	sec_start = day_start;
	day_start = strsep(&sec_start, ":");
	if (!sec_start)
		return 0;

	/* Check for enable specification */
	enable = sec_start;
	sec_start = strsep(&enable, ",");
	if (!enable)
		return 0;

	/* Check for description specification (optional) */
	desc = enable;
	enable = strsep(&desc, ",");

	/* Check for address range (optional) */
	lan_ipaddr1 = lan_ipaddr0; 
	lan_ipaddr0 = strsep(&lan_ipaddr1, "-");
	if (!lan_ipaddr1)
		lan_ipaddr1 = lan_ipaddr0;

	/* Check for port range (optional) */
	lan_port1 = lan_port0;
	lan_port0 = strsep(&lan_port1, "-");
	if (!lan_port1)
		lan_port1 = lan_port0;

	/* Check for day range (optional) */
	day_end = day_start;
	day_start = strsep(&day_end, "-");
	if (!day_end)
		day_end = day_start;

	/* Check for time range (optional) */
	sec_end = sec_start;
	sec_start = strsep(&sec_end, "-");
	if (!sec_end)
		sec_end = sec_start;

	{
		char checked[10];
		
	        if (index==0)
		      strcpy(checked, "checked");
                else
                      strcpy(checked, "");				

#if !defined(NEW_WEB_PAGES)
		if (strcmp(enable, "on") == 0) {
		    ret += websWrite(wp, "<tr><td nowrap width=\"7%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"X1\" onclick=\"checkr1(this.value)\"></td><td width=\"7%%\" align=\"center\">%d</td><td nowrap width=\"7%%\" align=\"center\"><input type=\"checkbox\" value=\"%d\" checked name=\"ChSer\"><input type=\"hidden\" name=\"SerIn\" value=%d><td nowrap width=\"35%%\" align=\"center\">%s</td>", index, checked, index+1,index,index, desc); 
		} 
		else {
		    ret += websWrite(wp, "<tr><td nowrap width=\"7%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"X1\" onclick=\"checkr1(this.value)\"></td><td width=\"7%%\" align=\"center\">%d</td><td nowrap width=\"7%%\" align=\"center\"><input type=\"checkbox\" value=\"%d\"  name=\"ChSer\"><input type=\"hidden\" name=\"SerIn\" value=%d><td nowrap width=\"42%%\" align=\"center\">%s</td>", index, checked, index+1,index, index, desc);     
		}    
#else
		ret += websWrite(wp, "<tr><td nowrap width=\"7%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"X1\" onclick=\"checkr1(this.value)\"></td><td width=\"7%%\" align=\"center\">%d</td><td nowrap width=\"35%%\" align=\"center\">%s</td>", index, checked, index+1, desc);     
#endif
		if (!strcmp(lan_port0, lan_port1)) {
			ret += websWrite(wp, "<td nowrap width=\"22%%\" align=\"center\">%s</td>", lan_port0);		
		}	
		else {	
			ret += websWrite(wp, "<td nowrap width=\"22%%\" align=\"center\">%s..%s</td>", lan_port0, lan_port1);
		}	
		if (strcmp(lan_ipaddr0, "0.0.0.0") == 0) {
			ret += websWrite(wp, "<td nowrap width=\"22%%\" align=\"center\">%s</td></tr>", "all");
		}
		else if(!strcmp(lan_ipaddr0, lan_ipaddr1)){ //pamela20060104
			ret += websWrite(wp, "<td nowrap width=\"22%%\" align=\"center\">%s</td></tr>", lan_ipaddr0);			
		}
		else {	
			ret += websWrite(wp, "<td nowrap width=\"22%%\" align=\"center\">%s-%s</td></tr>", lan_ipaddr0, lan_ipaddr1);
		}	
	}
	index++;
	
	return ret;	
}

//#define MAX_ROUTING		5
/* #defined in board.h */

/* ipaddr,netmask,gateway,on/off,rip_private,desc */

static int
ej_static_route_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name;	
	int ret = 0;
	char *ptr;
	char tmp[200];
	static int index = 0;
	char *ipaddr, *netmask, *gateway, *enable, *desc,*pri,*metric, *rip_private;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	if (strcmp(name, "static_route0") == 0)
		index = 0;
	ptr = nvram_safe_get(name);
	if (!*ptr)
		return 0;
		
	strcpy(tmp, ptr);
	/* Check for DIP specification */
	netmask = tmp;
	ipaddr = strsep(&netmask, ",");
	if (!ipaddr)
		return 0;

	gateway = netmask;
	netmask = strsep(&gateway, ",");
	if (!netmask)
		return 0;

	/* Check for gateway specification */
	metric = gateway;
	gateway = strsep(&metric, ",");
	if (!gateway)
		return 0;

	pri = metric;
	metric = strsep(&pri, ",");
	if (!gateway)
		return 0;

	/* Check for interface */
	enable = pri;
	pri = strsep(&enable, ",");
	if (!pri)
		return 0;
	
	/* Check for enable specification */
	rip_private = enable;
	enable = strsep(&rip_private, ",");
	if (!enable)
		return 0;
		
	/* Check for description specification */
	desc = rip_private;
	rip_private = strsep(&desc, ",");
	if (!desc)
		return 0;		

	if (index == 0) {
		ret += websWrite(wp, "<tr><td width=\"5%%\" align=\"center\"><span class=\"thead\"><input type=\"radio\" name=\"R1\" value=\"%d\"  %s></span> </td><td width=\"5%%\" align=\"center\">",index, "checked");						
	}	
	else {
		ret += websWrite(wp, "<tr><td width=\"5%%\" align=\"center\"><span class=\"thead\"><input type=\"radio\" name=\"R1\" value=\"%d\"  %s></span> </td><td width=\"5%%\" align=\"center\">",index, "");						
	
	}	
	if (strcmp(enable, "on") == 0) {		
		ret += websWrite(wp, "<span class=\"thead\">%d</span></td><td width=\"10%%\" align=\"center\"><span class=\"ttext\">%s</span></td>",index+1, "yes");			
	}	
	else {
		ret += websWrite(wp, "<span class=\"thead\">%d</span></td><td width=\"10%%\" align=\"center\"><span class=\"ttext\">%s</span></td>",index+1, "no");			
	}	
					
	ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">%s</span></td><td width=\"25%%\" align=\"center\">", desc);
	ret += websWrite(wp, "<span class=\"ttext\">%s</span></td><td width=\"25%%\" align=\"center\">", ipaddr);
	ret += websWrite(wp, "<span class=\"ttext\">%s</span></td></tr>", gateway);
	
	index++;
	
	return ret;
}

#if 0
static int
ej_dhcpc_res_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0, field = 0;
	char *ptr;
	char tmp[100];
	static int index = 0;
	char *ipaddr, *mac, *desc;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	if (strcmp(name, "dhcpc_res0") == 0)
		index = 0;
	ptr = nvram_safe_get(name);
	if (!*ptr)
		return 0;	
	
	strcpy(tmp, ptr);	
	
	/* Check for ipaddr address specification */
	mac = tmp;
	ipaddr = strsep(&mac, ",");
	if (!ipaddr)
		return 0;
		
	desc = mac;
	mac = strsep(&desc, ",");
	if (!mac)
		return 0;
	
	
	ret += websWrite(wp, "<tr><td nowrap width=\"5%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"X1\" onClick=\"check(this.value)\"></td><td nowrap width=\"3%%\" align=\"center\">%d</td><td nowrap width=\"24%%\" align=\"center\">%s</td>",index, ((index==0)? "checked" : ""), index+1, ipaddr);
			
	//MacAddress
	if( !desc || (strlen(desc) == 0)) {
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">%s</td>", "&nbsp");
	}	
	else {
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">%s</td>", desc);
	}	
	ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">");
	ret += websWrite(wp, "%s</td></tr>", mac);
	
	index++;
	
	return ret;
}
#endif

static int
ej_dhcpc_res_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	char res_count[3]={0};
	int ret = 0, field = 0, i = 0, count = 0;
	char *ptr;
	char tmp[100], nvram_name[20];
	int index = 0;
	char *ipaddr, *mac, *desc;
	dhcpcRsvdAddrCnt = 0;
	
#if 0
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		//return 0;
	}
#endif

	strcpy(res_count, nvram_get("dhcpc_res_count"));
	count = atoi(res_count);

	for (i = 0; i < count; i++) {
		sprintf(nvram_name, "dhcpc_res%d", i);
		ptr = nvram_safe_get(nvram_name);
		//printf("ptr = %s\n", ptr);
		if ((*ptr) == 0)
			continue;

		strcpy(tmp, ptr);
		mac = tmp;
		ipaddr = strsep(&mac, ",");
		//printf("1 - mac = %s\n", mac);
		//printf("ipaddr = %s\n", ipaddr);
		if (!ipaddr)
			continue;
		
		desc = mac;
		mac = strsep(&desc, ",");
		//printf("desc = %s\n", desc);
		//printf("mac = %s\n", mac);
		if (!mac)
			continue;

		//if ( !ipaddr || !mac )
			//continue;
		//else {
		if( !desc || (strlen(desc) == 0)) 
				strcpy(desc, "&nbsp;");
		
		ret += websWrite(wp, "<tr><td nowrap width=\"5%%\" align=\"center\"><input type=\"radio\" name=\"X1\" onClick=\"check(\'%d\')\"></td><td nowrap width=\"3%%\" align=\"center\">%d</td><td nowrap width=\"24%%\" align=\"center\">%s</td>", i, index+1, ipaddr);
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">%s</td>", desc);
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">");
		ret += websWrite(wp, "%s</td></tr>", mac);
		index++;
		

		memset(_dhcpc_rsvd_addr[i].rsvd_host, '\0', 34);
		memset(_dhcpc_rsvd_addr[i].rsvd_mac, '\0', 20);
		memset(_dhcpc_rsvd_addr[i].rsvd_ip, '\0', 20);
		memcpy(_dhcpc_rsvd_addr[i].rsvd_host, desc, strlen(desc));
		memcpy(_dhcpc_rsvd_addr[i].rsvd_mac, mac, strlen(mac));
		memcpy(_dhcpc_rsvd_addr[i].rsvd_ip, ipaddr, strlen(ipaddr));
		dhcpcRsvdAddrCnt++;	
	}

	return ret;
}

// ATTACHED_DEVICES
#if 1
int refresh_dhcpc_list(void) {

	FILE *fp;	
	struct in_addr addr;
	char sigusr1[] = "-XX";
	int found = 0;
	char line[100];
	unsigned char ipa[20]="", mac[20]="", host[34] = "";
	int pid, result = 0;//, count = 0;
	atchedDvisCnt = 0;
	non_atchedDvisCnt = 0;
	/* add the following additional variables due to the change from scanport_SendArpPkt( ) to scanport_arpping( ) */
	int i = 0, j = 0;
	//unsigned char yiaddr[20]="";
	//unsigned char sHadrr[20]="";
	unsigned char lan_ipaddr[20]="";
	unsigned char lan_hwaddr[20];
	unsigned char _lan_hwaddr[20];
	unsigned char dev[5]="";
	unsigned char tmp[5], tmp1[6];//, *value;
	int value = 0;
		
	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR2);    
		sleep(1);		/* waiting writed completed */
	}
	else {
		found = 0;
		return found;
	}			
	
	memset(dev, 0, 4);
	strncpy(dev, "eth1", 4);
	strcpy(lan_ipaddr, nvram_get("lan_ipaddr"));
	strcpy(lan_hwaddr, nvram_get("lan_hwaddr"));
	j = 0;
	for ( i = 0; i < strlen(lan_hwaddr); i++) {
		if ( (i  !=  2) && (i  !=  5) && (i  !=  8) && ( i  !=  11) && ( i  !=  14)) {
			_lan_hwaddr[j] = lan_hwaddr[i];
			DPRINTF("* _lan_hwaddr[%d] = %c\n", j,  _lan_hwaddr[j]);
			j++;
		}
	}

	j = 0;
	for ( i = 0; i < 12; i++) {
		value = 0;
		sprintf(tmp, "%c%c", _lan_hwaddr[i], _lan_hwaddr[i+1]);
		value = strtol(tmp, NULL, 16);
		tmp1[j++] = value;
		i++;
	}	
	
	if ((fp = fopen(DHCPD_LEASES_PATH "dhcps_client", "r")) == NULL)
		printf("unable to open file\n");
	else {
		while( fgets(line, sizeof(line), fp)!=NULL) {
			result = 0;
			strcpy(mac, " ");		
			strcpy(ipa, " ");
			strcpy(host, " ");
			
			if(sscanf(line, "%s %s %s\n", mac, ipa, host) != 3 )
				continue;
					
			//printf("\n1-mac = %s; ipa = %s; host = %s; dev = %s\n", mac, ipa, host, dev);
			//strncpy(yiaddr, ipa, strlen(ipa));
			//strncpy(sHadrr, mac, strlen(mac));
			//for (count = 0; count < 2; count++) {
			//while(count < 2) {
				//count++;
				//result = !scanport_arpping(yiaddr, lan_ipaddr, tmp1, dev, sHadrr);
				result = !scanport_arpping(ipa, lan_ipaddr, tmp1, dev, mac);
				
				//if ( result == 1)
					//break;
			//}
			//count = 0;
			if(sscanf(line, "%s %s %s\n", mac, ipa, host) != 3 )
				continue;
			//printf("\n2-mac = %s; ipa = %s; host = %s\n", mac, ipa, host);
			DPRINTF("result = %d\n", result);
			if ( (result == 1) && (atchedDvisCnt < MAX_ATTACHED_DEVICES)) {
			     found = 1;
			     if ( result ) {
				memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
				memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
				memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
				memcpy(_attached_devices[atchedDvisCnt].attached_host, host, strlen(host));
				memcpy(_attached_devices[atchedDvisCnt].attached_mac, mac, strlen(mac));
				memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
				//printf("_attached_devices[%d].attached_host = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_host);        
				//printf("_attached_devices[%d].attached_mac  = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_mac);	
 				//printf("_attached_devices[%d].attached_ip   = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_ip);
				atchedDvisCnt++;
				if (atchedDvisCnt >= MAX_ATTACHED_DEVICES)
					break;		/* break while */
			    }
			} else if( (result == 0) && (non_atchedDvisCnt < MAX_ATTACHED_DEVICES)) {
				memset(_non_attached_devices[non_atchedDvisCnt].attached_host, '\0', 34);
				memset(_non_attached_devices[non_atchedDvisCnt].attached_mac, '\0', 20);
				memset(_non_attached_devices[non_atchedDvisCnt].attached_ip, '\0', 20);
				memcpy(_non_attached_devices[non_atchedDvisCnt].attached_host, host, strlen(host));
				memcpy(_non_attached_devices[non_atchedDvisCnt].attached_mac, mac, strlen(mac));
				memcpy(_non_attached_devices[non_atchedDvisCnt].attached_ip, ipa, strlen(ipa));
				//printf("_non_attached_devices[%d].attached_host = %s\n", non_atchedDvisCnt, _non_attached_devices[non_atchedDvisCnt].attached_host);        
				//printf("_non_attached_devices[%d].attached_mac  = %s\n", non_atchedDvisCnt, _non_attached_devices[non_atchedDvisCnt].attached_mac);	
 				//printf("_non_attached_devices[%d].attached_ip   = %s\n", non_atchedDvisCnt, _non_attached_devices[non_atchedDvisCnt].attached_ip);
				non_atchedDvisCnt++;
				if (non_atchedDvisCnt >= MAX_ATTACHED_DEVICES)
					break;		/* break while */
			}else
				break;			
	    	}	/* while */
	}
	if (fp)
		fclose(fp);
	
	return found;		
}
#endif

#if 0
int refresh_dhcpc_list() {

	FILE *fp;	
	int i = 0, len = 0;
	struct in_addr addr;
	char sigusr1[] = "-XX";
	int found = 0;
	char line[100], ipa[20]="",mac[20]="";
	char host[34] = "";
	int pid, result = 0;//, count = 0;
	unsigned char argv[8][20];
	atchedDvisCnt = 0;
	non_atchedDvisCnt = 0;

	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR2);    
		sleep(1);		/* waiting writed completed */
	}

	if ((fp = fopen(DHCPD_LEASES_PATH "dhcps_client", "r")) == NULL)
		printf("unable to open file\n");
	else {
		while( fgets(line, sizeof(line), fp)!=NULL) {
			result = 0;
			strcpy(mac, " ");		
			strcpy(ipa, " ");
			strcpy(host, " ");
			
			if(sscanf(line, "%s %s %s\n", mac, ipa, host) != 3 )
				continue;
					
			strcpy(argv[1], "eth1");
			strcpy(argv[7], mac);
			strcpy(argv[8], ipa);
			//printf("mac = %s; ipa = %s; host = %s\n", mac, ipa, host);
			//for (count = 0; count < 2; count++) {
			//while(count < 2) {
				//count++;
				result = scanport_SendArpPkt(argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
				//if ( result == 1)
					//break;
			//}
			//count = 0;
			//printf("result = %d\n", result);
			if ( (result == 1) && (atchedDvisCnt < MAX_ATTACHED_DEVICES)) {
			     found = 1;
			     if ( result ) {
				memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
				memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
				memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
				memcpy(_attached_devices[atchedDvisCnt].attached_host, host, strlen(host));
				memcpy(_attached_devices[atchedDvisCnt].attached_mac, mac, strlen(mac));
				memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
				//printf("_attached_devices[%d].attached_host = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_host);        
				//printf("_attached_devices[%d].attached_mac  = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_mac);	
 				//printf("_attached_devices[%d].attached_ip   = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_ip);
				atchedDvisCnt++;
				if (atchedDvisCnt >= MAX_ATTACHED_DEVICES)
					break;		/* break while */
			    }
			} else if( (result == 0) && (non_atchedDvisCnt < MAX_ATTACHED_DEVICES)) {
				memset(_non_attached_devices[non_atchedDvisCnt].attached_host, '\0', 34);
				memset(_non_attached_devices[non_atchedDvisCnt].attached_mac, '\0', 20);
				memset(_non_attached_devices[non_atchedDvisCnt].attached_ip, '\0', 20);
				memcpy(_non_attached_devices[non_atchedDvisCnt].attached_host, host, strlen(host));
				memcpy(_non_attached_devices[non_atchedDvisCnt].attached_mac, mac, strlen(mac));
				memcpy(_non_attached_devices[non_atchedDvisCnt].attached_ip, ipa, strlen(ipa));
				//printf("_non_attached_devices[%d].attached_host = %s\n", non_atchedDvisCnt, _non_attached_devices[non_atchedDvisCnt].attached_host);        
				//printf("_non_attached_devices[%d].attached_mac  = %s\n", non_atchedDvisCnt, _non_attached_devices[non_atchedDvisCnt].attached_mac);	
 				//printf("_non_attached_devices[%d].attached_ip   = %s\n", non_atchedDvisCnt, _non_attached_devices[non_atchedDvisCnt].attached_ip);
				non_atchedDvisCnt++;
				if (non_atchedDvisCnt >= MAX_ATTACHED_DEVICES)
					break;		/* break while */
			}else
				break;			
	    	}	/* while */
	}
	if (fp)
		fclose(fp);
	
	return found;		
}
#endif
#if 0
int refresh_dhcpc_list() {

	FILE *fp;
	//struct lease_t lease, leases[254];
	struct lease_t lease;	
	int i = 0, len = 0;
	struct in_addr addr;
	unsigned long expires;
	char sigusr1[] = "-XX";
	int found = 0;
	char *ipaddr, ipa[20]="",mac[20]="", expires_time[50]="";
	char host[34] = "";
	int pid, result = 0, count = 0;
	unsigned char argv[8][20];
	atchedDvisCnt = 0;
	
	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR1);    
		sleep(1);		/* waiting writed completed */
	}

	if ((fp = fopen(DHCPD_LEASES_PATH "udhcpd.leases", "r+")) == NULL)
		printf("unable to open file\n");
	else {
		while (fread(&lease, sizeof(lease), 1, fp)) {
			printf("%d = atchedDvisCnt =%d\n", __LINE__, atchedDvisCnt);
			strcpy(mac,"");
			for (i = 0; i < 6; i++) {
				sprintf(mac+strlen(mac),"%02X", lease.chaddr[i]);
				if (i != 5) sprintf(mac+strlen(mac),":");
			}
			mac[17] = '\0';
			if(!strcmp(mac,"00:00:00:00:00:00"))
				continue;

			addr.s_addr = lease.yiaddr;

			ipaddr = inet_ntoa(addr);

			expires = ntohl(lease.expires);

			strcpy(expires_time,"");
			if (!expires){
				continue;
				strcpy(expires_time,"expired");
			}
			else {
				if (expires > 60*60*24) {
					sprintf(expires_time+strlen(expires_time),"%ld days, ",expires / (60*60*24));
					expires %= 60*60*24;
				}
				if (expires > 60*60) {
					sprintf(expires_time+strlen(expires_time),"%02ld:",expires / (60*60));	// hours
					expires %= 60*60;
				}
				else{
					sprintf(expires_time+strlen(expires_time),"00:");	// no hours
				}
				if (expires > 60) {
					sprintf(expires_time+strlen(expires_time),"%02ld:",expires / 60);	// minutes
					expires %= 60;
				}
				else{
					sprintf(expires_time+strlen(expires_time),"00:");	// no minutes
				}

				sprintf(expires_time+strlen(expires_time),"%02ld:",expires);		// seconds

				expires_time[strlen(expires_time)-1]='\0';
			}

			for (i = 0; i < strlen(lease.hostname); i++) {
				host[i] = toupper(lease.hostname[i]);
			}							 
			host[i] = 0;
			strcpy(argv[1], "eth1");
			strcpy(argv[7], mac);
			strcpy(argv[8], ipaddr);
			memset(ipa, '\0', 20);
			strcpy(ipa, ipaddr);
			printf("mac = %s; ipaddr = %s; host = %s\n", mac, ipaddr, host);
			for (count = 0; count < 2; count++) {
				result = scanport_SendArpPkt(argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7], argv[8]);
				if ( result == 1)
					break;
			}
			if ( result == 1 && atchedDvisCnt < MAX_ATTACHED_DEVICES) {
			     found = 1;
			     if ( result ) {
				result = 0;
				memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
				memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
				memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
				memcpy(_attached_devices[atchedDvisCnt].attached_host, host, strlen(host));
				memcpy(_attached_devices[atchedDvisCnt].attached_mac, mac, strlen(mac));
				memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
				printf("_attached_devices[%d].attached_host = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_host);        
				printf("_attached_devices[%d].attached_mac  = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_mac);	
 				printf("_attached_devices[%d].attached_ip   = %s\n", atchedDvisCnt, _attached_devices[atchedDvisCnt].attached_ip);
				atchedDvisCnt++;
				if (atchedDvisCnt >= MAX_ATTACHED_DEVICES)
					break;		/* break while */
			    }
			}		
	    	}	/* while */
	}
	if (fp)
		fclose(fp);
	
	return found;		
}
#endif
#if 0
#define MAX_DHCPC	2
static int
ej_dhcpc_res_lease_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0, field = 0;
	char *ptr;
	char tmp[100], nvram_name[20];
	int index = 0;
	char *ipaddr, *mac, *desc;
	int i;

#if 0	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
#endif	
	
	for (i = 0; i < MAX_DHCPC; i++) {
		sprintf(nvram_name, "dhcpc_res%d", i);
		ptr = nvram_safe_get(nvram_name);
		if ((*ptr) == 0)
			break;
			
		strcpy(tmp, ptr);	
	
		/* Check for ipaddr address specification */
		mac = tmp;
		ipaddr = strsep(&mac, ",");
		if (!ipaddr)
			continue;
		
		desc = mac;
		mac = strsep(&desc, ",");
		if (!mac)
			continue;

		ret += websWrite(wp, "<tr><td width=\"10%%\" align=\"center\"><input type=\"radio\" name=\"R1\" value=\"%d\"   onclick=\"check2(this.value)\"></td><td width=\"10 %%\" align=\"center\"><span class=\"thead\">%d</span></td>", index, index+1);
		ret += websWrite(wp, "<td width=\"20%%\" align=\"center\"><span class=\"ttext\">%s <input type=hidden name=ip value=%s></span></td>",ipaddr,ipaddr);
		ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">%s&nbsp <input type=hidden name=hostname value=%s></span></td>", desc,desc);
		ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">");
		ret += websWrite(wp, "<input type=hidden name=mac value=%s>", mac);
		ret += websWrite(wp, "</span>%s</td>", mac);
		ret += websWrite(wp, "</tr>");
		
		index++;
		
	}
	
	/* lease dhcpc */
	
	return ret;
}
#endif

//ATTACHED_DEVICES
static int
ej_dhcpc_res_lease_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	FILE *fp;
        char line[100];
	char ipa[50];           // ip address
	char hwa[50];           // HW address 
	char mask[50];          // ntemask   
	char dev[50];           // interface
	int type;               // HW type
	int flags;              // flags
	int index = 1;
	int i = 0, ret = 0, retValue = 0, dhcpClient = 0, non_atchedDvis = 0, j =0,len = 0;
	unsigned char name[34];

	retValue = refresh_dhcpc_list();
	//printf("%s = atchedDvisCnt = %d\n", __FUNCTION__, atchedDvisCnt);
	if ((fp = fopen("/proc/net/arp", "r")) == NULL) 
			//return NULL;
			goto show_dhcpc_res;
        	
        
	// Read the ARP cache entries.
	// IP address       HW type     Flags       HW address            Mask     Device
	// 192.168.2.1      0x1         0x2         xx:xx:xx:xx:xx:xx     *          br0
	while( fgets(line, sizeof(line), fp)!= NULL)
	{
		strcpy(hwa, " ");		
		strcpy(ipa, " ");
		strcpy(name, " ");
		dhcpClient = 0;
		non_atchedDvis = 0;
		if(sscanf(line, "%s 0x%x 0x%x %s %s %s\n", ipa, &type, &flags, hwa, mask, dev)!=6)
			continue;

		//printf("ipa = %s\n", ipa);
		//printf("hwa = %s\n", hwa);
		//printf("dev = %s\n", dev);
		//if (strncmp(dev, "eth0", strlen("eth0")) != NULL) {
		if ( retValue && (strncmp(dev , "eth1", 4) == 0)) {	
			for ( i = 0; i < atchedDvisCnt; i++) {	
					if (strlen(ipa) > strlen(_attached_devices[i].attached_ip)) {
						len = strlen(ipa);
						//printf("strlen(ipa) = %d\n", len);
					}
					else {
						len = strlen(_attached_devices[i].attached_ip);
						//printf("strlen(_dhcpc_rsvd_addr[i].rsvd_ip = %d\n", len);
					}
					if (!memcmp(_attached_devices[i].attached_ip, ipa, len/*strlen(_attached_devices[i].attached_ip)*/)  ||  !memcmp(_attached_devices[i].attached_mac, hwa, 17/*strlen(_attached_devices[i].attached_mac)*/))
						dhcpClient = 1;

			}
		}

		if (strncmp(dev, "eth1", 4) == 0) {
			for ( i = 0; i < non_atchedDvisCnt; i++) {
				if (strlen(ipa) > strlen(_non_attached_devices[i].attached_ip)) {
					len = strlen(ipa);
					//printf("strlen(ipa) = %d\n", len);
				}
				else {
					len = strlen(_non_attached_devices[i].attached_ip);
					//printf("strlen(_attached_devices[i].attached_ip = %d\n", len);
				}
				if (!memcmp(_non_attached_devices[i].attached_ip, ipa, len/*strlen(_attached_devices[i].attached_ip)*/)  ||  !memcmp(_non_attached_devices[i].attached_mac, hwa, 17/*strlen(_attached_devices[i].attached_mac)*/))
					non_atchedDvis = 1;
			}
		}

		//printf("ipa = %s\n", ipa);
		//printf("hwa = %s\n", hwa);
		if ( (dhcpClient == 0) && (non_atchedDvis == 0) && (atchedDvisCnt < MAX_ATTACHED_DEVICES) && (strncmp(dev , "eth1", 4) == 0)) {		
			if (attached_devices(ipa, hwa, name) != 0 ) {
				printf("ipa = %s\n", ipa);
				printf("hwa = %s\n", hwa);
				printf("name = %s\n", name);
				
				memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
				memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
				memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
				memcpy(_attached_devices[atchedDvisCnt].attached_host, name, strlen(name));
				memcpy(_attached_devices[atchedDvisCnt].attached_mac, hwa, strlen(hwa));
				memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
				atchedDvisCnt++;
				if (atchedDvisCnt >= MAX_ATTACHED_DEVICES)
					break;	/* break while */
				//index++;
			}
		}
	} /* while */
#if 0
	if(dhcpcRsvdAddrCnt == 0) {
		for (i = 0; i < atchedDvisCnt; i++) {
		printf("_attached_devices[%d].attached_host = %s\n", i, _attached_devices[i].attached_host);
		printf("_attached_devices[%d].attached_mac  = %s\n", i, _attached_devices[i].attached_mac);
		printf("_attached_devices[%d].attached_ip   = %s\n", i, _attached_devices[i].attached_ip);
		ret += websWrite(wp, "<tr><td width=\"10%%\" align=\"center\"><input type=\"radio\" name=\"R1\" value=\"%d\" onclick=\"check2(this.value)\"></td><td width=\"10 %%\" align=\"center\"><span class=\"thead\">%d</span></td>", index, "checked", index);
		ret += websWrite(wp, "<td width=\"20%%\" align=\"center\"><span class=\"ttext\">%s <input type=hidden name=ip value=%s></span></td>",_attached_devices[i].attached_ip,_attached_devices[i].attached_ip);
		ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">%s&nbsp; <input type=hidden name=hostname value=%s></span></td>", _attached_devices[i].attached_host,_attached_devices[i].attached_host);	
		ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">");
		ret += websWrite(wp, "<input type=hidden name=mac value=%s>", _attached_devices[i].attached_mac);
		ret += websWrite(wp, "</span>%s</td>", _attached_devices[i].attached_mac);
		ret += websWrite(wp, "</tr>");
		index++;
		/*ret += websWrite(wp, "<tr><td nowrap width=\"5%%\" align=\"center\"><input type=\"radio\" value=\"%d\" %s name=\"X1\" onClick=\"check(this.value)\"></td><td nowrap width=\"3%%\" align=\"center\">%d</td><td nowrap width=\"24%%\" align=\"center\">%s</td>",index, ((index==0)? "checked" : ""), index+1, ipaddr);
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">%s</td>", desc);
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">");
		ret += websWrite(wp, "%s</td></tr>", mac);*/
		}
	}
	else 
	{
#endif
		for ( j = 0; j < atchedDvisCnt; j++) {
			//printf("_attached_devices[%d].attached_host = %s\n", j, _attached_devices[j].attached_host);
			//printf("_attached_devices[%d].attached_mac  = %s\n", j, _attached_devices[j].attached_mac);
			//printf("_attached_devices[%d].attached_ip   = %s\n", j, _attached_devices[j].attached_ip);
			_attached_devices[j].addrResv = 0;
			
			for ( i = 0; i < dhcpcRsvdAddrCnt; i++) {
				if (strlen(_attached_devices[j].attached_ip) > strlen(_dhcpc_rsvd_addr[i].rsvd_ip)) {
					len = strlen(_attached_devices[j].attached_ip);
					//printf("strlen(_attached_devices[j].attached_ip) = %d\n", len);
				}
				else {
					len = strlen(_dhcpc_rsvd_addr[i].rsvd_ip);
					//printf("strlen(_dhcpc_rsvd_addr[i].rsvd_ip = %d\n", len);
				}
				if (memcmp(_attached_devices[j].attached_ip, _dhcpc_rsvd_addr[i].rsvd_ip, len/*strlen(_dhcpc_rsvd_addr[i].rsvd_ip)*/) == NULL && \
					memcmp(_attached_devices[j].attached_mac, _dhcpc_rsvd_addr[i].rsvd_mac, 17/*strlen(_dhcpc_rsvd_addr[i].rsvd_mac)*/) == NULL) {
					
					_attached_devices[j].addrResv = 1;
				}
			}
		}
//	}
show_dhcpc_res:
	for ( i = 0; i < atchedDvisCnt; i++) {
		//printf("_attached_devices[i].addrResv = %d\n", _attached_devices[i].addrResv);
		if(_attached_devices[i].addrResv == 0) {

			for ( j = 0; j < strlen(_attached_devices[i].attached_host); j++)
				_attached_devices[i].attached_host[j] = toupper(_attached_devices[i].attached_host[j]);

			if(_attached_devices[i].attached_host[0] == NULL)
				strcpy(_attached_devices[i].attached_host, "&nbsp;");

			ret += websWrite(wp, "<tr><td width=\"10%%\" align=\"center\"><input type=\"radio\" name=\"R1\" value=\"0\"   onclick=\"check2(\'%d\')\"></td><td width=\"10 %%\" align=\"center\"><span class=\"thead\">%d</span></td>", index-1, index);
			ret += websWrite(wp, "<td width=\"20%%\" align=\"center\"><span class=\"ttext\">%s <input type=hidden name=ip value=%s></span></td>",_attached_devices[i].attached_ip,_attached_devices[i].attached_ip);
			ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">%s&nbsp; <input type=hidden name=hostname value=%s></span></td>", _attached_devices[i].attached_host,_attached_devices[i].attached_host);	
			ret += websWrite(wp, "<td width=\"30%%\" align=\"center\"><span class=\"ttext\">");
			ret += websWrite(wp, "<input type=hidden name=mac value=%s>", _attached_devices[i].attached_mac);
			ret += websWrite(wp, "</span>%s</td>", _attached_devices[i].attached_mac);
			ret += websWrite(wp, "</tr>");
			index++;
		}
	}
	if (fp)
		fclose(fp);
	/* lease dhcpc */

	return ret;
}

static int ej_statistics_get(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c, *ptr;	
	int ret = 0, wlan_enable = 0;
	unsigned long counters[6];
	unsigned long wl_counters[5];
	static unsigned long pre_lan_counters[6] = {0};
	static unsigned long pre_lan_sysup = 0;
	static unsigned long pre_wan_counters[6] = {0};
	static unsigned long pre_wan_sysup = 0;
	static unsigned long pre_wlan_counters[5] = {0};
	static unsigned long pre_wlan_sysup = 0;	
	int up, speed, duplex, range;	
	unsigned int sysup, rate;
	unsigned long wluptime;
	int hours, mins, secs, tx_packets, rx_packets;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	sysup = GetSysUpTime();

	if (strstr(name, "wlan")) {

		/* need to get wireless statistic ??? */
		//get_wlan_status(&wl_counters);
		/* 0 -> rx bytes
	   	   1 -> rx packets
	   	   2 -> tx bytes
	   	   3 -> tx packets
	   	   4 -> wireless up time in seconds
		*/   
		ptr = nvram_get("enable_ap");
		wlan_enable = atoi(ptr);
		if(wlan_enable) {
			get_wlan_status(&wl_counters);

			//wluptime = (wl_counters[4] *60*60) +( wl_counters[5] *60) + wl_counters[6];
			wluptime =wl_counters[4];
			
			//printf("Present  %d %d %d %d %d\n", wl_counters[0], wl_counters[1], wl_counters[2], wl_counters[3], wl_counters[4]);
			//printf("Previous %d %d %d %d %d\n", pre_wlan_counters[0], pre_wlan_counters[1], pre_wlan_counters[2], pre_wlan_counters[3], pre_wlan_counters[4]);

			if (strcmp(name, "wlan_rate") == 0) {
				ret +=websWrite(wp, "%s", "Best");	
			}	
			else if (strcmp(name, "wlan_txpkts") == 0) {
				ret +=websWrite(wp, "%ld", wl_counters[3]);
			}
			else if (strcmp(name, "wlan_rxpkts") == 0) {
				ret +=websWrite(wp, "%ld", wl_counters[1]);
			}	
			else if (strcmp(name, "wlan_txrate") == 0) {
				if (((wluptime - pre_wlan_sysup) > 0) && ((wl_counters[2] - pre_wlan_counters[2]) > 0)) {
					rate = (wl_counters[2] - pre_wlan_counters[2])/(wluptime - pre_wlan_sysup);
				}	
				else
					rate = 0;	
				ret += websWrite(wp, "%ld", rate);	
			}	
			else if (strcmp(name, "wlan_rxrate") == 0) {
				if (((wluptime - pre_wlan_sysup) > 0) && ((wl_counters[0] - pre_wlan_counters[0]) > 0)) {
					rate = (wl_counters[0] - pre_wlan_counters[0])/(wluptime - pre_wlan_sysup);
				}	
				else
					rate = 0;	
				ret += websWrite(wp, "%ld", rate);			
				memcpy((char *)&pre_wlan_counters[0], (char *)&wl_counters[0], 5*4);
				pre_wlan_sysup = wluptime;	
			}
		} else {
			if (strcmp(name, "wlan_rate") == 0) {
				ret +=websWrite(wp, "%s", "Link Down");	
			}	
			else if (strcmp(name, "wlan_txpkts") == 0) {
				ret +=websWrite(wp, "%ld", 0);
			}
			else if (strcmp(name, "wlan_rxpkts") == 0) {
				ret +=websWrite(wp, "%ld", 0);
			}	
			else if (strcmp(name, "wlan_txrate") == 0) {
				ret += websWrite(wp, "%ld", 0);	
			}	
			else if (strcmp(name, "wlan_rxrate") == 0) {
				ret += websWrite(wp, "%ld", 0);			
			}
		}
	}
	else if (strstr(name, "lan")) {
		int i, lan_speed = 0;		

#if 0		
		for(i=1;i<4;i++) {
			rtl8651_checkEthernetPortStatus(i,&up, &speed, &duplex, &range);
			if (up) {
				lan_speed = speed;
				if (lan_speed == 1)
					break;
			}				
		}
#endif		
		
		rtl8651_getIntfCounters(0x2, &counters);
		/* 0 -> rx bytes
	   	   1 -> rx packets
	   	   2 -> tx bytes
	   	   3 -> tx packets
	   	   5 -> drop packets
		*/   
		if (strcmp(name, "lan_rate") == 0) {
			//ret +=websWrite(wp, "%s", (lan_speed ? "100M" : "10M"));
			ret +=websWrite(wp, "%s", "Link Up");	
		}	
		else if (strcmp(name, "lan_txpkts") == 0) {
			ret +=websWrite(wp, "%ld", counters[3]);
		}
		else if (strcmp(name, "lan_rxpkts") == 0) {
			ret +=websWrite(wp, "%ld", counters[1]);
		}	
		else if (strcmp(name, "lan_txrate") == 0) {
			//ret +=websWrite(wp, "%ld", counters[2]);
			if (((sysup - pre_lan_sysup) > 0) && ((counters[2] - pre_lan_counters[2]) > 0)) {
				rate = (counters[2] - pre_lan_counters[2])/(sysup - pre_lan_sysup);
			}	
			else
				rate = 0;	
			ret += websWrite(wp, "%ld", rate);	
		}
		else if (strcmp(name, "lan_rxrate") == 0) {
			//ret +=websWrite(wp, "%ld", counters[0]);
			if (((sysup - pre_lan_sysup) > 0) && ((counters[0] - pre_lan_counters[0]) > 0)) {
				rate = (counters[0] - pre_lan_counters[0])/(sysup - pre_lan_sysup);
			}	
			else
				rate = 0;	
			ret += websWrite(wp, "%ld", rate);			
			
			memcpy((char *)&pre_lan_counters[0], (char *)&counters[0], 6*4);
			pre_lan_sysup = sysup;			
			
		}	
		//memcpy((char *)&pre_lan_counters[0], (char *)&counters[0], 6*4);
		//pre_lan_sysup = sysup;
	}
	else if (strstr(name, "wan")) {
		
		rtl8651_checkEthernetPortStatus(0/*WAN_PORT*/,&up, &speed, &duplex, &range);
		rtl8651_getIntfCounters(0x1, &counters);
		
		if (strcmp(name, "wan_rate") == 0) {
			if (up) {
				//ret +=websWrite(wp, "%s", (speed ? "100M" : "10M"));
				ret +=websWrite(wp, "%s", "Link Up");				
			}
			else {
				ret += websWrite(wp, "Link Down");
			}		
		}	
		else if (strcmp(name, "wan_txpkts") == 0) {
			ret +=websWrite(wp, "%ld", counters[3]);
		}
		else if (strcmp(name, "wan_rxpkts") == 0) {
			ret +=websWrite(wp, "%ld", counters[1]);
		}	
		else if (strcmp(name, "wan_txrate") == 0) {
			//ret +=websWrite(wp, "%ld", counters[2]);
			if (((sysup - pre_wan_sysup) > 0) && ((counters[2] - pre_wan_counters[2]) > 0)) {
				rate = (counters[2] - pre_wan_counters[2])/(sysup - pre_wan_sysup);
			}	
			else
				rate = 0;	
			ret += websWrite(wp, "%ld", rate);			
		}
		else if (strcmp(name, "wan_rxrate") == 0) {
			//ret +=websWrite(wp, "%ld", counters[0]);
			if (((sysup - pre_wan_sysup) > 0) && ((counters[0] - pre_wan_counters[0]) > 0)) {
				rate = (counters[0] - pre_wan_counters[0])/(sysup - pre_wan_sysup);
			}	
			else
				rate = 0;	
			ret += websWrite(wp, "%ld", rate);
			memcpy((char *)&pre_wan_counters[0], (char *)&counters[0], 6*4);
			pre_wan_sysup = sysup;
							
		}
		//memcpy((char *)&pre_wan_counters[0], (char *)&counters[0], 6*4);
		//pre_wan_sysup = sysup;
	}	
	else if (strcmp(name, "sys_up_time") == 0) {
		//ret +=websWrite(wp, "%d", 20);
		
		//ej_localtime(int eid, webs_t wp, int argc, char **argv)		
		//ret = ej_localtime(0, wp, 0, NULL);		
		//counters[0] = GetSysUpTime();
		//ret += websWrite(wp, "%ld", counters[0]);
		ej_sys_up_time(0, wp, 0, NULL); 
		
	}				

	return ret;	
}

extern unsigned long ppp_ipup_time;

static int ej_pppoe_status(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c;	
	int ret = 0;
	int status;
	unsigned long sec;
	int hour, minute;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	status = pppoe_dial_status(0);	

	if (strcmp(name, "pppoe_connect_time") == 0) {
		//ret += websWrite(wp, "00:00:00");	
		if (status == 3) {	
			sec = GetSysUpTime() - pRomeCfgParam->pppoeCfgParam[0].uptime;			
			hour = sec / (60 * 60);
			sec = sec % (60 * 60);
			minute = sec / 60;
			sec = sec % 60;
			ret += websWrite(wp, "%02d:%02d:%02d", hour, minute, sec);
		}	
		else {
			ret += websWrite(wp, "---");
		}
	}
	else if (strcmp(name, "pppoe_discover") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, Idle,  */
		if (status == 1) {	
			ret += websWrite(wp, "Waiting");
		}
		else if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "Idle");						
		}	
	}
	else if (strcmp(name, "pppoe_lcp") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
	}
	else if (strcmp(name, "pppoe_auth") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
	}
	else if (strcmp(name, "pppoe_ipcp") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
	}
	return ret;				
}

static int ej_pptp_status(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c;	
	int ret = 0;
	int status;
	unsigned long sec;
	int hour, minute;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	status = pptp_dial_status(0);	
	if (strcmp(name, "pptp_connect_time") == 0) {
		//ret += websWrite(wp, "00:00:00");
		if (status == 3) {	
			sec = GetSysUpTime() - pRomeCfgParam->pptpCfgParam.uptime;			
			hour = sec / (60 * 60);
			sec = sec % (60 * 60);
			minute = sec / 60;
			sec = sec % 60;
			ret += websWrite(wp, "%02d:%02d:%02d", hour, minute, sec);
		}	
		else {
			ret += websWrite(wp, "---");
		}		
	}
	else if (strcmp(name, "pptp_discover") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, Idle */
		if (status == 1) {	
			ret += websWrite(wp, "Waiting");
		}
		else if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "Idle");						
		}	
		
	}
	else if (strcmp(name, "pptp_lcp") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
		
	}
	else if (strcmp(name, "pptp_auth") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
		
	}
	else if (strcmp(name, "pptp_ipcp") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
		
	}
	return ret;
}

static int ej_l2tp_status(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c;	
	int ret = 0;
	int status;
	unsigned long sec;
	int hour, minute;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	status = l2tp_dial_status(0);	
	if (strcmp(name, "l2tp_connect_time") == 0) {
		//ret += websWrite(wp, "00:00:00");
		if (status == 3) {	
			sec = GetSysUpTime() - pRomeCfgParam->l2tpCfgParam.uptime;			
			hour = sec / (60 * 60);
			sec = sec % (60 * 60);
			minute = sec / 60;
			sec = sec % 60;
			ret += websWrite(wp, "%02d:%02d:%02d", hour, minute, sec);
		}	
		else {
			ret += websWrite(wp, "---");
		}		
	}
	else if (strcmp(name, "l2tp_discover") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, Idle */
		if (status == L2TPCFGPARAM_DIALSTATE_DIALED_DHCP_DISCOVER) {
			ret += websWrite(wp, "DHCP Discovery");			
		}	
		else if (status == 1) {	
			ret += websWrite(wp, "Waiting");
		}
		else if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "Idle");						
		}	
		
	}
	else if (strcmp(name, "l2tp_lcp") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
		
	}
	else if (strcmp(name, "l2tp_auth") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
		
	}
	else if (strcmp(name, "l2tp_ipcp") == 0) {
		//ret += websWrite(wp, "OK");	/* OK, Trying, --- */
		if (status == 2) {	
			ret += websWrite(wp, "Trying");
		}
		else if (status == 3) {	
			ret += websWrite(wp, "OK");
		}
		else {
			ret += websWrite(wp, "---");						
		}		
		
	}
	return ret;
}

#define PER_HOUR_SECS	3600
#define PER_DAY_HOURS	24

static int ej_dhcpc_status(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c;	
	int ret = 0;
	unsigned long remain;
	int days, hours, minutes;	

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}

	if ((strcmp(nvram_safe_get("wan_proto"), "dhcp") != 0) && 
	    (strcmp(nvram_safe_get("wan_proto"), "telstra") != 0) ) {
		ret += websWrite(wp, "None");
		return ret;
	}	
	if (strcmp(name, "wan0_dhcps") == 0) {
		ret += websWrite(wp, nvram_safe_get("wan0_gateway"));			
	}
	else if (strcmp(name, "dhcpc_obtain") == 0) {

		//ret += ej_localtime(0, wp, 0, NULL);	
		//ret += websWrite(wp, "0 days,1 hrs,0 minutes");
		remain = pRomeCfgParam->dhcpcCfgParam.lease;
		if (remain < 0)
			remain = 0;
		days = remain / (PER_HOUR_SECS * PER_DAY_HOURS);
		remain = remain % (PER_HOUR_SECS * PER_DAY_HOURS);
		hours = remain / PER_HOUR_SECS;
		remain = remain % PER_HOUR_SECS;
		minutes	= remain / 60;
		ret += websWrite(wp, "%d days, %d hrs, %d minutes", days, hours, minutes);
		
	}
	else if (strcmp(name, "dhcpc_lease") == 0) {
		//ret += websWrite(wp, "3600");
		//ret += websWrite(wp, "0 days,0 hrs,58 minutes");		
		//remain = dhcpc_lease_obtain - GetSysUpTime();
		if (pRomeCfgParam->dhcpcCfgParam.lease_expired <= GetSysUpTime())
			remain = 0;
		else	
			remain = pRomeCfgParam->dhcpcCfgParam.lease_expired - GetSysUpTime();		
		if (remain < 0)
			remain = 0;
		days = remain / (PER_HOUR_SECS * PER_DAY_HOURS);
		remain = remain % (PER_HOUR_SECS * PER_DAY_HOURS);
		hours = remain / PER_HOUR_SECS;
		remain = remain % PER_HOUR_SECS;
		minutes	= remain / 60;
		ret += websWrite(wp, "%d days, %d hrs, %d minutes", days, hours, minutes);
	}
	
	return ret;		
}
	

#if 0
static int
ej_dumpleases_dhcpc(int eid, webs_t wp, int argc, char_t **argv) 
{
	FILE *fp;
	struct lease_t lease;
	int i;
	struct in_addr addr;
	unsigned long expires;
	char sigusr1[] = "-XX";
	int ret = 0;
	int count = 0;
	char *ipaddr, mac[20]="", expires_time[50]="";
	char host[64] = "";
	int pid;


#if 0
        /* Write out leases file */
        sprintf(sigusr1, "-%d", SIGUSR1);
        eval("killall", sigusr1, "udhcpd");
#endif    

	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR1);    
		sleep(1);		/* waiting writed completed */
	}	
	
	/* Parse leases file */
	if ((fp = fopen(DHCPD_LEASES_PATH "udhcpd.leases", "r"))) {
	    count = 1;	
	    while (fread(&lease, sizeof(lease), 1, fp)) {
		strcpy(mac,"");

		for (i = 0; i < 6; i++) {
			sprintf(mac+strlen(mac),"%02X", lease.chaddr[i]);
			if (i != 5) sprintf(mac+strlen(mac),":");
		}
		mac[17] = '\0';
		if(!strcmp(mac,"00:00:00:00:00:00"))
			continue;
		
		addr.s_addr = lease.yiaddr;

		ipaddr = inet_ntoa(addr);

		expires = ntohl(lease.expires);

             
		strcpy(expires_time,"");
		if (!expires){
			continue;
			strcpy(expires_time,"expired");
		}
		else {
			if (expires > 60*60*24) {
				sprintf(expires_time+strlen(expires_time),"%ld days, ",expires / (60*60*24));
				expires %= 60*60*24;
			}
			if (expires > 60*60) {
				sprintf(expires_time+strlen(expires_time),"%02ld:",expires / (60*60));	// hours
				expires %= 60*60;
			}
			else{
				sprintf(expires_time+strlen(expires_time),"00:");	// no hours
			}
			if (expires > 60) {
				sprintf(expires_time+strlen(expires_time),"%02ld:",expires / 60);	// minutes
				expires %= 60;
			}
			else{
				sprintf(expires_time+strlen(expires_time),"00:");	// no minutes
			}

			sprintf(expires_time+strlen(expires_time),"%02ld:",expires);		// seconds

			expires_time[strlen(expires_time)-1]='\0';
		}
#if 0		     
		ret += websWrite(wp, "%c\"%s\",\"%s\",\"%s\"", //Brian+ for change show order
				count ? ',' :' ',
				mac,  
				/*!*lease.hostname ? "&nbsp;" : lease.hostname, */
				"&nbsp;",
				ipaddr
				);
		count++;
#endif

		for (i = 0; i < strlen(lease.hostname); i++) {
			host[i] = toupper(lease.hostname[i]);
		}							 
		host[i] = 0;
	
		ret+=websWrite(wp, "<tr>");
		ret+=websWrite(wp, "<td><center>%d</center></td>", count);	
		ret+=websWrite(wp, "<td><center>%s</center></td>", ipaddr);	
		ret+=websWrite(wp, "<td><center>%s</center></td>", host/*lease.hostname*/);			
		ret+=websWrite(wp, "<td><center>%s</center></td>", mac);
		ret+=websWrite(wp, "</tr>");	
		
		count++;	
		
	    }	/* while */
	fclose(fp);
	}	
        
	return ret;
}
#endif
//ATTACHED_DEVICES
//static int
//ej_dumplandevices(int eid, webs_t wp, int argc, char_t **argv) 
static int
ej_dumpleases_dhcpc(int eid, webs_t wp, int argc, char_t **argv) 
{
	FILE *fp;
        char line[100];
	char ipa[50];           // ip address
	char hwa[50];           // HW address 
	char mask[50];          // ntemask   
	char dev[50];           // interface
	int type;               // HW type
	int flags;              // flags
	int i, j, ret = 0, retValue = 0, len =0, value=0;
	unsigned char name[34];
	int dhcpClient = 0, non_atchedDvis = 0, index = 1, wlan_acl_count = 0;
	char mac[20];
	
	retValue = refresh_dhcpc_list();
	//printf("%d = atchedDvisCnt = %d\n", __LINE__, atchedDvisCnt);
	
	if ((fp = fopen("/proc/net/arp", "r")) == NULL) {
		//return NULL;
		goto show_dhcpc;
	} else {
	      // Read the ARP cache entries.
	      // IP address       HW type     Flags       HW address            Mask     Device
	      // 192.168.2.1      0x1         0x2         xx:xx:xx:xx:xx:xx     *          br0		
		while( fgets(line, sizeof(line), fp)!=NULL) {
			strcpy(hwa, " ");		
			strcpy(ipa, " ");
			strcpy(name, " ");
			dhcpClient = 0;
			non_atchedDvis = 0;
			if(sscanf(line, "%s 0x%x 0x%x %s %s %s\n", ipa, &type, &flags, hwa, mask, dev) !=6)
				continue;
			
			/*printf("ipa = %s\n", ipa);
			printf("hwa = %s\n", hwa);
			printf("dev = %s\n", dev);*/
			if (retValue && (strncmp(dev, "eth1", 4) == 0)) {
				for ( i = 0; i < atchedDvisCnt; i++) {
					if (strlen(ipa) > strlen(_attached_devices[i].attached_ip)) {
						len = strlen(ipa);
					}
					else {
						len = strlen(_attached_devices[i].attached_ip);
					}
					if (!memcmp(_attached_devices[i].attached_ip, ipa, len/*strlen(_attached_devices[i].attached_ip)*/)  ||  !memcmp(_attached_devices[i].attached_mac, hwa, 17/*strlen(_attached_devices[i].attached_mac)*/))
						dhcpClient = 1;
				}
			  }
			
			 if (strncmp(dev, "eth1", 4) == 0) {
				for ( i = 0; i < non_atchedDvisCnt; i++) {
					if (strlen(ipa) > strlen(_non_attached_devices[i].attached_ip)) {
						len = strlen(ipa);
					}
					else {
						len = strlen(_non_attached_devices[i].attached_ip);
					}
					if (!memcmp(_non_attached_devices[i].attached_ip, ipa, len/*strlen(_attached_devices[i].attached_ip)*/)  ||  !memcmp(_non_attached_devices[i].attached_mac, hwa, 17/*strlen(_attached_devices[i].attached_mac)*/))
						non_atchedDvis = 1;
				}
			  }
			
			 	
			 if ((dhcpClient == 0) && (non_atchedDvis == 0) && (atchedDvisCnt < MAX_ATTACHED_DEVICES) &&  (strncmp(dev, "eth1", 4) == 0)) {		
				value = attached_devices(ipa, hwa, name);
				if ( value == 1) {
				
					//if(name[0] == '\0')
					//	strcpy(name, "&nbsp;");
					
					memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
					memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
					memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
					memcpy(_attached_devices[atchedDvisCnt].attached_host, name, strlen(name));
					memcpy(_attached_devices[atchedDvisCnt].attached_mac, hwa, strlen(hwa));
					memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
					atchedDvisCnt++;
					if (atchedDvisCnt >= MAX_ATTACHED_DEVICES)
						break;		/* break while */
				} else if ( value == 2) {
					/* if hostname is empty, then we copy hostname from DHCPC address reservation */
					refresh_dhcpc_res_list();
		
					memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
					memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
					memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
					memcpy(_attached_devices[atchedDvisCnt].attached_host, name, strlen(name));
					memcpy(_attached_devices[atchedDvisCnt].attached_mac, hwa, strlen(hwa));
					memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
					
					if ( dhcpcRsvdAddrCnt != 0) {
						for ( i = 0; i < dhcpcRsvdAddrCnt; i++) {						
							if (strlen(_attached_devices[atchedDvisCnt].attached_ip) > strlen(_dhcpc_rsvd_addr[i].rsvd_ip)) {
								len = strlen(_attached_devices[atchedDvisCnt].attached_ip);
								//printf("strlen(_attached_devices[atchedDvisCnt].attached_ip) = %d\n", len);
							}
							else {
								len = strlen(_dhcpc_rsvd_addr[i].rsvd_ip);
								//printf("strlen(_dhcpc_rsvd_addr[i].rsvd_ip = %d\n", len);
							}
							
							if (memcmp(_attached_devices[atchedDvisCnt].attached_ip, _dhcpc_rsvd_addr[i].rsvd_ip, len/*strlen(_dhcpc_rsvd_addr[i].rsvd_ip)*/) == NULL && \
								memcmp(_attached_devices[atchedDvisCnt].attached_mac, _dhcpc_rsvd_addr[i].rsvd_mac, 17/*strlen(_dhcpc_rsvd_addr[i].rsvd_mac)*/) == NULL) {

								memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
								//memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
								//memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
								memcpy(_attached_devices[atchedDvisCnt].attached_host, _dhcpc_rsvd_addr[i].rsvd_host, strlen(_dhcpc_rsvd_addr[i].rsvd_host));
								//memcpy(_attached_devices[atchedDvisCnt].attached_mac, hwa, strlen(hwa));
								//memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
							}
						}
					} else {
						wlan_acl_count = refresh_wlan_acl_list();
						if(wlan_acl_count != 0) {
							for  ( i = 0; i < wlan_acl_count; i++) {						
								sprintf(mac, "%02x:%02x:%02x:%02x:%02x:%02x", pRomeCfgParam->wlanCfgParam[0].acMacList[i][0], pRomeCfgParam->wlanCfgParam[0].acMacList[i][1], pRomeCfgParam->wlanCfgParam[0].acMacList[i][2], \
																		pRomeCfgParam->wlanCfgParam[0].acMacList[i][3], pRomeCfgParam->wlanCfgParam[0].acMacList[i][4], pRomeCfgParam->wlanCfgParam[0].acMacList[i][5]);
								if (memcmp(_attached_devices[atchedDvisCnt].attached_mac, mac, 17/*strlen(_dhcpc_rsvd_addr[i].rsvd_mac)*/) == NULL) {
									memset(_attached_devices[atchedDvisCnt].attached_host, '\0', 34);
									//memset(_attached_devices[atchedDvisCnt].attached_mac, '\0', 20);
									//memset(_attached_devices[atchedDvisCnt].attached_ip, '\0', 20);
									memcpy(_attached_devices[atchedDvisCnt].attached_host, pRomeCfgParam->wlanCfgParam[0].acComment[i], strlen(pRomeCfgParam->wlanCfgParam[0].acComment[i]));
									//memcpy(_attached_devices[atchedDvisCnt].attached_mac, hwa, strlen(hwa));
									//memcpy(_attached_devices[atchedDvisCnt].attached_ip, ipa, strlen(ipa));
								}
							}
						}
					}
					atchedDvisCnt++;
					if (atchedDvisCnt >= MAX_ATTACHED_DEVICES)
						break;		/* break while */
				}
				else
					continue;
			  }
			
		}
show_dhcpc:		
		printf("atchedDvisCnt = %d\n",atchedDvisCnt);
		for ( i = 0; i < atchedDvisCnt; i++) {
 			//printf("_attached_devices[%d].attached_host = %s\n", i, _attached_devices[i].attached_host);        
			//printf("_attached_devices[%d].attached_mac  = %s\n", i, _attached_devices[i].attached_mac);	
 			//printf("_attached_devices[%d].attached_ip   = %s\n", i, _attached_devices[i].attached_ip);      	
 			for ( j = 0; j < strlen(_attached_devices[i].attached_host); j++)
				_attached_devices[i].attached_host[j] = toupper(_attached_devices[i].attached_host[j]);

			//printf("_attached_devices[%d].attached_host = %s\n", i, _attached_devices[i].attached_host);
        		ret += websWrite(wp, "<tr><td width=\"10%%\" align=\"center\">%d</td><td nowrap width=\"20%%\" align=\"center\">%s</td><td nowrap width=\"20%%\" align=\"center\">%s</td><td nowrap width=\"50%%\" align=\"center\">%s</td>", index++, _attached_devices[i].attached_ip, ((_attached_devices[i].attached_host[0] == '\0') ? "&nbsp" : _attached_devices[i].attached_host), _attached_devices[i].attached_mac); 
			
		}
	}
	if (fp)
		fclose(fp);	
		
	return ret;

}

void
refresh_dhcpc_res_list(void)
{	
	char *name, *c;
	char res_count[3]={0};
	int ret = 0, field = 0, i = 0, count = 0;
	char *ptr;
	char tmp[100], nvram_name[20];
	int index = 0;
	char *ipaddr, *mac, *desc;
	dhcpcRsvdAddrCnt = 0;
	
	strcpy(res_count, nvram_get("dhcpc_res_count"));
	count = atoi(res_count);

	for (i = 0; i < count; i++) {
		sprintf(nvram_name, "dhcpc_res%d", i);
		ptr = nvram_safe_get(nvram_name);
		//printf("ptr = %s\n", ptr);
		if ((*ptr) == 0)
			continue;

		strcpy(tmp, ptr);
		mac = tmp;
		ipaddr = strsep(&mac, ",");
		//printf("1 - mac = %s\n", mac);
		//printf("ipaddr = %s\n", ipaddr);
		if (!ipaddr)
			continue;
		
		desc = mac;
		mac = strsep(&desc, ",");
		//printf("desc = %s\n", desc);
		//printf("mac = %s\n", mac);
		if (!mac)
			continue;

		//if ( !ipaddr || !mac )
			//continue;
		//else {
		if( !desc || (strlen(desc) == 0)) 
				strcpy(desc, "&nbsp;");
		
		memset(_dhcpc_rsvd_addr[i].rsvd_host, '\0', 34);
		memset(_dhcpc_rsvd_addr[i].rsvd_mac, '\0', 20);
		memset(_dhcpc_rsvd_addr[i].rsvd_ip, '\0', 20);
		memcpy(_dhcpc_rsvd_addr[i].rsvd_host, desc, strlen(desc));
		memcpy(_dhcpc_rsvd_addr[i].rsvd_mac, mac, strlen(mac));
		memcpy(_dhcpc_rsvd_addr[i].rsvd_ip, ipaddr, strlen(ipaddr));
		dhcpcRsvdAddrCnt++;	
	}
}

int
refresh_wlan_acl_list(void) 
{
	char *name, *c;	
	int ret = 0, i = 0, j = 0, k = 0, count = 0;
	char *ptr;
	char tmp[50]={0};
	char nvram_name[20]={0};
	int index = 0;
	char mac[50]={0};
	char *desc;
	char hostname[20]={0};
	char acl_count[3]={0};
	int wlan_acl_count = 0;
	int val = 0;
	unsigned char _mac[20]={0};
	//char macaddr[20]={0};

	strcpy(acl_count , nvram_get("wlan_acl_count"));
	wlan_acl_count = atoi(acl_count);
	//printf("%d %s wlan_acl_count = %d\n", __LINE__, __FUNCTION__, wlan_acl_count);

	//for ( w=0 ; w<totalWlanCards ; w++ )
	//{
	for (i = 0; i < wlan_acl_count; i++) {
		sprintf(nvram_name, "wlan_acl%d", i);
		ptr = nvram_get(nvram_name);
		//printf("4564 ptr = %s\n", ptr);
		if ((*ptr) == 0)
			continue;

		strcpy(tmp, ptr);
		desc = tmp;
		//printf("4569 desc = %s\n", desc);
		strcpy(ptr, "");
		ptr = strsep(&desc, ",");
		//printf("4572 ptr = %s\n", ptr);
		if (!ptr)
			continue;
		memset(hostname, 0, 20);
		memset(mac, 0, 50);
		strncpy(hostname, desc, strlen(desc));
		strncpy(mac, ptr, strlen(ptr));
		
		j = 0;
		memset(tmp, 0, 50);
		for ( k = 0; k < 12; k++) {
			sprintf(tmp, "%c%c", mac[k], mac[k+1]);
			val = strtol(tmp, NULL, 16);
			_mac[j++] = val;
			k++;
		}

		//printf("%d %s %02x %02x %02x %02x %02x %02x \n", __LINE__, __FUNCTION__, _mac[0], _mac[1],  _mac[2], _mac[3], _mac[4], _mac[5]);
		memset(pRomeCfgParam->wlanCfgParam[0].acComment[i], '\0', 20);
		memset(pRomeCfgParam->wlanCfgParam[0].acMacList[i], '\0', 6);
		memcpy(pRomeCfgParam->wlanCfgParam[0].acComment[i], hostname, strlen(hostname));
		memcpy(pRomeCfgParam->wlanCfgParam[0].acMacList[i], _mac, 6);
		//printf("%d %s %02x %02x %02x %02x %02x %02x \n", __LINE__, __FUNCTION__, pRomeCfgParam->wlanCfgParam[0].acMacList[i][0],  pRomeCfgParam->wlanCfgParam[0].acMacList[i][1] ,  pRomeCfgParam->wlanCfgParam[0].acMacList[i][2], \
			//pRomeCfgParam->wlanCfgParam[0].acMacList[i][3],  pRomeCfgParam->wlanCfgParam[0].acMacList[i][4],  pRomeCfgParam->wlanCfgParam[0].acMacList[i][5]);
	}
	
	return wlan_acl_count;

}

#if 0
nvram_get_current_dhcpc_res(int dhcpc_index)
{
	char *ptr;
	int i;
	
	i = dhcpc_res_cur_index;			/* current */
	
	DPRINTF("nvram_get(\"%s\")\n", name);
	
	if (strcmp(name, "desc") == 0) {	
		if (i < MAX_DHCPC)
			sprintf(str, pRomeCfgParam->dhcpsCfgParam.manu_desc[i]); 
		else
			strcpy(str, "");
	
		return str;
	}
	else if (strcmp(name, "ipaddr") == 0) {	
		if (i < MAX_DHCPC)
			strcpy(str, (char *)(inet_string((uint32)&(pRomeCfgParam->dhcpsCfgParam.manualIp[i])))); 
		else
			strcpy(str, "");	
		return str;
	}
	else if (strcmp(name, "macaddr") == 0) {	
		if (i < MAX_DHCPC)
			ether_etoa(pRomeCfgParam->dhcpsCfgParam.hardwareAddr[i], str);
		else
			strcpy(str, "");	
		return str;
	}
	else
		return ("");	
		
}
#endif

static int
ej_dhcpc_lease_count(int eid, webs_t wp, int argc, char_t **argv) 
{
	FILE *fp;
	struct lease_t lease;
	int i;
	struct in_addr addr;
	unsigned long expires;
	char sigusr1[] = "-XX";
	int ret = 0;
	int count = 0;
	char *ipaddr, mac[20]="", expires_time[50]="";
	int pid;

	return websWrite(wp, "0");

}

static int
ej_dumplandevices(int eid, webs_t wp, int argc, char_t **argv) 
{
	ej_dumpleases_dhcpc(eid, wp, argc, argv); 		
}	


#define LAN_IF_NAME	"eth1"
//CL
char * 
get_http_client_hwaddr(char *ip)
{
	FILE *fp;
        char line[100];

	char ipa[50];           // ip address
	char hwa[50];           // HW address 
	char mask[50];          // ntemask   
	char dev[50];           // interface
	int type;               // HW type
	int flags;              // flags
	static char mac[20]; 
	memset(mac,0,20);						

        if ((fp = fopen("/proc/net/arp", "r")) == NULL) 
			return NULL;
        	
        
	      // Read the ARP cache entries.
	      // IP address       HW type     Flags       HW address            Mask     Device
	      // 192.168.2.1      0x1         0x2         xx:xx:xx:xx:xx:xx     *          br0
		while( fgets(line, sizeof(line), fp)!=NULL)
		{
			if(sscanf(line, "%s 0x%x 0x%x %s %s %s\n", ipa, &type, &flags, hwa, mask, dev)!=6)
				continue;
			
			if(strcmp(ip, ipa))
				continue;
			
				strcpy(mac, hwa);
				break;					
		}
		/* for check MAC not found, default WAN mac address */
		if (strcmp(LAN_IF_NAME, dev))
		    strcpy(mac,nvram_get("wan0_hwaddr"));

	fclose(fp);
	return (mac);
}

static int
ej_getclonemac(int eid, webs_t wp, int argc, char_t **argv)
{
	//char zero_mac[20];
	char str_clientip[20];
	char *str_mac;
	struct sockaddr_in peer;
	int    peer_len=sizeof(peer);
	int sockfd;
	int ret = 0;
	
	sockfd = wp->fd;
	if (getpeername(sockfd,(struct sockaddr*)&peer,&peer_len ) < 0)
	{
		return ret;
		
	}
	strcpy(str_clientip, inet_ntoa(peer.sin_addr));
	str_mac = get_http_client_hwaddr(str_clientip);
	ret = websWrite( wp, str_mac );	
	

	return ret;

}
//end of dvd.chen

//#define CONFIG_RTL8185
#ifdef CONFIG_RTL8185
#include "../rtl865x/rtl_wireless.h"
#define SIOCGIWRTLSTAINFO		0x8B30
#if 0
	#define SIOCGIWRTLSTANUM		0x8B31
	#define SIOCGIWRTLDRVVERSION    0x8B32
	#define SIOCGIWRTLSCANREQ		0x8B33
	#define SIOCGIWRTLGETBSSDB		0x8B34
	#define SIOCGIWRTLJOINREQ		0x8B35
	#define SIOCGIWRTLJOINREQSTATUS	0x8B36
	#define SIOCGIWRTLGETBSSINFO	0x8B37
	#define SIOCGIWRTLGETWDSINFO	0x8B38
	#define STA_INFO_FLAG_80211A        	0x10
	#define STA_INFO_FLAG_80211B        	0x20   //if this bit is not set, it is a 11G station
#endif
typedef struct _sta_info_2_web {
	unsigned short		aid;
	unsigned char		addr[6];
	unsigned long		tx_packets;
	unsigned long		rx_packets;
	unsigned long		expired_time;	// 10 msec unit
	unsigned short		flags;
	unsigned char		TxOperaRate;
	unsigned char		rssi;
	unsigned long		link_time;		// 1 sec unit
	unsigned long		tx_fail;
	unsigned char 		resv[16];
} sta_info_2_web;

unsigned char wlan_hostname[20]={0};

static int
ej_wlan_Client_List(int eid, webs_t wp, int argc, char_t **argv)
//void asp_setWlanAccessCtrl(request * wp, char *path, char *query)
{
	int w;
	char cmdBuf[40];
	char iface[16];
#define GETSIZE   64
#ifdef CONFIG_RTL8185
	sta_info_2_web  info[GETSIZE];
#endif/*CONFIG_RTL8185*/

	for ( w=0 ; w<totalWlanCards ; w++ )
	{
		int total, i, j;
		sprintf(iface, "wlan%d",w); 
		sprintf(cmdBuf, "list%d",w); 	

		//if ( strcmp(argv[0],cmdBuf) == 0 )
		{
#ifdef CONFIG_RTL8185
			int i,count=0, retval=-1;
			int size;
			int oldsize;
			char temp[6]={0};
next_round:
			size = sizeof(info);
			oldsize = size;
			memset(&info[0], 0, sizeof(info));
			retval = rtl8185_Get_StaInfo(iface, &info[0], &size);
			if ( 0 != retval )
			{
				/* do nothing because get station fail */
				printf("Get Station Info not success: %d\n", retval);
			}
			else if ( size != oldsize )
			{
				if (size)
				{
					/* buffer not filled completely. reached the end of list*/
					for( i = 0 ; i < size/sizeof(sta_info_2_web) ; i++ )
					{
						int retval = 0;
						retval = get_wlan_hostname_from_dhcpc_list(&info[i]);

						if(!retval)
							retval = get_wlan_hostname_from_arp_list(&info[i]);

						/*if(!retval)
							strcpy(wlan_hostname, "UNKNOWN");*/

						if(retval)	
							print_STA(wp, count+i,&info[i]);
					}
					count += size/sizeof(sta_info_2_web);
				}else if( count == 0 )
				{
					//No clients associated. List is empty now
					//websWrite(wp,
			 		//"<tr bgColor=#ddeeff>"
			 		//"<td colspan=9 align=center>--No clients associated--</td></tr>\n");
			 		return 0;
				}
			}else
			{
				/*buffer filled. Get next round*/
				for( i = 0 ; i < size/sizeof(sta_info_2_web) ; i++ )
				{
					int retval = 0;
					retval = get_wlan_hostname_from_dhcpc_list(&info[i]);

					if(!retval)
						retval = get_wlan_hostname_from_arp_list(&info[i]);

					//if(!retval) {
						//strcpy(wlan_hostname, "UNKNOWN");
						//memset(info->addr, 0, 6);
					//}
					
					if(retval)	
							print_STA(wp, count+i,&info[i]);
				}
				count += size/sizeof(sta_info_2_web);
				if (count < GETSIZE)
				{
					goto next_round;
				}
			}			
			break;
#endif			
		}	
	}		
	return 0;
	
}	

int rtl8185_Get_StaInfo(char *name, unsigned char *dataptr, unsigned int *size)
{
	struct iwreq wrq;
	struct sta_info_2_web *info;
	int sockfd, retval=-1;
	if(!size){
		retval=-2;
		goto out;
	}else if(*size==0 || *size>(sizeof( sta_info_2_web)*64)){
		retval=-3;
		goto out;
	}
	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		retval=-4;
		goto out;
	}

	memset(&wrq, 0, sizeof(struct iwreq));
	strncpy(wrq.ifr_ifrn.ifrn_name, name, IFNAMSIZ);
	memset(dataptr, 0, *size);
	wrq.u.data.pointer = (caddr_t)dataptr;
	wrq.u.data.length = *size;
	retval=ioctl(sockfd, SIOCGIWRTLSTAINFO, &wrq);
	if(retval<0){
       		*size=0;
       }else{
       		*size=retval;
		retval=0;
	}
	close(sockfd);
out:
        return retval;
}

int get_wlan_hostname_from_dhcpc_list(sta_info_2_web *info) {

	FILE *fp;	
	struct in_addr addr;
	char sigusr1[] = "-XX";
	int found = 0;
	char line[100];
	char ipa[20];
	char mac[20];
	char host[20];
	int pid, result = 0;//, count = 0;

	/* add the following additional variables due to the change from scanport_SendArpPkt( ) to scanport_arpping( ) */
	int i = 0, j = 0;
	unsigned char lan_ipaddr[20];
	unsigned char lan_hwaddr[20];
	unsigned char _lan_hwaddr[20];
	unsigned char dev[5]="";
	unsigned char tmp[5], tmp1[6];//, *value;
	int value = 0;
	char str[20];
	//char macstr[17];
	
	/* Write out leases file */
	pid = get_task_pid("udhcpd");
	if (pid != 0) {
		kill(pid, SIGUSR2);    
		sleep(1);		/* waiting writed completed */
	}
	else {
		found = 0;
		return found;
	}			
	
	memset(dev, 0, 5);
	strncpy(dev, "eth1", 5);
	strcpy(lan_ipaddr, nvram_get("lan_ipaddr"));
	strcpy(lan_hwaddr, nvram_get("lan_hwaddr"));
	j = 0;
	for ( i = 0; i < strlen(lan_hwaddr); i++) {
		if ( (i  !=  2) && (i  !=  5) && (i  !=  8) && ( i  !=  11) && ( i  !=  14)) {
			_lan_hwaddr[j] = lan_hwaddr[i];
			DPRINTF("* _lan_hwaddr[%d] = %c\n", j,  _lan_hwaddr[j]);
			j++;
		}
	}

	j = 0;
	for ( i = 0; i < 12; i++) {
		value = 0;
		sprintf(tmp, "%c%c", _lan_hwaddr[i], _lan_hwaddr[i+1]);
		value = strtol(tmp, NULL, 16);
		tmp1[j++] = value;
		i++;
	}	

	ether_etoa(info->addr, str);
	
	if ((fp = fopen(DHCPD_LEASES_PATH "dhcps_client", "r")) == NULL)
		printf("unable to open file\n");
	else {
		while( fgets(line, sizeof(line), fp)!=NULL) {
			result = 0;
			found = 0;
			memset(mac, 0, 20);		
			memset(ipa, 0, 20);
			memset(host, 0, 20);
			
			if(sscanf(line, "%s %s %s\n", mac, ipa, host) != 3 )
				continue;
			
			//ether_atoe(mac, macstr);
			//printf("macstr = %02x %02x %02x %02x %02x %02x\n", macstr[0], macstr[1], macstr[2], macstr[3], macstr[4], macstr[5]);
			if ( memcmp(str, mac, 17) == NULL)  {
				result = !scanport_arpping(ipa, lan_ipaddr, tmp1, dev, mac);
			}
			else
				continue;
							
			if ( result ) {
				if(sscanf(line, "%s %s %s\n", mac, ipa, host) != 3 )
					continue;
				memset(wlan_hostname, 0, 20);
				if(host[0] == '\0')
					strcpy(wlan_hostname, "UNKNOWN");
				else
					memcpy(wlan_hostname, host, strlen(host));
				found = 1;
				break;
			}
	    	}	/* while */
	}
	if (fp)
		fclose(fp);
	
	return found;		
}

static int
get_wlan_hostname_from_arp_list(sta_info_2_web *info)
{	
	FILE *fp;
        char line[100];
	char ipa[50];           // ip address
	char hwa[50];           // HW address 
	char mask[50];          // ntemask   
	char dev[50];           // interface
	int type;               // HW type
	int flags;              // flags
	int index = 1;
	int i = 1, ret = 0, retValue = 0, j =0,len = 0;
	unsigned char name[20];
	char str[20];
	int found = 0;
	
	if ((fp = fopen("/proc/net/arp", "r")) == NULL) {
			strcpy(wlan_hostname, "");
			goto exit;
	}
        	
       ether_etoa(info->addr, str);
			
	// Read the ARP cache entries.
	// IP address       HW type     Flags       HW address            Mask     Device
	// 192.168.2.1      0x1         0x2         xx:xx:xx:xx:xx:xx     *          br0
	while( fgets(line, sizeof(line), fp)!= NULL)
	{
		memset(hwa, 0, 50);		
		memset(ipa, 0, 50);
		memset(name, 0, 20);
		
		if(sscanf(line, "%s 0x%x 0x%x %s %s %s\n", ipa, &type, &flags, hwa, mask, dev)!=6)
			continue;

		if ( memcmp(str, hwa, 17) == NULL)  {
			if (attached_devices(ipa, hwa, name) ) {
				/*printf("ipa = %s\n", ipa);
				printf("hwa = %s\n", hwa);
				printf("name = %s\n", name);*/
				memset(wlan_hostname, 0, 20);
				
				if(name[0] == '\0')
					strcpy(wlan_hostname, "UNKNOWN");
				else
					memcpy(wlan_hostname, name, strlen(name));
				
				found = 1;
				break;
			}
		} else
			continue;
	} /* while */
exit:
	if (fp)
		fclose(fp);

	return found;
}

void print_STA(webs_t  wp, int index, sta_info_2_web *info){

	int ret = 0, i;

	/*
	websWrite(wp,
 		"<tr><td>%d</td>
 		<td>%s</td>
 		<td>%02X:%02X:%02X:%02X:%02X:%02X</td></tr>",
			index,
			wlan_hostname,
			info->addr[0],info->addr[1],info->addr[2],info->addr[3],info->addr[4],info->addr[5]
	);
	*/
	for ( i = 0; i < strlen(wlan_hostname); i++)
		wlan_hostname[i] = toupper(wlan_hostname[i]);

	ret += websWrite(wp, "<tr><td width=\"10%%\" align=\"center\"><span class=\"ttext\"><input type=\"radio\" name=\"Macselect\" onclick=\"sel(\'%s\',\'%02X:%02X:%02X:%02X:%02X:%02X\')\"></td>", wlan_hostname, info->addr[0], info->addr[1], info->addr[2], info->addr[3], info->addr[4], info->addr[5]);
	ret += websWrite(wp, "<td nowrap align=\"center\"><span class=\"ttext\">%s</span></td>",wlan_hostname);
	ret += websWrite(wp, "<td nowrap align=\"center\"><span class=\"ttext\">%02X:%02X:%02X:%02X:%02X:%02X</span></td>", info->addr[0], info->addr[1], info->addr[2], info->addr[3], info->addr[4], info->addr[5]);	
	ret += websWrite(wp, "</tr>");

	return ret;
} 



static int
ej_wlan_acl_list(int eid, webs_t wp, int argc, char_t **argv)
{	
	char *name, *c;	
	int ret = 0, i = 0, j = 0, k = 0, count = 0;
	char *ptr;
	char tmp[50]={0};
	char nvram_name[20]={0};
	int index = 0;
	char mac[50]={0};
	char *desc;
	char hostname[20]={0};
	char acl_count[3]={0};
	int wlan_acl_count = 0;
	int val = 0;
	unsigned char _mac[20]={0};
	char macaddr[20]={0};

	strcpy(acl_count , nvram_get("wlan_acl_count"));
	wlan_acl_count = atoi(acl_count);

	//for ( w=0 ; w<totalWlanCards ; w++ )
	//{
	for (i = 0; i < wlan_acl_count; i++) {
		sprintf(nvram_name, "wlan_acl%d", i);
		ptr = nvram_get(nvram_name);
		//printf("4564 ptr = %s\n", ptr);
		if ((*ptr) == 0)
			continue;

		strcpy(tmp, ptr);
		desc = tmp;
		strcpy(ptr, "");
		ptr = strsep(&desc, ",");
		if (!ptr)
			continue;
		memset(hostname, 0, 20);
		memset(mac, 0, 50);
		strncpy(hostname, desc, strlen(desc));
		strncpy(mac, ptr, strlen(ptr));
		
		j = 0;
		memset(tmp, 0, 50);
		for ( k = 0; k < 12; k++) {
			sprintf(tmp, "%c%c", mac[k], mac[k+1]);
			val = strtol(tmp, NULL, 16);
			_mac[j++] = val;
			k++;
		}

		memset(pRomeCfgParam->wlanCfgParam[0].acComment[i], '\0', 20);
		memset(pRomeCfgParam->wlanCfgParam[0].acMacList[i], '\0', 6);
		memcpy(pRomeCfgParam->wlanCfgParam[0].acComment[i], hostname, strlen(hostname));
		memcpy(pRomeCfgParam->wlanCfgParam[0].acMacList[i], _mac, 6);
		//printf("%d %s %02x %02x %02x %02x %02x %02x \n", __LINE__, __FUNCTION__, pRomeCfgParam->wlanCfgParam[0].acMacList[i][0],  pRomeCfgParam->wlanCfgParam[0].acMacList[i][1] ,  pRomeCfgParam->wlanCfgParam[0].acMacList[i][2], \
			//pRomeCfgParam->wlanCfgParam[0].acMacList[i][3],  pRomeCfgParam->wlanCfgParam[0].acMacList[i][4],  pRomeCfgParam->wlanCfgParam[0].acMacList[i][5]);

		ether_etoa(_mac, macaddr);

		if( !desc || (strlen(desc) == 0)) 
			strcpy(desc, "&nbsp;");
		
		ret += websWrite(wp, "<tr><td nowrap width=\"5%%\" align=\"center\"><input type=\"radio\" name=\"X1\" onClick=\"check(\'%d\')\"></td>", index++);
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">%s</td>", hostname);
		ret += websWrite(wp, "<td nowrap width=\"34%%\" align=\"center\">");
		ret += websWrite(wp, "%s</td></tr>", macaddr);

	}
return ret;
}

extern int wlan_acl_cur_index;
static int ej_wlan_acl_info(int eid, webs_t wp, int argc, char_t **argv)  {
	
	char *name, *c, str[200];	
	int ret = 0;
	char tmp[20];
	int i;

	if (ejArgs(argc, argv, "%s", &name) < 1) {
		//websError(wp, 400, "Insufficient args\n");
		return 0;
	}
	
	//if ( dhcpcRsvdAddrCnt == 0)
		//return;
	if (strcmp(name, "device") == 0) {
		sprintf(str, pRomeCfgParam->wlanCfgParam[0].acComment[wlan_acl_cur_index]); 
		c = str;
		ret +=websWrite(wp, "%s", c);
	}	
	else if (strcmp(name, "adr") == 0) {
		sprintf(str, "%02x:%02x:%02x:%02x:%02x:%02x", pRomeCfgParam->wlanCfgParam[0].acMacList[wlan_acl_cur_index][0], \
				pRomeCfgParam->wlanCfgParam[0].acMacList[wlan_acl_cur_index][1], \
				pRomeCfgParam->wlanCfgParam[0].acMacList[wlan_acl_cur_index][2], \
				pRomeCfgParam->wlanCfgParam[0].acMacList[wlan_acl_cur_index][3], \ 
				pRomeCfgParam->wlanCfgParam[0].acMacList[wlan_acl_cur_index][4], \
				pRomeCfgParam->wlanCfgParam[0].acMacList[wlan_acl_cur_index][5]);	
		c = str;
		/*name = tmp;
		for (i = 0; i < strlen(c); i++) {
			if (c[i] != ':') 
				*name++= c[i];
		}
		*name = 0;		
		c = tmp;*/	
		ret +=websWrite(wp, "%s", c);
	}

	return ret;	
}

void
get_wlan_status(unsigned long *counters)
{	
	FILE *fp;
	int i = 0;
	char *up_time = "up_time";
	//char *day = "day";
	char *hr = "hr";
	char *min = "min";
	char *sec = "sec";
	char *tx_packet = "tx_packets";
	char *tx_byte = "tx_bytes";
	char *rx_packet = "rx_packets";
	char *rx_byte = "rx_bytes";
       char line[100];
	char name[20];
	char time[10][10];
	unsigned long value[10];
	int tx_packets, tx_bytes, rx_packets, rx_bytes;
	
	if ((fp = fopen("/proc/wlan0/stats", "r")) == NULL) {
			printf("unable to open /proc/wlan0/stats file\n");
			goto exit;
	}

	for ( i = 0; i < 5; i++ )
		counters[i] = 0;
	
	while( fgets(line, sizeof(line), fp) != NULL)
	{
		//printf("line = %s\n", line);
		if (strstr(line, up_time)) {
			
			for (i = 0; i < 12; i++) {
				value[i] = 0;
				//memset(time[i], 0, 10);
			}
			/* just in case if end user open router for over one month or one year */
			/* up_time: x year x mon x day x hr x day x min x sec */
			/* the update time is always caculated by adding hours , minutes and seconds only */
			/* as our maximum interval time is set as "9999", which is 9999 / 60 / 60 = 2.7775 hr */
			if(sscanf(line, "%s %d %s %d %s %d %s %d %s %d %s %d %s\n", name, &value[0], time[0], &value[1], time[1], &value[2], time[2], &value[3], time[3], &value[4], time[4], &value[5], time[5]) != 0 ){
				//printf("%s %d %s %d %s %d %s %d %s %d %s %d %s\n", name, value[0], time[0], value[1], time[1], value[2], time[2], value[3], time[3], value[4], time[4], value[5], time[5]);
				for( i = 0; i < 7; i++) {
					if (value[i] != 0) {
						if (!strcmp(sec, time[i])) 
							counters[4] = counters[4] + value[i];
						else if (!strcmp(min, time[i])) 
							counters[4] = counters[4] + (value[i] * 60);
						else if (!strcmp(hr, time[i])) 
							counters[4] = counters[4] + (value[i] * 60 * 60);
						//else if (!strcmp(day, time[i])) 
							//counters[4] = counters[4] + (value[i] * 24 * 60 * 60);
					} 
				}
			}
			//printf("sec = %d \n", counters[4]);

		} else if (strstr(line, tx_packet)){
			if(sscanf(line, "%s %d\n", name, &tx_packets) == 2) {
				//printf("name = %s\n", name);
				//printf("tx_packets = %d\n", tx_packets);
				counters[3] = tx_packets;
			} else
				continue;
		} else if (strstr(line, tx_byte)){
			if(sscanf(line, "%s %d\n", name, &tx_bytes) == 2) {
				//printf("name = %s\n", name);
				//printf("tx_bytes = %d\n", tx_bytes);
				counters[2] = tx_bytes;
			} else
				continue;
		} else if (strstr(line, rx_packet)){
			if(sscanf(line, "%s %d\n", name, &rx_packets) == 2) {
				//printf("name = %s\n", name);
				//printf("rx_packets = %d\n", rx_packets);
				counters[1] = rx_packets;
			} else
				continue;
		} else if (strstr(line, rx_byte)){
			if(sscanf(line, "%s %d\n", name, &rx_bytes) == 2) {
				//printf("name = %s\n", name);
				//printf("rx_packets = %d\n", rx_bytes);
				counters[0] = rx_bytes;
			} else
				continue;
		}
		else
			continue;
			
	}
	//printf("%d %d %d %d %d\n", counters[0], counters[1], counters[2], counters[3], counters[4]);
	
exit:
	if (fp)
		fclose(fp);

}

#endif //end of "#ifdef CONFIG_RTL8185"

static int
ej_getsockname(int eid, webs_t wp, int argc, char_t **argv)
{
	//char zero_mac[20];
	char str_sockname[20];
	int ret = 0;
	struct sockaddr_in salocal;
	int dummy;

	dummy = sizeof(salocal);
	if (getsockname(wp->fd, (struct sockaddr *) &salocal, &dummy) == -1){
		return ret;
	}
	strcpy(str_sockname, inet_ntoa(salocal.sin_addr));
	ret += websWrite( wp, "%s", str_sockname );			

	return ret;

}



struct ej_handler ej_handlers[] = {
        { "get_firmware_ver", ej_get_firmware_ver},
	{ "nvram_get", ej_nvram_get },
	{ "nvram_match", ej_nvram_match },
	{ "nvram_invmatch", ej_nvram_invmatch },
	{ "nvram_list", ej_nvram_list },		
	{ "nvram_show", ej_nvram_show },
	{ "wanstatus_get", ej_wanstatus_get },	
	{ "localtime", ej_localtime },	
	{ "getclonemac", ej_getclonemac},	
	{ "dumplog", ej_dumplog },	
	{ "dumpleases", ej_dumpleases}, 
	{ "wan_dhcp_get", ej_wan_dhcp_get },	
	{ "lan_dhcp_get", ej_lan_dhcp_get },	
	
	{ "nvram_fw_domain_list", ej_nvram_fw_domain_list },	
	{ "checkbox_checked", ej_checkbox_checked },
	{ "checked_match", ej_checked_match },			
	{ "checked_getblockdays", ej_checked_getblockdays },	
	{ "selected_match", ej_selected_match },
	{ "field_nvram_get", ej_field_nvram_get },
	{ "vs_lan_mask", ej_vs_lan_mask },
	{ "forward_port_list", ej_forward_port_list },	
	{ "filter_client_list", ej_filter_client_list },
	{ "dumpdhcpclease", ej_dumpleases_dhcpc},
	{ "dhcpc_lease_count", ej_dhcpc_lease_count},
	{ "dumplandevices", ej_dumplandevices }, 			
	{ "client_selected_match", ej_client_selected_match },
	{ "client_info", ej_client_info },
	{ "forward_selected_match", ej_forward_selected_match },	
	{ "forward_port_info", ej_forward_port_info },	
	{ "trigger_port_list", ej_trigger_port_list },	
	{ "trigger_port_info", ej_trigger_port_info },	
	
	{ "static_route_list", ej_static_route_list },
	{ "static_route_info", ej_static_route_info },	
	
	{ "dhcpc_res_info", ej_dhcpc_res_info },	

	{ "logout", ej_logout },	
	{ "login_ipaddr", ej_login_ipaddr },
	{ "login_hostname", ej_login_hostname },	
	
	{ "statistics_get", ej_statistics_get },
	
	{ "pppoe_status", ej_pppoe_status },
	{ "pptp_status", ej_pptp_status },
	{ "l2tp_status", ej_l2tp_status },	
		
	
	{ "get_refresh_page", ej_get_refresh_page },	
	
	{ "get_refresh_time", ej_get_refresh_time },		
	
	{ "dhcpc_status", ej_dhcpc_status },	
	
	{ "dhcpc_res_list", ej_dhcpc_res_list },
	
	{ "dhcpc_res_lease_list", ej_dhcpc_res_lease_list },
	
	{ "sys_up_time", ej_sys_up_time },	
	
	{ "ddns_status", ej_ddns_status },	

	{ "POT_get", ej_POT_get },	//ch###
	
	{ "upnp_port_list", ej_upnp_port_list },	//ch###
	
	{ "bpa_login_status", ej_bpa_login_status },
	
	{ "wlan_Client_List", ej_wlan_Client_List },	
        { "wlan_acl_list", ej_wlan_acl_list },
        { "wlan_acl_info", ej_wlan_acl_info },
        
	{ "qos_port_list", ej_qos_port_list },	
	{ "qos_selected_match", ej_qos_selected_match },	
	{ "qos_port_info", ej_qos_port_info },		        
	
	{ "get_udpserver_status", ej_get_udpserver_status },

	{ "sock_local_ipaddr", ej_getsockname },        
        
	{ NULL, NULL }	
};


#if !defined(NVRAM_SHARED_LIB)

extern char *nvram_get_lan_ipaddr(char *name);
extern char *nvram_get_lan_netmask(char *name);

extern char *nvram_get_dhcp_start(char *name);
extern char *nvram_get_dhcp_end(char *name);

extern char *nvram_get_lan_proto(char *name);

extern char *nvram_get_wan_proto(char *name);


int
nvram_set_lan_ipaddr(char *name, char *value)
{
	int ret = 0;

	DPRINTF("nvram_set(\"%s\", \"%s\")\n", name, value);
	
	//(void) inet_aton(value, (struct in_addr *)&config.lan.profile[0].ipAddress);
	return 1;

}

extern int nvram_set_lan_netmask(char *name, char *value);

char *
nvram_get_wan_ifname(char *name)
{
	int ret = 0;

	DPRINTF("nvram_get(\"%s\")\n", name);
	
	return ("eth0");

}

extern int nvram_set_dhcp_start(char *name, char *value);
extern int nvram_set_dhcp_end(char *name, char *value);


#define ONE_DAY_SEC	86400
#define BELKIN_LEASE_UNIT	(30*60)			/* unit 30 minutes */

char *
nvram_get_dhcp_lease(char *name)
{
	char *c;
	int ret = 0;
	int half_unit;

	DPRINTF("nvram_get(\"%s\")\n", name);
	
	return ("30");

}

int
nvram_set_dhcp_lease(char *name, char *value)
{
	int ret = 0;
	
	unsigned int lease_time;
	

	DPRINTF("nvram_set(\"%s\". \"%s\")\n", name, value);

	return 1;

}


extern int nvram_set_wan_proto(char *name, char *value);

extern char *nvram_get_wan_domain(char *name);
extern int nvram_set_wan_domain(char *name, char *value);

int
nvram_get_wan_hostname(char *name)
{
	char *c=NULL;
	int ret = 0;
	
	DPRINTF("nvram_get(\"%s\")\n", name);

	//return ("RP614v4");
	return _PRODUCT_NAME;
	
} 

int
nvram_set_wan_hostname(char *name, char *value)
{
	char *c=NULL;
	int ret = 0;
	
	DPRINTF("nvram_set(\"%s\")\n", name);

	//strncpy(config.isp.profile[0].description, value, DCPT_LEN);
	
	return 1;
	
} 

extern int nvram_set_lan_proto(char *name, char *value);

extern char *nvram_get_wan_ipaddr(char *name);
extern int nvram_set_wan_ipaddr(char *name, char *value);

extern char *nvram_get_wan_netmask(char *name);
extern int nvram_set_wan_netmask(char *name, char *value);

extern char *nvram_get_wan_gateway(char *name);
extern int nvram_set_wan_gateway(char *name, char *value);

extern char *nvram_get_wan0_ipaddr(char *name);
extern char *nvram_get_wan0_netmask(char *name);
extern char *nvram_get_wan0_gateway(char *name);


extern char *nvram_get_wan_pppoe_username(char *name);
extern int nvram_set_wan_pppoe_username(char *name, char *value);

extern char *nvram_get_wan_pppoe_passwd(char *name);
extern int nvram_set_wan_pppoe_passwd(char *name, char *value);


extern char *nvram_get_wan_pppoe_service(char *name);
extern int nvram_set_wan_pppoe_service(char *name, char *value);


extern char *nvram_get_wan_pppoe_demand(char *name);
extern int nvram_set_wan_pppoe_demand(char *name, char *value);


extern char *nvram_get_wan_pppoe_mtu(char *name);
extern int nvram_set_wan_pppoe_mtu(char *name, char *value);


extern char *nvram_get_wan_pppoe_idletime(char *name);
extern int nvram_set_wan_pppoe_idletime(char *name, char *value);


extern char *nvram_get_wan_pptp_username(char *name);
extern int nvram_set_wan_pptp_username(char *name, char *value);

extern char *nvram_get_wan_pptp_passwd(char *name);
extern int nvram_set_wan_pptp_passwd(char *name, char *value);

extern char *nvram_get_wan_pptp_idle_disconnect(char *name);
extern int nvram_set_wan_pptp_idle_disconnect(char *name, char *value);

extern char *nvram_get_wan_pptp_idle_interval(char *name);
extern int nvram_set_wan_pptp_idle_interval(char *name, char *value);

extern char *nvram_get_wan_pptp_id(char *name);
extern int nvram_set_wan_pptp_id(char *name, char *value);


extern char *nvram_get_wan_pptp_ipaddr(char *name);
extern int nvram_set_wan_pptp_ipaddr(char *name, char *value);


extern char *nvram_get_wan_pptp_netmask(char *name);
extern int nvram_set_wan_pptp_netmask(char *name, char *value);


extern char *nvram_get_wan_pptp_server_ipaddr(char *name);
extern int nvram_set_wan_pptp_server_ipaddr(char *name, char *value);


extern char *nvram_get_wan0_hwaddr(char *name);
extern int nvram_set_wan0_hwaddr(char *name, char *value);
extern char *nvram_get_wan_hwaddr(char *name);
extern int nvram_set_wan_hwaddr(char *name, char *value);
extern char *nvram_get_lan_hwaddr(char *name);
extern int nvram_set_lan_hwaddr(char *name, char *value);
extern char *nvram_get_wl_ac_enable(char *name);
extern int  nvram_set_wl_ac_enable(char *name, char *value);


char *
nvram_get_wireless_supported(char *name)
{
	return ("1");
}

extern char *nvram_get_fw_pppoe_type(char *name);

extern char *nvram_get_wan0_dns1(char *name);
extern char *nvram_get_wan0_dns2(char *name);
extern char *nvram_get_wan_dns1(char *name);
extern char *nvram_get_wan_dns2(char *name);
extern char *nvram_get_wan_autodns(char *name);

int nvram_set_wan0_dns1(char* name)
{
}
int nvram_set_wan0_dns2(char* name)
{
}	

extern int nvram_set_wan_dns1(char* name);
extern int nvram_set_wan_dns2(char* name);
extern int nvram_set_wan_autodns(char* name);

extern char *nvram_get_http_remoteenable(char *name);
extern int nvram_set_http_remoteenable(char *name, char *value);

extern char *nvram_get_http_wanport(char *name);
extern int nvram_set_http_wanport(char *name, char *value);

extern char *nvram_get_remote_config_ip(char *name);
extern int nvram_set_remote_config_ip(char *name, char *value);

extern char *nvram_get_remote_config_ip_start(char *name);
extern int nvram_set_remote_config_ip_start(char *name, char *value);

extern char *nvram_get_remote_config_ip_end(char *name);
extern int nvram_set_remote_config_ip_end(char *name, char *value);


extern char *nvram_get_client_filter_ip(char *name);
extern int nvram_set_client_filter_ip(char *name, char *value);

extern char *nvram_get_client_filter_s_ip(char *name);
extern int nvram_set_client_filter_s_ip(char *name, char *value);

extern char *nvram_get_client_filter_e_ip(char *name);
extern int nvram_set_client_filter_e_ip(char *name, char *value);


extern char *nvram_get_wan_ping(char *name);
extern int nvram_set_wan_ping(char *name, char *value);

extern char *nvram_get_wan_mtu(char *name);
extern int nvram_set_wan_mtu(char *name, char *value);


extern char *nvram_get_fw_keywordenable(char *name);
extern int nvram_set_fw_keywordenable(char *name, char *value);

extern char *nvram_get_fw_block_keyword_domain(char *name);
extern int nvram_set_fw_block_keyword_domain(char *name, char *value);
extern int nvram_set_add_block_keyword_domain(char *name, char *value);
extern int nvram_set_del_block_keyword_domain(char *name, char *value);
extern int nvram_set_clr_block_keyword_domain(char *name, char *value);


extern char *nvram_get_fw_packrulenumber(char *name);
extern int *nvram_set_fw_packrulenumber(char *name, char *value);

extern char *nvram_get_fw_packenable(char *name);
extern int *nvram_set_fw_packenable(char *name, char *value);


extern char *nvram_get_vs_count(char *name);

extern char *nvram_get_forward_port(char *name);
extern int nvram_set_forward_port(char *name, char *value);

extern char *nvram_get_filter_client(char *name);
extern int nvram_set_filter_client(char *name, char *value);

extern char *nvram_get_static_route(char *name);
extern int nvram_set_static_route(char *name, char *value);


extern char *nvram_get_fw_inidays(char *name);


extern char *nvram_get_fw_blockdays(char *name);
extern int nvram_set_fw_blockdays(char *name, char *value);

extern char *nvram_get_fw_everyday(char *name);
extern int nvram_set_fw_everyday(char *name, char *value);

extern char *nvram_get_fw_blockallday(char *name);
extern int nvram_set_fw_blockallday(char *name, char *value);


extern char *nvram_get_fw_starthour(char *name);
extern int nvram_set_fw_starthour(char *name, char *value);

extern char *nvram_get_fw_endhour(char *name);
extern int nvram_set_fw_endhour(char *name, char *value);

extern char *nvram_get_fw_startmin(char *name);
extern int nvram_set_fw_startmin(char *name, char *value);

extern char *nvram_get_fw_endmin(char *name);
extern int nvram_set_fw_endmin(char *name, char *value);


char *
nvram_get_show_Product(char *name)
{
	return ("2");
}	

char *
nvram_get_time_zone(char *name)
{
	return ("20");		/* Taipei */
}

int
nvram_set_time_zone(char *name, char *value)
{
	return 1;
}

char *
nvram_get_sys_clock(char *name)
{
	return ("1");
}	

char *
nvram_get_available_localtime(char *name)
{
	return ("1");
}	

char *
nvram_get_blankstate(char *name)
{
	return ("1");		/* 0 -> done */
				/* 1 -> doing */
}

int
nvram_set_blankstate(char *name, char *value)
{
	return 1;
}

extern char *nvram_get_emailenable(char *name);
extern int nvram_set_emailenable(char *name, char *value);
extern char *nvram_get_logsubject(char *name);
extern int nvram_set_logsubject(char *name, char *value);
extern char *nvram_get_alertsubject(char *name);
extern int nvram_set_alertsubject(char *name, char *value);
extern char *nvram_get_mailserver(char *name);
extern int nvram_set_mailserver(char *name, char *value);
extern char *nvram_get_tomail(char *name);
extern int nvram_set_tomail(char *name, char *value);
extern char *nvram_get_frommail(char *name);
extern int nvram_set_frommail(char *name, char *value);
extern char *nvram_get_alertimm(char *name);
extern int nvram_set_alertimm(char *name, char *value);
extern char *nvram_get_alerttype(char *name);
extern int nvram_set_alerttype(char *name, char *value);
extern char *nvram_get_alertday(char *name);
extern int nvram_set_alertday(char *name, char *value);
extern char *nvram_get_alerthour(char *name);
extern int nvram_set_alerthour(char *name, char *value);
extern char *nvram_get_alertampm(char *name);
extern int nvram_set_alertampm(char *name, char *value);
extern char *nvram_get_email_sameaddr(char *name);
extern int nvram_set_email_sameaddr(char *name, char *value);
extern char *nvram_get_email_diffaddr(char *name);
extern int nvram_set_email_diffaddr(char *name, char *value);
extern char *nvram_get_email_serverauth(char *name);
extern int nvram_set_email_serverauth(char *name, char *value);
extern char *nvram_get_email_authname(char *name);
extern int nvram_set_email_authname(char *name, char *value);
extern char *nvram_get_email_authpasswd(char *name);
extern int nvram_set_email_authpasswd(char *name, char *value);


extern char *nvram_get_ddns_enable(char *name);
extern int nvram_set_ddns_enable(char *name, char *value);

extern char *nvram_get_ddns_enable(char *name);
extern int nvram_set_ddns_enable(char *name, char *value);

extern char *nvram_get_ddns_host(char *name);
extern int nvram_set_ddns_host(char *name, char *value);

extern char *nvram_get_ddns_hostname(char *name);
extern int nvram_set_ddns_hostname(char *name, char *value);


extern char *nvram_get_ddns_login(char *name);
extern int nvram_set_ddns_login(char *name, char *value);

extern char *nvram_get_ddns_passwd(char *name);
extern int nvram_set_ddns_passwd(char *name, char *value);

extern char *nvram_get_ddns_wildcard(char *name);
extern int nvram_set_ddns_wildcard(char *name, char *value);


extern char *nvram_get_upnp_enable(char *name);
extern int nvram_set_upnp_enable(char *name, char *value);
extern char *nvram_get_upnp_advertime(char *name);
extern int nvram_set_upnp_advertime(char *name, char *value);
extern char *nvram_get_upnp_timetolive(char *name);
extern int nvram_set_upnp_timetolive(char *name, char *value);


extern char *nvram_get_fw_porttrigenable(char *name);
extern int nvram_set_fw_porttrigenable(char *name, char *value);
extern char *nvram_get_fw_triggertime(char *name);
extern int nvram_set_fw_triggertime(char *name, char *value);
extern char *nvram_get_trigger_count(char *name);
extern int nvram_set_trigger_count(char *name, char *value);

extern char *nvram_get_trigger_port(char *name);
extern int nvram_set_trigger_port(char *name, char *value);

extern int nvram_set_restore_defaults(char *name, char *value);

char *
nvram_get_restore_defaults(char *name)
{
	return ("0");
}	

extern char *nvram_get_http_passwd(char *name);
extern int nvram_set_http_passwd(char *name, char *value);

extern char *nvram_get_fw_disable(char *name);
extern int nvram_set_fw_disable(char *name, char *value);

extern char *nvram_get_dmz_enable(char *name);
extern int nvram_set_dmz_enable(char *name, char *value);

extern char *nvram_get_dmz_ipaddr(char *name);
extern int nvram_set_dmz_ipaddr(char *name, char *value);

extern char *nvram_get_wan_mac_mode(char *name);
extern int nvram_set_wan_mac_mode(char *name, char *value);

extern char *nvram_get_fw_remoteenable(char *name);
extern int nvram_set_fw_remoteenable(char *name, char *value);

extern char *nvram_get_wan_spoofmac(char *name);
extern int nvram_set_wan_spoofmac(char *name, char *value);

extern char *nvram_get_wizard_wan_spoofmac_detected(char *name);
extern int nvram_set_wizard_wan_spoofmac_detected(char *name, char *value);

extern char *nvram_get_dhcpc_reservation(char *name);
extern int nvram_set_dhcpc_reservation(char *name, char *value);

extern char *nvram_get_dhcpc_res_count(char *name);

extern char *nvram_get_wlan_acl(char *name);
extern int nvram_set_wlan_acl(char *name, char *value);

extern char *nvram_get_wlan_count(char *name);
extern int nvram_set_wlan_count(char *name, char *value);


#define OS_VERSION	"0.01.00 NA"
#define HW_VERSION	"V4H1"

char*
nvram_get_os_version(char *name)
{
	return (OS_VERSION);
}

char*
nvram_get_hw_version(char *name)
{
	return (HW_VERSION);
}

//KEVIN:06DEC05:BlankState+++
#define SmtWzd_S_FILE_PATH "/var/SmtWzd.state"

int
nvram_set_Smartwizard_State(char *name, char *value)
{
	FILE *fp=NULL;
	int	retval=-1;
	int 	iVal= atoi(value);
	
	fp=fopen(SmtWzd_S_FILE_PATH,"w+");	

	if(fp) {
		fprintf(fp,"%s",value);
		fclose(fp);
		retval=1;
	}
	
	return retval;
	
}

char g_str[50];
char *
nvram_get_Smartwizard_State(char *name)
{
	FILE *fp=NULL;
	char	retval[10];

	fp=fopen(SmtWzd_S_FILE_PATH,"r");	

	if(!fp) { 
		nvram_set_Smartwizard_State("SmtWzdState","1");
		return ("1");
	}

	fscanf(fp, "%s", g_str);	
	fclose(fp);
	
	return g_str;
}	

//KEVIN:BlankState---


struct ej_nvram_handler nvram_handlers[] = {
	{ "lan_ipaddr", nvram_get_lan_ipaddr, nvram_set_lan_ipaddr },
	{ "lan_netmask", nvram_get_lan_netmask, nvram_set_lan_netmask },
	{ "default_lan_ipaddr", nvram_get_lan_ipaddr, nvram_set_lan_ipaddr },
	{ "default_lan_netmask", nvram_get_lan_netmask, nvram_set_lan_netmask },
	{ "default_lan_ipaddr_br", nvram_get_lan_ipaddr, nvram_set_lan_ipaddr },
	{ "default_lan_netmask_br", nvram_get_lan_netmask, nvram_set_lan_netmask },
	{ "wan_ifname", nvram_get_wan_ifname, NULL },	
	{ "dhcp_start", nvram_get_dhcp_start, nvram_set_dhcp_start },
	{ "dhcp_end", nvram_get_dhcp_end, nvram_set_dhcp_end },
	{ "dhcp_lease", nvram_get_dhcp_lease, nvram_set_dhcp_lease },
	{ "wan_proto", nvram_get_wan_proto, nvram_set_wan_proto },
	{ "wan_hostname", nvram_get_wan_hostname, nvram_set_wan_hostname },
	{ "wan_domain",	nvram_get_wan_domain, nvram_set_wan_domain },
	{ "lan_proto", nvram_get_lan_proto, nvram_set_lan_proto },
	{ "wan_ipaddr", nvram_get_wan_ipaddr, nvram_set_wan_ipaddr },
	{ "wan_netmask", nvram_get_wan_netmask, nvram_set_wan_netmask },
	{ "wan_gateway", nvram_get_wan_gateway, nvram_set_wan_gateway },	
	{ "wan0_ipaddr", nvram_get_wan0_ipaddr, NULL },
	{ "wan0_netmask", nvram_get_wan0_netmask, NULL },
	{ "wan0_gateway", nvram_get_wan0_gateway, NULL },		
	{ "wan_pppoe_username", nvram_get_wan_pppoe_username, nvram_set_wan_pppoe_username },
	{ "wan_pppoe_passwd", nvram_get_wan_pppoe_passwd, nvram_set_wan_pppoe_passwd },
	{ "wan_pppoe_service", nvram_get_wan_pppoe_service, nvram_set_wan_pppoe_service },
	{ "wan_pppoe_mtu", nvram_get_wan_pppoe_mtu, nvram_set_wan_pppoe_mtu },
	{ "wan_pppoe_demand", nvram_get_wan_pppoe_demand, nvram_set_wan_pppoe_demand },
	{ "wan_pppoe_idletime", nvram_get_wan_pppoe_idletime, nvram_set_wan_pppoe_idletime },					
	{ "wan_pptp_username", nvram_get_wan_pptp_username, nvram_set_wan_pptp_username },
	{ "wan_pptp_password", nvram_get_wan_pptp_passwd, nvram_set_wan_pptp_passwd },
	{ "wan_pptp_idle_disconnect", nvram_get_wan_pptp_idle_disconnect, nvram_set_wan_pptp_idle_disconnect },
	{ "wan_pptp_idle_interval", nvram_get_wan_pptp_idle_interval, nvram_set_wan_pptp_idle_interval },
	{ "wan_pptp_conn_id", nvram_get_wan_pptp_id, nvram_set_wan_pptp_id },
	{ "wan_pptp_ipaddr", nvram_get_wan_pptp_ipaddr, nvram_set_wan_pptp_ipaddr },
	{ "wan_pptp_netmask", nvram_get_wan_pptp_netmask, nvram_set_wan_pptp_netmask },
	{ "wan_pptp_server_ipaddr", nvram_get_wan_pptp_server_ipaddr, nvram_set_wan_pptp_server_ipaddr },
	
	{ "lan_hwaddr", nvram_get_lan_hwaddr, nvram_set_lan_hwaddr },
	{ "wan0_hwaddr", nvram_get_wan0_hwaddr, nvram_set_wan0_hwaddr },
	{ "wan_hwaddr", nvram_get_wan_hwaddr, nvram_set_wan_hwaddr },

	{ "wan0_dns1", nvram_get_wan0_dns1, nvram_set_wan0_dns1 },
	{ "wan0_dns2", nvram_get_wan0_dns2, nvram_set_wan0_dns2 },	
	{ "wan_dns1", nvram_get_wan_dns1, nvram_set_wan_dns1 },
	{ "wan_dns2", nvram_get_wan_dns2, nvram_set_wan_dns2 },		
	{ "wan_autodns", nvram_get_wan_autodns, nvram_set_wan_autodns },
	
	{ "http_remoteenable", nvram_get_http_remoteenable, nvram_set_http_remoteenable },
	{ "http_wanport", nvram_get_http_wanport, nvram_set_http_wanport },	
	{ "remote_config_ip", nvram_get_remote_config_ip, nvram_set_remote_config_ip },	
	{ "remote_config_ip_start", nvram_get_remote_config_ip_start, nvram_set_remote_config_ip_start },	
	{ "remote_config_ip_end", nvram_get_remote_config_ip_end, nvram_set_remote_config_ip_end },	
	
	{ "wan_ping", nvram_get_wan_ping, nvram_set_wan_ping },	
	{ "fw_wan_mtu", nvram_get_wan_mtu, nvram_set_wan_mtu },	
	
	{ "vs_count", nvram_get_vs_count, NULL },
	/* d-d>ip:d-d,proto,on/off,desc */	
	{ "forward_port0", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port1", nvram_get_forward_port, nvram_set_forward_port },	
	{ "forward_port2", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port3", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port4", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port5", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port6", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port7", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port8", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port9", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port10", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port11", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port12", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port13", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port14", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port15", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port16", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port17", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port18", nvram_get_forward_port, nvram_set_forward_port },
	{ "forward_port19", nvram_get_forward_port, nvram_set_forward_port },	
	{ "fw_keywordenable", nvram_get_fw_keywordenable, nvram_set_fw_keywordenable },		
	
	{ "add_block_keyword_domain", NULL, nvram_set_add_block_keyword_domain	},
	{ "del_block_keyword_domain", NULL, nvram_set_del_block_keyword_domain	},
	{ "clr_block_keyword_domain", NULL, nvram_set_clr_block_keyword_domain	},	
		
	{ "fw_block_keyword_domain0", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain1", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain2", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain3", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain4", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain5", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain6", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	{ "fw_block_keyword_domain7", nvram_get_fw_block_keyword_domain, nvram_set_fw_block_keyword_domain },	
	

	{ "fw_packenable", nvram_get_fw_packenable, nvram_set_fw_packenable },
	{ "fw_packrulenumber", nvram_get_fw_packrulenumber, nvram_set_fw_packrulenumber },
	
	/* %s-%s:%d-%d,%s,%d,%d,%d,%d,%d,%d,%d:%d-%d,%s,%s */	
	{ "filter_client0", nvram_get_filter_client, nvram_set_filter_client },	
	{ "filter_client1", nvram_get_filter_client, nvram_set_filter_client },	
	{ "filter_client2", nvram_get_filter_client, nvram_set_filter_client },	
	{ "filter_client3", nvram_get_filter_client, nvram_set_filter_client },	
	{ "filter_client4", nvram_get_filter_client, nvram_set_filter_client },	
	{ "filter_client5", nvram_get_filter_client, nvram_set_filter_client },						
	{ "filter_client6", nvram_get_filter_client, nvram_set_filter_client },						
	{ "filter_client7", nvram_get_filter_client, nvram_set_filter_client },						
	{ "filter_client8", nvram_get_filter_client, nvram_set_filter_client },						

	{ "client_filter_ip", nvram_get_client_filter_ip, nvram_set_client_filter_ip },	
	{ "client_filter_s_ip", nvram_get_client_filter_s_ip, nvram_set_client_filter_s_ip },	
	{ "client_filter_e_ip", nvram_get_client_filter_e_ip, nvram_set_client_filter_e_ip },	


	{ "fw_inidays", nvram_get_fw_inidays, NULL },	
	{ "fw_everyday", nvram_get_fw_everyday, nvram_set_fw_everyday },	
	{ "fw_blockdays", nvram_get_fw_blockdays, nvram_set_fw_blockdays },
	{ "fw_blockallday", nvram_get_fw_blockallday, nvram_set_fw_blockallday },
	{ "fw_starthour", nvram_get_fw_starthour, nvram_set_fw_starthour },						
	{ "fw_endhour", nvram_get_fw_endhour, nvram_set_fw_endhour },						
	{ "fw_startmin", nvram_get_fw_startmin, nvram_set_fw_startmin },						
	{ "fw_endmin", nvram_get_fw_endmin, nvram_set_fw_endmin },	
	
	{ "time_zone", nvram_get_time_zone, nvram_set_time_zone },
	{ "SysClock", nvram_get_sys_clock, NULL },
	{ "available_localtime", nvram_get_available_localtime, NULL },	
	{ "blankstate", nvram_get_blankstate, nvram_set_blankstate },	


	{ "emailenable", nvram_get_emailenable, nvram_set_emailenable },
	{ "logsubject", nvram_get_logsubject, nvram_set_logsubject },
	{ "alertsubject", nvram_get_alertsubject, nvram_set_alertsubject },
	{ "mailserver", nvram_get_mailserver, nvram_set_mailserver },	
	{ "tomail", nvram_get_tomail, nvram_set_tomail },
	{ "alertimm", nvram_get_alertimm, nvram_set_alertimm },	
	{ "alerttype", nvram_get_alerttype, nvram_set_alerttype },
	{ "alertday", nvram_get_alertday, nvram_set_alertday },	
	{ "alerthour", nvram_get_alerthour, nvram_set_alerthour },
	{ "alertampm", nvram_get_alertampm, nvram_set_alertampm },	
	
	
	{ "frommail", nvram_get_frommail, nvram_set_frommail },
	{ "email_authname", nvram_get_email_authname, nvram_set_email_authname },	
	{ "email_authpasswd", nvram_get_email_authpasswd, nvram_set_email_authpasswd },
	{ "email_serverauth", nvram_get_email_serverauth, nvram_set_email_serverauth },	
	{ "email_sameaddr", nvram_get_email_sameaddr, nvram_set_email_sameaddr },
	{ "email_diffaddr", nvram_get_email_diffaddr, nvram_set_email_diffaddr },								
							

	{ "ddns_enable", nvram_get_ddns_enable, nvram_set_ddns_enable },	
	{ "ddns_hostname", nvram_get_ddns_hostname, nvram_set_ddns_hostname },		
	{ "ddns_login", nvram_get_ddns_login, nvram_set_ddns_login },	
	{ "ddns_passwd", nvram_get_ddns_passwd, nvram_set_ddns_passwd },	
	{ "ddns_wildcard", nvram_get_ddns_wildcard, nvram_set_ddns_wildcard },	
	
	{ "upnp_enable", nvram_get_upnp_enable, nvram_set_upnp_enable },	
	{ "upnp_advertime", nvram_get_upnp_advertime, nvram_set_upnp_advertime },	
	{ "upnp_timetolive", nvram_get_upnp_timetolive, nvram_set_upnp_timetolive },	
	
	{ "fw_porttrigenable", nvram_get_fw_porttrigenable, nvram_set_fw_porttrigenable },	
	{ "fw_triggertime", nvram_get_fw_triggertime, nvram_set_fw_triggertime },	
	{ "trigger_count", nvram_get_trigger_count, NULL },		

	/* d-d>ip:d-d,proto,on/off,desc,used */
	{ "trigger_port0", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port1", nvram_get_trigger_port, nvram_set_trigger_port },	
	{ "trigger_port2", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port3", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port4", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port5", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port6", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port7", nvram_get_trigger_port, nvram_set_trigger_port },
	{ "trigger_port8", nvram_get_trigger_port, nvram_set_trigger_port },

	{ "restore_defaults", nvram_get_restore_defaults, nvram_set_restore_defaults },	
	
	{ "static_route0", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route1", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route2", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route3", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route4", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route5", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route6", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route7", nvram_get_static_route, nvram_set_static_route },	
	{ "static_route8", nvram_get_static_route, nvram_set_static_route },	

	{ "http_passwd", nvram_get_http_passwd, nvram_set_http_passwd },	

	{ "fw_disable", nvram_get_fw_disable, nvram_set_fw_disable },

	{ "dmz_enable", nvram_get_dmz_enable, nvram_set_dmz_enable },
	{ "dmz_ipaddr", nvram_get_dmz_ipaddr, nvram_set_dmz_ipaddr },
	

	{ "dhcpc_res_count", nvram_get_dhcpc_res_count, NULL },		
	
	/* ipaddr,mac,desc */
	{ "dhcpc_res0", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res1", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	//ATTACHED_DEVICES 
	{ "dhcpc_res2", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res3", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res4", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res5", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res6", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res7", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res8", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res9", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res10", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res11", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res12", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res13", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res14", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res15", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res16", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res17", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res18", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res19", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res20", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res21", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res22", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res23", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res24", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res25", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res26", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res27", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	{ "dhcpc_res28", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },	
	{ "dhcpc_res29", nvram_get_dhcpc_reservation, nvram_set_dhcpc_reservation },
	//ATTACHED_DEVICES 

	{ "os_version", nvram_get_os_version, NULL},
	
	{ "hw_version", nvram_get_hw_version, NULL},	
	

#if 0	
	{ "restore_defaults", NULL, nvram_set_restore_defaults },
	{ "os_version", nvram_get_os_version, NULL},
	{ "pmon_ver",  nvram_get_boot_version, NULL},
	{ "product_id",  nvram_get_product_id, NULL},
	{ "serial_no",  nvram_get_serial_no, nvram_set_serial_no/*NULL*/},
	//mac filter
	{ "filter_maclist", nvram_get_macfl_list, nvram_set_macfl_list },
	{ "filter_macmode", nvram_get_filter_macmode, nvram_set_filter_macmode },	
	{ "ntp_dst_enabled", nvram_get_ntp_dst, nvram_set_ntp_dst },	
	{ "http_login_timeout", nvram_get_http_login_timeout, nvram_set_http_login_timeout },

	{ "router_disable", nvram_get_router_disable, nvram_set_router_disable },	

	{ "fw_auto_detect", nvram_get_fw_auto_detect, nvram_set_fw_auto_detect },
	{ "new_fw_avail", nvram_get_new_fw_avail, nvram_set_new_fw_avail },
	
	{ "ntp_server", nvram_get_ntp_server, NULL },
	
	{ "heartbeat_state", nvram_get_heartbeat_state, nvram_set_heartbeat_state },
	{ "heartbeat_is_manual_server", nvram_get_heartbeat_is_manual_server, nvram_set_heartbeat_is_manual_server },
	{ "heartbeat_server", nvram_get_heartbeat_server, nvram_set_heartbeat_server },
	{ "heartbeat_username", nvram_get_heartbeat_username, nvram_set_heartbeat_username },
	{ "heartbeat_password", nvram_get_heartbeat_password, nvram_set_heartbeat_password },
	
	{ "is_wifi", nvram_get_is_wifi, nvram_set_is_wifi },	

	{ "rip_dir", nvram_get_rip_dir, nvram_set_rip_dir },	
	{ "rip_ver", nvram_get_rip_ver, nvram_set_rip_ver },			
	{ "nat_disable", nvram_get_nat_disable, nvram_set_nat_disable },	
#endif	

	{ "wireless", nvram_get_wireless_supported, NULL },
	{ "fw_pppoetype", nvram_get_fw_pppoe_type, NULL },		
	{ "show_Product", nvram_get_show_Product, NULL },	
	
	{ "wan_mac_mode", nvram_get_wan_mac_mode, nvram_set_wan_mac_mode },
	
	{ "wan_spoofmac", nvram_get_wan_spoofmac, nvram_set_wan_spoofmac },	
	
	{ "wizard_wan_spoofmac_detected", nvram_get_wizard_wan_spoofmac_detected, nvram_set_wizard_wan_spoofmac_detected },	

	{ "fw_remoteenable", nvram_get_fw_remoteenable, nvram_set_fw_remoteenable },		
	
	{ "SmtWzdState", nvram_get_Smartwizard_State, nvram_set_Smartwizard_State },	//KEVIN:ADD:06DEC05:BlankState
	
	{ "wlan_acl0", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl1", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl2", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl3", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl4", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl5", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl6", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl7", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl8", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl9", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl10", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl11", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl12", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl13", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl14", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl15", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl16", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl17", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl18", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl19", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl20", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl21", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl22", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl23", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl24", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl25", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl26", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl27", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl28", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl29", nvram_get_wlan_acl, nvram_set_wlan_acl },	
	{ "wlan_acl30", nvram_get_wlan_acl, nvram_set_wlan_acl },
	{ "wlan_acl31", nvram_get_wlan_acl, nvram_set_wlan_acl },

	{ "wlan_acl_count", nvram_get_wlan_count, nvram_set_wlan_count },	
	
	{ NULL, NULL, NULL },	
	
};

#endif /* #if !defined(NVRAM_SHARED_LIB) */

